import org.gradle.api.tasks.Internal
import org.gradle.api.tasks.TaskAction
import java.io.File

private const val DIR_SOXR = "soxr"
private const val VERSION_SOXR = "0.1.3"

abstract class BuildDesktopNativeTask : BuildNativeLibrariesTask() {

    @get:Internal
    lateinit var target: String

    @TaskAction
    fun buildLibraries() {
        val params = NativeBuildParameters(
            target = target.takeIf { it.isNotEmpty() },
            vpxTarget = when (target) {
                "x86_64-w64-mingw32" -> "x86_64-win64-gcc"
                else -> null
            },
            sysrootDir = mingwSysRootPath.takeIf { it.isNotEmpty() }?.let { File(it) },
            prefixDir = File(buildDir, "_${getDesktopOs()}_prefix_${getDesktopArch()}"),
            forceFpic = true,
        )
        val pkgConfig = File(params.prefixDir, "lib/pkgconfig").absolutePath
        environment("PKG_CONFIG_LIBDIR", pkgConfig)
        environment("PKG_CONFIG_PATH", pkgConfig)
        openBuildDirectory()
        buildNativeLibraries(params)
        buildSoxrLibrary(params)
    }

    private fun buildSoxrLibrary(params: NativeBuildParameters) {
        if (prefixOutputExists(params, "soxr") && prefixOutputExists(params, "soxr-lsr")) return
        val dir = initBuildDirectory(DIR_SOXR) {
            gitClone(
                repo = "https://github.com/chirlu/soxr",
                depth = 1,
                branch = VERSION_SOXR,
                dirName = DIR_SOXR,
            )
        }
        File(dir, "_build").useDirectory {
            cmake(
                relative = "..",
                args = mapOf(
                    "BUILD_SHARED_LIBS" to "OFF",
                    "BUILD_TESTS" to "OFF",
                    "WITH_LSR_BINDINGS" to "ON",
                    "WITH_CR32S" to "OFF",
                    "WITH_CR64S" to "OFF",
                    "CMAKE_POSITION_INDEPENDENT_CODE" to "ON",
                    "CMAKE_INSTALL_PREFIX" to params.prefixDir.absolutePath,
                    "CMAKE_PREFIX_PATH" to params.prefixDir.absolutePath,
                )
            )
            if (File(it, "Makefile").exists()) {
                makeInstall()
            } else {
                File(it, "src").useDirectory {
                    val libraryDir = File(params.prefixDir, "lib")
                    copyFile("libsoxr.a", libraryDir)
                    copyFile("libsoxr-lsr.a", libraryDir)
                }
                File(dir, "src").useDirectory {
                    val includeDir = File(params.prefixDir, "include")
                    copyFile("soxr.h", includeDir)
                    copyFile("soxr-lsr.h", includeDir)
                }
            }
        }
    }
}