#include <algorithm>
#include "SampleBuffer.h"
#include "logging_macros.h"

SampleBuffer::SampleBuffer(int capacity) : capacity(capacity) {
    bufferSize = 0;
    buffer = new short[capacity];
}

SampleBuffer::~SampleBuffer() {
    delete[] buffer;
}

int SampleBuffer::writeSamples(const short *input, int numFrames, long streamId) {
    int s = sizeof(short);
    int maxPos = std::max(std::min(bufferSize, capacity - numFrames), 0);
    int pos = positions[streamId];
    int inputPos = 0;
    int amount = std::min(numFrames, capacity);
    //LOGD("%d pos = %d, bufferSize = %d", streamId, pos, bufferSize);
    while (pos < bufferSize && amount > 0) {
        buffer[pos++] += input[inputPos++];
        amount--;
    }
    if (amount > 0) {
        int shift = std::max((bufferSize + amount) - capacity, 0);
        if (shift > 0) {
            memmove(buffer, buffer + shift, (capacity - shift) * s);
            //LOGW("writeSamples shift = %d", shift);
        }
        memcpy(buffer + pos, input + inputPos, amount * s);
        bufferSize = std::min(bufferSize + amount, capacity);
    }
    positions[streamId] = std::min(pos + amount, capacity);
    return capacity - positions[streamId];
}

int SampleBuffer::readSamples(short *output, int numFrames) {
    int s = sizeof(short);
    //LOGD("readSamples(numFrames = %d)", numFrames, bufferSize);
    if (bufferSize >= numFrames) {
        memcpy(output, buffer, numFrames * s);
        memmove(buffer, buffer + numFrames, (capacity - numFrames) * s);
        bufferSize -= numFrames;
        for (auto i : positions) {
            positions[i.first] = std::max(positions[i.first] - numFrames, 0);
        }
        //LOGD("readSamples(numFrames = %d, bufferSize = %d)", numFrames, bufferSize);
        return numFrames;
    } else {
        memset(output, 0, numFrames * s);
        return 0;
    }
}

short *SampleBuffer::getSamples() {
    return buffer;
}

int SampleBuffer::getCapacity() const {
    return capacity;
}

int SampleBuffer::getBufferSize() {
    return bufferSize;
}

int SampleBuffer::getStreamPosition(long streamId) {
    return positions[streamId];
}

bool SampleBuffer::isFilled() const {
    return bufferSize == capacity;
}