#include "audiojni.h"
#include "AudioEngine.h"

jlong Java_nand_apps_chat_audio_AndroidAudioNative_initEngine(JNIEnv *, jclass, jboolean isPlaybackEngine) {
    auto *engine = new(std::nothrow) AudioEngine(isPlaybackEngine ? oboe::Direction::Output : oboe::Direction::Input);
    return engine != NULL ? reinterpret_cast<jlong>(engine) : NULL;
}

jint Java_nand_apps_chat_audio_AndroidAudioNative_startEngine(JNIEnv *, jclass, jlong ptr, jint deviceId, jint sampleRate, jint channelCount, jboolean isLowLatency) {
    auto *engine = reinterpret_cast<AudioEngine*>(ptr);
    return static_cast<jint>(engine->start(deviceId, sampleRate, channelCount, isLowLatency));
}

jint Java_nand_apps_chat_audio_AndroidAudioNative_writeSamples(JNIEnv *env, jclass, jlong ptr, jshortArray samples, jlong streamId) {
    auto *engine = reinterpret_cast<AudioEngine*>(ptr);
    jshort *samplesPtr = env->GetShortArrayElements(samples, NULL);
    int numSamples = env->GetArrayLength(samples);
    int vacancy = engine->writeSamples((const short *)samplesPtr, numSamples, streamId);
    env->ReleaseShortArrayElements(samples, samplesPtr, 0);
    return vacancy;
}

jint Java_nand_apps_chat_audio_AndroidAudioNative_readSamples(JNIEnv *env, jclass, jlong ptr, jshortArray samples) {
    auto *engine = reinterpret_cast<AudioEngine*>(ptr);
    jshort *samplesPtr = env->GetShortArrayElements(samples, NULL);
    int numSamples = env->GetArrayLength(samples);
    int read = engine->readSamples((short *)samplesPtr, numSamples);
    env->ReleaseShortArrayElements(samples, samplesPtr, 0);
    return read;
}

jint Java_nand_apps_chat_audio_AndroidAudioNative_getBufferSize(JNIEnv *, jclass, jlong ptr) {
    auto *engine = reinterpret_cast<AudioEngine*>(ptr);
    return engine->getBufferSize();
}

jint Java_nand_apps_chat_audio_AndroidAudioNative_getStreamPosition(JNIEnv *, jclass, jlong ptr, jlong streamId) {
    auto *engine = reinterpret_cast<AudioEngine*>(ptr);
    return engine->getStreamPosition(streamId);
}

jint Java_nand_apps_chat_audio_AndroidAudioNative_getVacancy(JNIEnv *, jclass, jlong ptr, jlong streamId) {
    auto *engine = reinterpret_cast<AudioEngine*>(ptr);
    return engine->getVacancy(streamId);
}

jint Java_nand_apps_chat_audio_AndroidAudioNative_stopEngine(JNIEnv *, jclass, jlong ptr) {
    auto *engine = reinterpret_cast<AudioEngine*>(ptr);
    return engine->stop();
}

void Java_nand_apps_chat_audio_AndroidAudioNative_destroyEngine(JNIEnv *, jclass, jlong ptr) {
    auto *engine = reinterpret_cast<AudioEngine*>(ptr);
    engine->stop();
    delete engine;
}