#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#include <gif_lib.h>
#include "gifjni.h"

#define BYTES_PER_PIXEL 4

#define GET_BYTES(src) env->GetByteArrayElements(src, NULL)
#define BYTES_LENGTH(src) env->GetArrayLength(src)
#define RELEASE_BYTES(src, ptr) env->ReleaseByteArrayElements(src, (jbyte*)ptr, 0)

typedef struct ReadBuffer {
	int length;
	int position;
	GifByteType *buffer;
} ReadBuffer;

int read_bytes(GifFileType *gif, GifByteType *buf, int toRead) {
	ReadBuffer *rb = (ReadBuffer*)gif->UserData;
	int remaining = rb->length - rb->position;
	int numRead = remaining >= toRead ? toRead : remaining;
	memcpy(buf, rb->buffer + rb->position, numRead);
	rb->position += numRead;
	return numRead;
}

jobject Java_nand_apps_gif_GifNative_openGif(JNIEnv *env, jclass, jbyteArray data) {

    ReadBuffer rb;
    rb.position = 0;
    rb.length = BYTES_LENGTH(data);
    rb.buffer = (GifByteType*) GET_BYTES(data);
    
    int error;
    GifFileType* gif = DGifOpen(&rb, read_bytes, &error);
    if (gif && DGifSlurp(gif) != GIF_ERROR) {
        RELEASE_BYTES(data, rb.buffer);
        jclass cls = env->FindClass("nand/apps/gif/GifImage");
        jmethodID constructor = env->GetMethodID(cls, "<init>", "(JIIII)V");
        return env->NewObject(cls, constructor, gif, gif->SWidth, gif->SHeight, BYTES_PER_PIXEL, gif->ImageCount);
    } else {
        if (gif) DGifCloseFile(gif, &error);
        RELEASE_BYTES(data, rb.buffer);
        return NULL;
    }
}

jlong Java_nand_apps_gif_GifNative_getFrame(JNIEnv *env, jclass, jlong ptr, jint frameNum, jbyteArray canvas) {
    GifFileType *gif = (GifFileType*)ptr;
    int width = gif->SWidth;
    int height = gif->SHeight;
    int delay;
    
    int canvasSize = width * height * BYTES_PER_PIXEL;
    if (canvasSize != BYTES_LENGTH(canvas)) {
        return -1;
    }
    
    jbyte *canvasPtr = GET_BYTES(canvas);
    
    ColorMapObject* commonMap = gif->SColorMap;
    const SavedImage saved = gif->SavedImages[frameNum];
    const GifImageDesc desc = saved.ImageDesc;
    const ColorMapObject* colorMap = desc.ColorMap ? desc.ColorMap : commonMap;
    GraphicsControlBlock gcb;
    DGifSavedExtensionToGCB(gif, frameNum, &gcb);
    
    if (gcb.DisposalMode != DISPOSE_DO_NOT) {
        memset(canvasPtr, 0, canvasSize);
    }

    for (int y = 0; y < desc.Height; ++y) {
        for (int x = 0; x < desc.Width; ++x) {
            int c = saved.RasterBits[y * desc.Width + x];
            if (colorMap && c != gcb.TransparentColor) {
                GifColorType rgb = colorMap->Colors[c];
                int cIndex = ((y + desc.Top) * width + x + desc.Left) * BYTES_PER_PIXEL;
                if (cIndex >= 0 && cIndex <= canvasSize - 4) {
                    canvasPtr[cIndex] = rgb.Red;
                    canvasPtr[cIndex + 1] = rgb.Green;
                    canvasPtr[cIndex + 2] = rgb.Blue;
                    canvasPtr[cIndex + 3] = 0xFF;
                }
            }
        }
    }
    
    RELEASE_BYTES(canvas, canvasPtr);
    
    return gcb.DelayTime > 1 ? gcb.DelayTime * 10 : 100;
}
  
void Java_nand_apps_gif_GifNative_closeGif(JNIEnv *, jclass, jlong ptr) {
    int error;
    GifFileType *gif = (GifFileType*)ptr;
    DGifCloseFile(gif, &error);
}
