#include <tox/tox.h>
#include <tox/toxav.h>

#include "av.h"

jlong Java_nand_apps_tox_ToxNative_initAv(JNIEnv *, jclass, jlong toxPtr) {
    Tox *tox = (Tox*) toxPtr;
    Toxav_Err_New error;
    ToxAV *av = toxav_new(tox, &error);
    return (jlong) (error == TOXAV_ERR_NEW_OK ? av : NULL);
}

void Java_nand_apps_tox_ToxNative_destroyAv(JNIEnv *, jclass, jlong avPtr) {
    toxav_kill((ToxAV*) avPtr);
}

jint Java_nand_apps_tox_ToxNative_iterateAudio(JNIEnv *, jclass, jlong avPtr) {
    ToxAV *av = (ToxAV*) avPtr;
    toxav_audio_iterate(av);
    return toxav_audio_iteration_interval(av);
}

jboolean Java_nand_apps_tox_ToxNative_startCall(JNIEnv *, jclass, jlong avPtr, jint friendNum, jint audioBitRate, jint videoBitRate) {
    return toxav_call((ToxAV*) avPtr, friendNum, audioBitRate, videoBitRate, NULL);
}

jboolean Java_nand_apps_tox_ToxNative_answerCall(JNIEnv *, jclass, jlong avPtr, jint friendNum, jint audioBitRate, jint videoBitRate) {
    return toxav_answer((ToxAV*) avPtr, friendNum, audioBitRate, videoBitRate, NULL);
}

jboolean Java_nand_apps_tox_ToxNative_sendCallControl(JNIEnv *, jclass, jlong avPtr, jint friendNum, jint control) {
    return toxav_call_control((ToxAV*) avPtr, friendNum, (Toxav_Call_Control) control, NULL);
}

jboolean Java_nand_apps_tox_ToxNative_sendAudioFrame(JNIEnv *env, jclass, jlong avPtr, jint friendNum, jshortArray pcm, jint channels, jint sampleRate) {
    jshort *pcmPtr = env->GetShortArrayElements(pcm, NULL);
    int pcmLength = env->GetArrayLength(pcm) / channels;
    jboolean success = toxav_audio_send_frame((ToxAV*) avPtr, friendNum, (const int16_t*) pcmPtr, pcmLength, channels, sampleRate, NULL);
    env->ReleaseShortArrayElements(pcm, pcmPtr, 0);
    return success;
}

jboolean Java_nand_apps_tox_ToxNative_sendAudioBitRate(JNIEnv *, jclass, jlong avPtr, jint friendNum, jint bitRate) {
    return toxav_audio_set_bit_rate((ToxAV*) avPtr, friendNum, bitRate, NULL);
}