#include <tox/tox.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>

#include "bootstrap.h"
#include "util.h"
#include "logger.h"

typedef struct BootstrapArgs {
    Tox *tox;
    char host[TOX_MAX_HOSTNAME_LENGTH];
    uint16_t udpPort;
    uint16_t tcpPort;
    uint8_t publicKey[TOX_PUBLIC_KEY_SIZE];
} BootstrapArgs;

void *bootstrap(void *arg) {
    JNIEnv* env;
#ifdef JNI_ANDROID
    getJvm()->AttachCurrentThread(&env, NULL);
#else
    getJvm()->AttachCurrentThread((void**)&env, NULL);
#endif

    BootstrapArgs *args = (BootstrapArgs*) arg;
    Tox_Err_Bootstrap error;
    
    if (args->udpPort > 0) {
        tox_bootstrap(args->tox, args->host, args->udpPort, args->publicKey, &error);
        if (error == TOX_ERR_BOOTSTRAP_OK) {
            LOGGER_WRITE(TOX_LOG_LEVEL_DEBUG, "Finished bootstrapping UDP node %s", args->host);
        } else {
            LOGGER_WRITE(TOX_LOG_LEVEL_WARNING, "Failed to bootstrap UDP node %s", args->host);
        }
    }
    if (args->tcpPort > 0) {
        tox_add_tcp_relay(args->tox, args->host, args->tcpPort, args->publicKey, &error);
        if (error == TOX_ERR_BOOTSTRAP_OK) {
            LOGGER_WRITE(TOX_LOG_LEVEL_DEBUG, "Finished bootstrapping TCP node %s", args->host);
        } else {
            LOGGER_WRITE(TOX_LOG_LEVEL_WARNING, "Failed to bootstrap TCP node %s", args->host);
        }
    }
    
    free(args);
    getJvm()->DetachCurrentThread();
    
    return NULL;
}

jboolean Java_nand_apps_tox_ToxNative_bootstrap(JNIEnv *env, jclass, jlong toxPtr, jstring host, jint udpPort, jint tcpPort, jbyteArray publicKey) {
    Tox *tox = (Tox *) toxPtr;
    
    const char *hostPtr = env->GetStringUTFChars(host, NULL);
    jbyte *pkPtr = env->GetByteArrayElements(publicKey, NULL);
    
    bool success = false;
    
    if (udpPort > 0 || tcpPort > 0) {
        BootstrapArgs *args = (BootstrapArgs*) malloc(sizeof(BootstrapArgs));
        memcpy(args->host, hostPtr, strlen(hostPtr) + 1);
        memcpy(args->publicKey, pkPtr, TOX_PUBLIC_KEY_SIZE);
        args->tox = tox;
        args->udpPort = udpPort;
        args->tcpPort = tcpPort;
        pthread_t bootstrapThread;
        success = pthread_create(&bootstrapThread, NULL, bootstrap, args) == 0;
    }
    
    env->ReleaseStringUTFChars(host, hostPtr);
    env->ReleaseByteArrayElements(publicKey, pkPtr, 0);
    
    return success;
}