#include <jni.h>
#include <tox/tox.h>
#include <tox/toxav.h>
#include <string.h>

#include "util.h"

#ifndef jtox_callbacks
#define jtox_callbacks

/** FRIEND CALLBACKS **/

void self_connection_status_cb(Tox *tox, TOX_CONNECTION connection_status, void *user_data);

void friend_request_cb(Tox *tox, const uint8_t *public_key, const uint8_t *message, size_t length, void *user_data);

void friend_message_cb(Tox *tox, Tox_Friend_Number friend_num, TOX_MESSAGE_TYPE type, const uint8_t *message, size_t length, void *user_data);

void friend_name_cb(Tox *tox, Tox_Friend_Number friend_num, const uint8_t *name, size_t length, void *user_data);

void friend_status_cb(Tox *tox, Tox_Friend_Number friend_num, Tox_User_Status status, void *user_data);

void friend_status_message_cb(Tox *tox, Tox_Friend_Number friend_num, const uint8_t *message, size_t length, void *user_data);

void friend_connection_status_cb(Tox *tox, Tox_Friend_Number friend_num, TOX_CONNECTION connection_status, void *user_data);

void friend_read_receipt_cb(Tox *tox, Tox_Friend_Number friend_number, Tox_Friend_Message_Id message_id, void *user_data);

void friend_packet_cb(Tox *tox, Tox_Friend_Number friend_num, const uint8_t data[], size_t length, void *user_data);

/** CONFERENCE CALLBACKS **/

void conference_invite_cb(Tox *tox, Tox_Friend_Number friend_num, TOX_CONFERENCE_TYPE type, const uint8_t *cookie, size_t length, void *user_data);

void conference_connected_cb(Tox *tox, Tox_Conference_Number conference_number, void *user_data);

void conference_title_cb(Tox *tox, uint32_t group_num, uint32_t peer_num, const uint8_t *title, size_t length, void *user_data);

void conference_message_cb(Tox *tox, uint32_t group_num, uint32_t peer_num, TOX_MESSAGE_TYPE type, const uint8_t *message, size_t length, void *user_data);

void conference_audio_cb(void *tox, uint32_t group_num, uint32_t peer_num, const int16_t pcm[], uint32_t samples, uint8_t channels, uint32_t sample_rate, void *user_data);

void conference_peer_list_changed_cb(Tox *tox, uint32_t group_num, void *user_data);

void conference_peer_name_cb(Tox *tox, uint32_t group_num, uint32_t peer_num, const uint8_t *name, size_t length, void *user_data);

/** GROUP CALLBACKS **/

void group_peer_name_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, const uint8_t name[], size_t name_length, void *user_data);

void group_peer_status_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, Tox_User_Status status, void *user_data);

void group_topic_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, const uint8_t topic[], size_t topic_length, void *user_data);

void group_privacy_state_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Privacy_State privacy_state, void *user_data);

void group_voice_state_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Voice_State voice_state, void *user_data);

void group_topic_lock_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Topic_Lock topic_lock, void *user_data);

void group_peer_limit_cb(Tox *tox, Tox_Group_Number group_number, uint32_t peer_limit, void *user_data);

void group_password_cb(Tox *tox, Tox_Group_Number group_number, const uint8_t password[], size_t password_length, void *user_data);

void group_message_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, Tox_Message_Type message_type, const uint8_t message[], size_t message_length, Tox_Group_Message_Id message_id, void *user_data);

void group_private_message_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, Tox_Message_Type message_type, const uint8_t message[], size_t message_length, Tox_Group_Message_Id message_id, void *user_data);

void group_custom_packet_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, const uint8_t data[], size_t data_length, void *user_data);

void group_custom_private_packet_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, const uint8_t data[], size_t data_length, void *user_data);

void group_invite_cb(Tox *tox, Tox_Friend_Number friend_number, const uint8_t invite_data[], size_t invite_data_length, const uint8_t group_name[], size_t group_name_length, void *user_data);

void group_peer_join_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, void *user_data);

void group_peer_exit_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number peer_id, Tox_Group_Exit_Type exit_type, const uint8_t name[], size_t name_length, const uint8_t part_message[], size_t part_message_length, void *user_data);

void group_self_join_cb(Tox *tox, Tox_Group_Number group_number, void *user_data);

void group_join_fail_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Join_Fail fail_type, void *user_data);

void group_moderation_cb(Tox *tox, Tox_Group_Number group_number, Tox_Group_Peer_Number source_peer_id, Tox_Group_Peer_Number target_peer_id, Tox_Group_Mod_Event mod_type, void *user_data);

/** FILE CALLBACKS **/

void file_recv_control_cb(Tox *tox, Tox_Friend_Number friend_num, Tox_File_Number file_num, Tox_File_Control control, void *user_data);

void file_recv_cb(Tox *tox, Tox_Friend_Number friend_num, Tox_File_Number file_num, uint32_t kind, uint64_t file_size, const uint8_t filename[], size_t filename_length, void *user_data);

void file_recv_chunk_cb(Tox *tox, Tox_Friend_Number friend_num, Tox_File_Number file_num, uint64_t position, const uint8_t data[], size_t length, void *user_data);

void file_chunk_request_cb(Tox *tox, Tox_Friend_Number friend_num, Tox_File_Number file_num, uint64_t position, size_t length, void *user_data);

/** AV CALLBACKS */

void av_call_cb(ToxAV *av, uint32_t friend_number, bool audio_enabled, bool video_enabled, void *user_data);

void av_call_state_cb(ToxAV *av, uint32_t friend_number, uint32_t state, void *user_data);

void av_audio_rate_cb(ToxAV *av, uint32_t friend_number, uint32_t audio_bit_rate, void *user_data);

void av_video_rate_cb(ToxAV *av, uint32_t friend_number, uint32_t video_bit_rate, void *user_data);

void av_audio_receive_frame_cb(ToxAV *av, uint32_t friend_number, const int16_t pcm[], size_t sample_count, uint8_t channels, uint32_t sampling_rate, void *user_data);

void av_video_receive_frame_cb(ToxAV *av, uint32_t friend_number, uint16_t width, uint16_t height, const uint8_t y[], const uint8_t u[], const uint8_t v[], int32_t ystride, int32_t ustride, int32_t vstride, void *user_data);

#endif