#include <tox/tox.h>

#include "filetransfers.h"

jint Java_nand_apps_tox_ToxNative_sendFile(JNIEnv *env, jclass, jlong toxPtr, jint friendNum, jint kind, jlong fileSize, jbyteArray fileName, jbyteArray fileId) {
    Tox *tox = (Tox *) toxPtr;
    
    jbyte *fileIdPtr = fileId != NULL ? env->GetByteArrayElements(fileId, NULL) : NULL;
    jbyte *fileNamePtr = env->GetByteArrayElements(fileName, NULL);
    int fileNameLength = env->GetArrayLength(fileName);
    
    Tox_Err_File_Send error;
    int fileNumber = tox_file_send(tox, friendNum, kind, fileSize, (uint8_t*) fileIdPtr, (uint8_t*) fileNamePtr, fileNameLength, &error);
    
    if (fileId != NULL) {
        env->ReleaseByteArrayElements(fileId, fileIdPtr, 0);
    }
    env->ReleaseByteArrayElements(fileName, fileNamePtr, 0);
    
    return error == TOX_ERR_FILE_SEND_OK ? fileNumber : -1;
}

jboolean Java_nand_apps_tox_ToxNative_sendFileChunk(JNIEnv *env, jclass, jlong toxPtr, jint friendNum, jint fileNum, jlong position, jbyteArray data) {
    Tox *tox = (Tox *) toxPtr;
    
    jbyte *dataPtr = env->GetByteArrayElements(data, NULL);
    int length = env->GetArrayLength(data);
    
    bool success = tox_file_send_chunk(tox, friendNum, fileNum, position, (uint8_t*) dataPtr, length, NULL);
    
    env->ReleaseByteArrayElements(data, dataPtr, 0);
    
    return success;
}

jint Java_nand_apps_tox_ToxNative_sendFileControl(JNIEnv *env, jclass, jlong toxPtr, jint friendNum, jint fileNum, jint control) {
    Tox *tox = (Tox *) toxPtr;
    Tox_Err_File_Control error;
    tox_file_control(tox, friendNum, fileNum, (Tox_File_Control) control, &error);
    return error;
}