#include <tox/tox.h>
#include <stdlib.h>

#include "friends.h"
#include "util.h"
#include "logger.h"

jobjectArray Java_nand_apps_tox_ToxNative_getFriendList(JNIEnv *env, jclass, jlong toxPtr) {
    Tox *tox = (Tox *) toxPtr;
    
    jclass cls = env->FindClass("nand/apps/tox/ToxUserData");
    size_t sz = tox_self_get_friend_list_size(tox);
    uint32_t *friend_list = (uint32_t *) malloc(sizeof(uint32_t) * sz);
    tox_self_get_friend_list(tox, friend_list);
    jobjectArray friendArray = env->NewObjectArray(sz, cls, NULL);

    size_t len;
    uint8_t *name;
    uint8_t *status_message;
    uint8_t pubkey[TOX_PUBLIC_KEY_SIZE];

    for (int i = 0; i < sz; i++) {
        Tox_Friend_Number friend_num = friend_list[i];

        len = tox_friend_get_name_size(tox, friend_num, NULL) + 1;
        name = (uint8_t *) calloc(1, len);
        tox_friend_get_name(tox, friend_num, name, NULL);

        len = tox_friend_get_status_message_size(tox, friend_num, NULL) + 1;
        status_message = (uint8_t *) calloc(1, len);
        tox_friend_get_status_message(tox, friend_num, status_message, NULL);

        tox_friend_get_public_key(tox, friend_num, pubkey, NULL);
        
        Tox_User_Status status = tox_friend_get_status(tox, friend_num, NULL);
        uint64_t last_online = tox_friend_get_last_online(tox, friend_num, NULL);
        
        jobject friendData = getUserData(env, tox, friend_num, name, status, status_message, pubkey, TOX_GROUP_ROLE_USER, TOX_CONNECTION_NONE, last_online);
        env->SetObjectArrayElement(friendArray, i, friendData);
        
        free(name);
        free(status_message);
    }
    free(friend_list);
    
    return friendArray;
}

jint Java_nand_apps_tox_ToxNative_sendFriendRequest(JNIEnv *env, jclass, jlong toxPtr, jbyteArray address, jbyteArray message) {
    Tox *tox = (Tox *) toxPtr;
    
    jbyte *addressPtr = env->GetByteArrayElements(address, NULL);
    jbyte *messagePtr = env->GetByteArrayElements(message, NULL);
    int messageLength = env->GetArrayLength(message);
    Tox_Err_Friend_Add error;
    
    jint friend_num = tox_friend_add(tox, (const uint8_t*) addressPtr, (const uint8_t*) messagePtr, messageLength, &error);
    
    env->ReleaseByteArrayElements(address, addressPtr, 0);
    env->ReleaseByteArrayElements(message, messagePtr, 0);
    
    return error == TOX_ERR_FRIEND_ADD_OK ? friend_num : -1;
}

jint Java_nand_apps_tox_ToxNative_acceptFriendRequest(JNIEnv *env, jclass, jlong toxPtr, jbyteArray publicKey) {
    Tox *tox = (Tox *) toxPtr;
    
    jbyte *publicKeyPtr = env->GetByteArrayElements(publicKey, NULL);
    Tox_Err_Friend_Add error;
    
    jint friend_num = tox_friend_add_norequest(tox, (const uint8_t*) publicKeyPtr, &error);
    
    env->ReleaseByteArrayElements(publicKey, publicKeyPtr, 0);
    
    return error == TOX_ERR_FRIEND_ADD_OK ? friend_num : -1;
}

jboolean Java_nand_apps_tox_ToxNative_removeFriend(JNIEnv *env, jclass, jlong toxPtr, jint friendNum) {
    return tox_friend_delete((Tox *) toxPtr, friendNum, NULL);
}

jint Java_nand_apps_tox_ToxNative_sendFriendMessage(JNIEnv *env, jclass, jlong toxPtr, jint friendNum, jbyteArray message, jint messageType) {
    Tox *tox = (Tox *) toxPtr;
    Tox_Err_Friend_Send_Message error;
    jbyte *messagePtr = env->GetByteArrayElements(message, NULL);
    int messageLength = env->GetArrayLength(message);
    int messageId = tox_friend_send_message(tox, friendNum, (Tox_Message_Type) messageType, (uint8_t*) messagePtr, messageLength, &error);
    env->ReleaseByteArrayElements(message, messagePtr, 0);
    return error == TOX_ERR_FRIEND_SEND_MESSAGE_OK ? messageId : -1;
}

jboolean Java_nand_apps_tox_ToxNative_sendFriendPacket(JNIEnv *env, jclass, jlong toxPtr, jint friendNum, jbyteArray packet) {
    Tox *tox = (Tox *) toxPtr;
    jbyte *packetPtr = env->GetByteArrayElements(packet, NULL);
    int packetLength = env->GetArrayLength(packet);
    Tox_Err_Friend_Custom_Packet error;
    jboolean success = tox_friend_send_lossless_packet(tox, friendNum, (const uint8_t*) packetPtr, packetLength, &error);
    if (!success) {
        LOGGER_WRITE(TOX_LOG_LEVEL_WARNING, "Failed to send packet to friend #%d (%d) %d", friendNum, error);
    }
    return success;
}