#include <tox/tox.h>
#include <stdlib.h>

#include "self.h"
#include "util.h"

jobject Java_nand_apps_tox_ToxNative_getSelfUserData(JNIEnv *env, jclass, jlong toxPtr) {
    Tox *tox = (Tox *) toxPtr;
    
    size_t len;
    uint8_t *name;
    uint8_t *status_message;
    uint8_t pubkey[TOX_PUBLIC_KEY_SIZE];
    
    len = tox_self_get_name_size(tox) + 1;
    name = (uint8_t *) calloc(1, len);
    tox_self_get_name(tox, name);

    len = tox_self_get_status_message_size(tox) + 1;
    status_message = (uint8_t *) calloc(1, len);
    tox_self_get_status_message(tox, status_message);
    
    Tox_User_Status status = tox_self_get_status(tox);

    tox_self_get_public_key(tox, pubkey);
    
    jobject selfData = getUserData(env, tox, -1, name, status, status_message, pubkey, TOX_GROUP_ROLE_USER, TOX_CONNECTION_NONE, 0L);
    
    free(name);
    free(status_message);
    
    return selfData;
}

jboolean Java_nand_apps_tox_ToxNative_setSelfUserData(JNIEnv *env, jclass, jlong toxPtr, jbyteArray name, jbyteArray status) {
    Tox *tox = (Tox *) toxPtr;
    
    jbyte *namePtr = env->GetByteArrayElements(name, NULL);
    jbyte *statusPtr = env->GetByteArrayElements(status, NULL);
    int nameLength = env->GetArrayLength(name);
    int statusLength = env->GetArrayLength(status);
    Tox_Err_Friend_Add error;
    
    bool success = false;
    
    success |= tox_self_set_name(tox, (uint8_t *) namePtr, nameLength, NULL);
    success |= tox_self_set_status_message(tox, (uint8_t *) statusPtr, statusLength, NULL);
    
    env->ReleaseByteArrayElements(name, namePtr, 0);
    env->ReleaseByteArrayElements(status, statusPtr, 0);
    
    return success;
}

void Java_nand_apps_tox_ToxNative_setSelfStatus(JNIEnv *, jclass, jlong toxPtr, jint status) {
    Tox *tox = (Tox *) toxPtr;
    tox_self_set_status(tox, (Tox_User_Status) status);
}

jbyteArray Java_nand_apps_tox_ToxNative_getSelfAddress(JNIEnv *env, jclass, jlong toxPtr) {
    Tox *tox = (Tox *) toxPtr;
    uint8_t addr[TOX_ADDRESS_SIZE] = {0x00};
    tox_self_get_address(tox, addr);
    return newByteArray(env, addr, TOX_ADDRESS_SIZE);
}

void Java_nand_apps_tox_ToxNative_setSelfNoSpam(JNIEnv *, jclass, jlong toxPtr, jint nospam) {
    tox_self_set_nospam((Tox *) toxPtr, nospam);
}