package nand.apps.chat.audio

import android.content.Context
import android.media.AudioDeviceInfo
import android.media.AudioDeviceInfo.*
import android.media.AudioManager
import android.os.Build

private const val RESULT_OK = 0

private typealias AudioDevices = Array<out AudioDeviceInfo>
private typealias AudioDeviceTypeMap = Map<Int, Int>

abstract class AndroidAudioEngine(
    context: Context,
    isPlaybackEngine: Boolean,
) {

    protected val enginePtr = AndroidAudioNative.initEngine(isPlaybackEngine = isPlaybackEngine)
    protected val audioManager = context.getSystemService(Context.AUDIO_SERVICE) as AudioManager
    protected val defaultSampleRate by lazy { audioManager.getProperty(AudioManager.PROPERTY_OUTPUT_SAMPLE_RATE).toInt() }

    protected fun startEngine(
        deviceId: Int,
        channels: Int,
        isLowLatency: Boolean,
    ): Boolean {
        val result = AndroidAudioNative.startEngine(
            ptr = enginePtr,
            deviceId = deviceId,
            sampleRate = defaultSampleRate,
            channelCount = channels,
            isLowLatency = isLowLatency,
        )
        return result == RESULT_OK
    }

    protected fun stopEngine(): Boolean {
        val result = AndroidAudioNative.stopEngine(enginePtr)
        return result == RESULT_OK
    }

    fun finalize() {
        AndroidAudioNative.destroyEngine(enginePtr)
    }
}

fun getBleHeadsetType() = when {
    Build.VERSION.SDK_INT >= Build.VERSION_CODES.S -> TYPE_BLE_HEADSET
    else -> null
}

fun getBleSpeakerType() = when {
    Build.VERSION.SDK_INT >= Build.VERSION_CODES.S -> TYPE_BLE_SPEAKER
    else -> null
}

fun mapOfDeviceTypes(vararg elements: Int?) =
    elements.mapIndexedNotNull { index, type -> type?.let { it to index } }.toMap()

fun AudioDevices.sortedBy(typeMap: AudioDeviceTypeMap) = sortedBy { device ->
    typeMap[device.type] ?: Int.MAX_VALUE
}