package nand.apps.chat.audio

import kotlinx.coroutines.*
import nand.apps.chat.log.getLogger
import nand.apps.chat.time.currentTimeMillis
import java.util.concurrent.ConcurrentLinkedDeque

private const val CYCLE_DELAY = 20L
private const val STALE_TIMEOUT = 60 * 1000L // Stale after 60 seconds of no audio

class AndroidAudioPlaybackStream(
    private val coroutineScope: CoroutineScope,
    private val enginePtr: Long,
    private val streamId: Long,
) {

    var isActive = false
        private set
    
    private val frames = ConcurrentLinkedDeque<AudioFrame>()
    private val logger = getLogger("AndroidAudioPlaybackStream")
    
    fun stop() {
        if (isActive) {
            logger.debug("STOPPING STREAM $streamId")
            isActive = false
        }
    }
    
    fun queue(frame: AudioFrame) {
        frames += frame
        if (!isActive) {
            isActive = true
            coroutineScope.launch(Dispatchers.IO) { run() }
        }
    }
    
    private suspend fun run() {
        var lastQueueTime = currentTimeMillis()
        logger.debug("STARTING STREAM $streamId")
        while (isActive) {
            val curTime = currentTimeMillis()
            if (frames.isNotEmpty()) {
                //logger.debug("Queueing ${frames.size} for stream #$streamId")
                lastQueueTime = curTime
            }
            for (frame in frames) {
                AndroidAudioNative.writeSamples(
                    ptr = enginePtr,
                    samples = frame.samples,
                    streamId = streamId,
                )
            }
            frames.clear()
            if (curTime - lastQueueTime > STALE_TIMEOUT) {
                isActive = false
                logger.debug("STOPPING STREAM $streamId")
                break
            }
            delay(CYCLE_DELAY)
        }
    }
}