package nand.apps.chat.camera

import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts.StartActivityForResult
import androidx.activity.result.contract.ActivityResultContracts.TakePicture
import androidx.core.content.FileProvider
import nand.apps.chat.MainActivity
import nand.apps.chat.model.event.*
import nand.apps.chat.event.ChatEventHandler
import okio.Path
import org.koin.core.Koin

const val INTENT_ACTION_QR_CODE_CAMERA_RESULT = "QR_CODE_CAMERA_RESULT"

class AndroidCameraLauncher(
    private val appContext: Context,
    koin: Koin,
) : CameraLauncher {

    override val isSupported = appContext.packageManager.hasSystemFeature(PackageManager.FEATURE_CAMERA_ANY)
    
    private var activity: MainActivity? = null
    private var activityLauncher: ActivityResultLauncher<Intent>? = null
    private var photoLauncher: ActivityResultLauncher<Uri>? = null
    private var photoPath: Path? = null
    
    private val eventHandler: ChatEventHandler by koin.inject()
    
    fun start(activity: MainActivity) {
        this.activity = activity
        activityLauncher = activity.registerForActivityResult(StartActivityForResult()) { result ->
            when (result.data?.action) {
                INTENT_ACTION_QR_CODE_CAMERA_RESULT -> {
                    val text = result.data?.data ?: return@registerForActivityResult
                    eventHandler += TakePhotoQrCode(text.toString())
                }
            }
        }
        photoLauncher = activity.registerForActivityResult(TakePicture()) { result ->
            if (result) {
                photoPath?.let { eventHandler += TakePhoto(it) }
            }
            photoPath = null
        }
    }
    
    fun stop() {
        activityLauncher?.unregister()
        photoLauncher?.unregister()
        activity = null
    }

    override fun launchForPhoto(path: Path) {
        photoPath = path
        val uri = FileProvider.getUriForFile(appContext, "${appContext.packageName}.provider", path.toFile())
        photoLauncher?.launch(uri)
    }

    override fun launchForQrCode() {
        activity?.let {
            activityLauncher?.launch(Intent(it, QrCodeCameraActivity::class.java))
        }
    }
}