package nand.apps.chat.camera

import android.Manifest
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.result.contract.ActivityResultContracts.RequestPermission
import androidx.core.content.ContextCompat
import androidx.core.net.toUri
import nand.apps.chat.ui.theme.ChatThemeSurface

class QrCodeCameraActivity : ComponentActivity() {

    private val cameraPermissionRequest = registerForActivityResult(RequestPermission()) { isGranted ->
        if (isGranted) {
            setCameraPreview()
        } else {
            finishActivity(RESULT_CANCELED)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        when (PackageManager.PERMISSION_GRANTED) {
            ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) -> {
                setCameraPreview()
            }
            else -> {
                cameraPermissionRequest.launch(Manifest.permission.CAMERA)
            }
        }
    }
    
    private fun setCameraPreview() {
        setContent {
            ChatThemeSurface {
                QrCodeCameraScreen(
                    onResult = { result ->
                        setResult(RESULT_OK, Intent(INTENT_ACTION_QR_CODE_CAMERA_RESULT).setData(result.text.toUri()))
                        finish()
                    },
                    onCancel = { finishActivity(RESULT_CANCELED) },
                )
            }
        }
    }
}