package nand.apps.chat.di

import android.content.Context
import nand.apps.chat.audio.AndroidAudioPlayer
import nand.apps.chat.audio.AndroidAudioRecorder
import nand.apps.chat.audio.AudioPlayer
import nand.apps.chat.audio.AudioRecorder
import nand.apps.chat.camera.AndroidCameraLauncher
import nand.apps.chat.camera.CameraLauncher
import nand.apps.chat.dev.DevCommandLogger
import nand.apps.chat.dev.NoOpCommandLogger
import nand.apps.chat.image.DrawablePixelReader
import nand.apps.chat.image.ImagePixelReader
import nand.apps.chat.io.AndroidFileBrowser
import nand.apps.chat.io.AndroidFilePaths
import nand.apps.chat.io.ChatFileBrowser
import nand.apps.chat.io.ChatFilePaths
import nand.apps.chat.net.AndroidUriLauncher
import nand.apps.chat.net.UriLauncher
import nand.apps.chat.notification.AndroidNotificationController
import nand.apps.chat.platform.AndroidAppStateProvider
import nand.apps.chat.platform.AppStateProvider
import nand.apps.chat.profile.AndroidCredentialsStore
import nand.apps.chat.profile.ProfileCredentialsStore
import nand.apps.chat.share.AndroidContentShare
import nand.apps.chat.share.ContentShare
import nand.apps.chat.ui.shortcut.AndroidKeyEventDispatcher
import org.koin.android.ext.koin.androidContext
import org.koin.android.ext.koin.androidLogger
import org.koin.core.module.dsl.singleOf
import org.koin.dsl.module

fun initAndroidKoin(context: Context) {
    System.loadLibrary("toxjni")
    System.loadLibrary("gifjni")
    System.loadLibrary("opusjni")
    System.loadLibrary("androidjni")
    initKoin {
        androidLogger()
        androidContext(context)
        val platformModules = module {
            single<ProfileCredentialsStore> { AndroidCredentialsStore(get()) }
            single<ChatFileBrowser> { AndroidFileBrowser(get(), get()) }
            single<AppStateProvider> { AndroidAppStateProvider(get()) }
            single<UriLauncher> { AndroidUriLauncher(get()) }
            single<ContentShare> { AndroidContentShare(get()) }
            single<ChatFilePaths> { AndroidFilePaths(get()) }
            single<CameraLauncher> { AndroidCameraLauncher(get(), getKoin()) }
            single<AudioPlayer> { AndroidAudioPlayer(get(), get()) }
            single<AudioRecorder> { AndroidAudioRecorder(get()) }
            single<ImagePixelReader> { DrawablePixelReader(get()) }
            single<DevCommandLogger> { NoOpCommandLogger() }
            singleOf(::AndroidNotificationController)
            singleOf(::AndroidKeyEventDispatcher)
        }
        modules(platformModules)
    }
}