package nand.apps.chat.image

import android.content.Context
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.drawable.BitmapDrawable
import androidx.compose.ui.unit.IntRect
import coil3.Image
import coil3.PlatformContext
import coil3.asDrawable
import coil3.request.ImageRequest
import coil3.request.allowHardware

class DrawablePixelReader(
    private val context: Context,
) : ImagePixelReader {

    override fun createImageRequestBuilder(context: PlatformContext): ImageRequest.Builder {
        return ImageRequest.Builder(context).allowHardware(false)
    }
    
    override fun readPixels(image: Image, rect: IntRect): ByteArray? {
        return (image.asDrawable(context.resources) as? BitmapDrawable)?.bitmap?.let { bitmap ->
            val canvasBmp = Bitmap.createBitmap(rect.width, rect.height, Bitmap.Config.ARGB_8888)
            val canvas = Canvas(canvasBmp)
            canvas.drawBitmap(bitmap, -rect.left.toFloat(), -rect.top.toFloat(), null)
            val intPixels = IntArray(rect.width * rect.height)
            canvasBmp.getPixels(intPixels, 0, rect.width, 0, 0, rect.width, rect.height)
            var p = 0
            val pixels = ByteArray(rect.width * rect.height * 4)
            for (i in intPixels) {
                pixels[p++] = i.toSignedByte(16)
                pixels[p++] = i.toSignedByte(8)
                pixels[p++] = i.toSignedByte(0)
                pixels[p++] = i.toSignedByte(24)
            }
            pixels
        }
    }
    
    private fun Int.toSignedByte(shift: Int) = ((this shr shift) and 0xFF).toUByte().toByte()
}