package nand.apps.chat.image

import android.graphics.Bitmap
import androidx.compose.ui.graphics.ImageBitmap
import androidx.compose.ui.graphics.asImageBitmap
import java.io.ByteArrayOutputStream
import kotlin.math.roundToInt

actual fun SimpleBitmap.toComposeBitmap(): ImageBitmap {
    return toAndroidBitmap().asImageBitmap()
}

actual fun SimpleBitmap.toPngByteArray(): ByteArray {
    val bitmap = toAndroidBitmap()
    return ByteArrayOutputStream().use { stream ->
        bitmap.compress(Bitmap.CompressFormat.PNG, 100, stream)
        stream.toByteArray()
    }
}

actual fun SimpleBitmap.toJpegByteArray(quality: Float): ByteArray {
    val bitmap = toAndroidBitmap()
    return ByteArrayOutputStream().use { stream ->
        bitmap.compress(Bitmap.CompressFormat.JPEG, (quality * 100).roundToInt(), stream)
        stream.toByteArray()
    }
}

private fun SimpleBitmap.toAndroidBitmap(): Bitmap {
    var p = 0
    val colors = IntArray(pixels.size / 4) {
        val red = pixels[p++].toInt() and 0xFF
        val green = pixels[p++].toInt() and 0xFF
        val blue = pixels[p++].toInt() and 0xFF
        val alpha = pixels[p++].toInt() and 0xFF
        (alpha shl 24) or (red shl 16) or (green shl 8) or blue
    }
    return Bitmap.createBitmap(colors, width, height, Bitmap.Config.ARGB_8888)
}