package nand.apps.chat.intent

import android.content.Intent
import android.os.Build
import android.os.Bundle
import nand.apps.chat.model.uid.ContactUid
import nand.apps.chat.model.uid.FriendRequestUid
import nand.apps.chat.model.uid.GroupUid
import nand.apps.chat.model.uid.UserUid
import java.io.Serializable

enum class ContactUidType {
    USER,
    GROUP,
    FRIEND_REQUEST,
}

data class SerializableContactUid(
    val value: String,
    val type: ContactUidType,
) : Serializable

fun ContactUid.toSerializable(): SerializableContactUid {
    val type = when (this) {
        is UserUid -> ContactUidType.USER
        is GroupUid -> ContactUidType.GROUP
        is FriendRequestUid -> ContactUidType.FRIEND_REQUEST
    }
    return SerializableContactUid(value = value, type = type)
}

fun Intent.putExtra(name: String, uid: ContactUid) =
    putExtra(name, uid.toSerializable())

fun Bundle.putExtra(name: String, uid: ContactUid) =
    putSerializable(name, uid.toSerializable())

fun Intent.getUidExtra(name: String) = extras?.getUid(name)

fun Bundle.getUid(name: String): ContactUid? {
    val uid = getCustom<SerializableContactUid>(name)
    val type = uid?.type ?: return null
    return when (type) {
        ContactUidType.USER -> UserUid(uid.value)
        ContactUidType.GROUP -> GroupUid(uid.value)
        ContactUidType.FRIEND_REQUEST -> FriendRequestUid(uid.value)
    }
}

inline fun <reified T : Serializable> Intent.getCustomExtra(name: String) =
    extras?.getCustom<T>(name)

@Suppress("DEPRECATION")
inline fun <reified T : Serializable> Bundle.getCustom(name: String): T? {
    return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
        getSerializable(name, T::class.java)
    } else {
        getSerializable(name) as? T
    }
}