package nand.apps.chat.io

import android.content.Context
import android.net.Uri
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.PickVisualMediaRequest
import androidx.activity.result.contract.ActivityResultContracts.*
import androidx.documentfile.provider.DocumentFile
import nand.apps.chat.MainActivity
import okio.Path

class AndroidFileBrowser(
    private val context: Context,
    fileSystem: ChatFileSystem,
) : ChatFileBrowser {

    private var activity: MainActivity? = null
    private var browserLauncher: ActivityResultLauncher<Array<String>>? = null
    private var galleryLauncher: ActivityResultLauncher<PickVisualMediaRequest>? = null
    private var onFileSelected: ((Path?) -> Unit)? = null
    private val tempDirectory = fileSystem.getTempDirectory("filebrowser")

    fun start(activity: MainActivity) {
        this.activity = activity
        browserLauncher = activity.registerForActivityResult(OpenDocument()) { uri ->
            uri?.let { onSelection(it) }
        }
        galleryLauncher = activity.registerForActivityResult(PickVisualMedia()) { uri ->
            uri?.let { onSelection(it) }
        }
    }

    fun stop() {
        browserLauncher?.unregister()
        galleryLauncher?.unregister()
        onFileSelected = null
        activity = null
    }

    override fun isBrowserSupported() = browserLauncher != null

    override fun isGallerySupported() = galleryLauncher != null

    override fun launchBrowser(onFileSelected: (Path?) -> Unit) {
        this.onFileSelected = onFileSelected
        browserLauncher?.launch(arrayOf("*/*"))
    }

    override fun launchGallery(onFileSelected: (Path?) -> Unit) {
        this.onFileSelected = onFileSelected
        galleryLauncher?.launch(PickVisualMediaRequest())
    }

    private fun onSelection(uri: Uri) {
        val onFileSelected = this.onFileSelected ?: return
        val documentFile = DocumentFile.fromSingleUri(context, uri) ?: return
        val fileName = documentFile.name ?: return
        val contentBytes = context.contentResolver.openInputStream(uri)?.use { it.readBytes() } ?: return
        if (tempDirectory.writeToFile(fileName, contentBytes)) {
            onFileSelected(tempDirectory.getFilePath(fileName))
        } else {
            onFileSelected(null)
        }
        this.onFileSelected = null
    }
}