package nand.apps.chat.ui.shortcut

import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import android.view.KeyEvent as AndroidKeyEvent
import androidx.compose.ui.input.key.KeyEvent
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.ui.modifier.toShortcutAction

class AndroidKeyEventDispatcher(
    private val settingsRepo: SettingsRepo,
) {

    private val keyEventDispatcher = KeyEventDispatcher()
    private val shortcutDispatcher = ShortcutDispatcher()
    
    fun onKeyDown(event: AndroidKeyEvent): Boolean {
        val composeKeyEvent = KeyEvent(event)
        val shortcutAction = composeKeyEvent.toShortcutAction(settingsRepo.settings.shortcutSettings)
        return keyEventDispatcher(composeKeyEvent) || shortcutAction?.let { shortcutDispatcher(it) } ?: false
    }
    
    fun onKeyUp(event: AndroidKeyEvent): Boolean {
        return keyEventDispatcher(KeyEvent(event))
    }

    @Composable
    fun CompositionLocalProvider(content: @Composable () -> Unit) = CompositionLocalProvider(
        LocalShortcutDispatcher provides shortcutDispatcher,
        LocalKeyEventDispatcher provides keyEventDispatcher,
        content = content,
    )
}