package nand.apps.chat.ui.viewmodel

import androidx.compose.runtime.Composable
import androidx.compose.runtime.RememberObserver
import androidx.compose.runtime.remember
import androidx.lifecycle.ViewModelStore
import com.hoc081098.kmp.viewmodel.ViewModelStoreOwner
import com.hoc081098.kmp.viewmodel.toKmp
import androidx.lifecycle.ViewModelStoreOwner as AndroidXViewModelStoreOwner

private class ComposableViewModelStoreOwner(
    private val impl: AndroidXViewModelStoreOwner,
) : ViewModelStoreOwner, RememberObserver {

    override val viewModelStore by lazy {
        impl.viewModelStore.toKmp()
    }

    override fun onAbandoned() {
        viewModelStore.clear()
    }

    override fun onForgotten() {
        viewModelStore.clear()
    }

    override fun onRemembered() {
        // Do nothing
    }
}

private class ComposableViewModelStoreOwnerImpl : AndroidXViewModelStoreOwner {

    override val viewModelStore = ViewModelStore()
}

@Composable
actual fun getComposableViewModelStoreOwner(): ViewModelStoreOwner {
    return remember {
        ComposableViewModelStoreOwner(ComposableViewModelStoreOwnerImpl())
    }
}