package nand.apps.chat.audio

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Job
import kotlinx.coroutines.launch

class AudioRepeater(
    private val audioPlayer: AudioPlayer,
    private val soundName: String,
    private val onPlay: suspend () -> Unit,
    private val onReset: () -> Unit = {},
) {
    
    private var isActive = false
    private var job: Job? = null
    private var soundHandle: AudioHandle? = null
    
    fun update(isActive: Boolean, coroutineScope: CoroutineScope) {
        if (this.isActive != isActive) {
            reset()
            this.isActive = isActive
            if (isActive) {
                job = coroutineScope.launch {
                    while (this@AudioRepeater.isActive) {
                        soundHandle = audioPlayer.queueSound(soundName)
                        onPlay()
                    }
                }
            }
        }
    }
    
    fun reset() {
        job?.cancel()
        job = null
        soundHandle?.let { audioPlayer.stopSound(it) }
        soundHandle = null
        onReset()
    }
}