package nand.apps.chat.embed.amazon

import io.ktor.client.*
import nand.apps.chat.embed.EmbedUrlParser
import nand.apps.chat.util.getHtmlDocument

private val PRODUCT_URL_REGEX = Regex("(?:https?://)?(?:www)?\\.amazon\\.[a-z.]+(?:/.*)?/dp/B[A-Z0-9]{9}(?:[/?#].*)?")

class AmazonUrlParser : EmbedUrlParser {

    override fun matches(url: String) = PRODUCT_URL_REGEX.matches(url)

    override suspend fun parse(url: String, client: HttpClient): AmazonProductMetadata? {
        val document = client.getHtmlDocument(url) ?: return null
        val name = document.selectFirst("#productTitle")?.text()
        val imageUrl = document.selectFirst("#main-image-container #landingImage")?.attr("src")
        val priceElement = document.selectFirst("#corePriceDisplay_desktop_feature_div span.a-price")
        val priceSymbol = priceElement?.selectFirst("span.a-price-symbol")?.text()
        val priceWhole = priceElement?.selectFirst("span.a-price-whole")?.ownText()
        val priceFraction = priceElement?.selectFirst("span.a-price-fraction")?.ownText()
        val features = document.selectFirst("#feature-bullets ul li")?.text()
        return if (name != null && imageUrl != null && priceSymbol != null && priceWhole != null) {
            AmazonProductMetadata(
                url = url,
                name = name,
                imageUrl = imageUrl,
                priceSymbol = priceSymbol,
                priceWhole = priceWhole,
                priceFraction = priceFraction,
                features = features,
            )
        } else {
            null
        }
    }
}