package nand.apps.chat.embed.fourchan

import io.ktor.client.*
import kotlinx.serialization.json.*
import nand.apps.chat.embed.EmbedUrlParser
import nand.apps.chat.util.*

private val THREAD_URL_MATCHER = Regex("(?:https?://)?(?:www\\.|boards\\.)?4chan(?:nel)?\\.org/(\\w+)/thread/(\\d+)(#p\\d+)?")

class FourChanUrlParser : EmbedUrlParser {

    override fun matches(url: String) = THREAD_URL_MATCHER.matches(url)
    
    override suspend fun parse(url: String, client: HttpClient): FourChanPostMetadata? {
        val matches = THREAD_URL_MATCHER.matchEntire(url)?.groupValues ?: return null
        val board = matches[1]
        val threadNum = matches[2].toInt()
        val postNum = matches[3].takeIf { it.isNotBlank() }?.substring(2)?.toIntOrNull() ?: threadNum
        val json = client.getJsonObject("https://a.4cdn.org/$board/thread/$threadNum.json") ?: return null
        val posts = json["posts"].jsonArraySafe()
        val originalPost = posts.firstOrNull().jsonObjectSafe()
        val post = posts.find { (it as? JsonObject)?.get("no").getInt() == postNum } as? JsonObject ?: originalPost
        val timestampMicro = post["tim"].getLong()
        return FourChanPostMetadata(
            id = post["no"].getInt(),
            timestampMs = (timestampMicro / 1000L).takeIf { it > 0 } ?: (post["time"].getInt() * 1000L),
            timestampMicro = timestampMicro,
            timestampString = post["now"].getString().orEmpty(),
            name = post["name"].getString() ?: "Anonymous",
            comment = post["com"].getString().orEmpty(),
            image = post.getImage(),
            thread = FourChanThreadMetadata(
                id = threadNum,
                board = board,
                subject = originalPost["sub"].getString(),
                replyCount = originalPost["replies"].getInt(),
                imageCount = originalPost["images"].getInt(),
            ),
        )
    }
    
    private fun JsonObject.getImage(): FourChanImageMetadata? {
        val baseName = this["filename"].getString() ?: return null
        return FourChanImageMetadata(
            baseName = baseName,
            extension = this["ext"].getString().orEmpty(),
            size = this["fsize"].getLong(),
            width = this["w"].getInt(),
            height = this["h"].getInt(),
            isSpoiler = containsKey("spoiler"),
        )
    }
}