package nand.apps.chat.embed.mediawiki

import io.ktor.client.*
import nand.apps.chat.embed.EmbedUrlParser
import nand.apps.chat.util.*

private val PAGE_URL_REGEX = Regex("(?:https?://)?([a-z]+)\\.wikipedia\\.org/wiki/([^/#?&]+)(?:[#?&].*)?")

class WikipediaUrlParser : EmbedUrlParser {

    override fun matches(url: String) = PAGE_URL_REGEX.matches(url)

    override suspend fun parse(url: String, client: HttpClient): MediaWikiPageMetadata? {
        val (_, lang, pageName) = PAGE_URL_REGEX.matchEntire(url)?.groupValues ?: return null
        if (pageName.startsWith("File:") || pageName.startsWith("Special:")) return null
        
        val json = client.getJsonObject("https://$lang.wikipedia.org/api/rest_v1/page/summary/$pageName") ?: return null
        val title = json["title"].getString()
        val summary = json["extract_html"].getString()
        val thumbnail = json["thumbnail"].jsonObjectSafe()
        val thumbUrl = thumbnail["source"].getString() ?: "https://$lang.wikipedia.org/static/images/project-logos/${lang}wiki.png"
        val thumbWidth = thumbnail["width"].getInt()
        val thumbHeight = thumbnail["height"].getInt()
        
        return if (title != null && summary != null) {
            MediaWikiPageMetadata(
                url = url,
                title = title,
                summary = summary.replace("<p>", "").replace("</p>", ""),
                thumbnail = MediaWikiImageMetadata(
                    url = thumbUrl,
                    fileName = thumbUrl.substringAfterLast('/'),
                    width = thumbWidth,
                    height = thumbHeight,
                )
            )
        } else {
            null
        }
    }
}