package nand.apps.chat.embed.stackexchange

import io.ktor.client.*
import kotlinx.datetime.Instant
import kotlinx.datetime.format.DateTimeComponents
import kotlinx.datetime.format.FormatStringsInDatetimeFormats
import kotlinx.datetime.format.byUnicodePattern
import nand.apps.chat.embed.EmbedUrlParser
import nand.apps.chat.util.getHtmlDocument

private val PAGE_URL_REGEX = Regex("(?:https?://)?(?:[\\w-]+\\.)?[\\w-]+\\.[a-z]{1,3}/questions/[0-9]+/[\\w\\-?&=#]*")

@OptIn(FormatStringsInDatetimeFormats::class)
private val DATE_FORMAT = DateTimeComponents.Format {
    byUnicodePattern("yyyy-MM-dd'T'HH:mm:ss")
}

class StackExchangeUrlParser : EmbedUrlParser {

    override fun matches(url: String) = PAGE_URL_REGEX.matches(url)

    override suspend fun parse(url: String, client: HttpClient): StackExchangePageMetadata? {
        val doc = client.getHtmlDocument(url)
        val title = doc?.selectFirst("#question-header h1")?.text() ?: return null
        val iconUrl = doc.selectFirst("head link[rel=\"image_src\"]")?.attr("href")
        val question = doc.selectFirst("#question") ?: return null
        val body = question.selectFirst(".postcell .js-post-body > p")?.html() ?: return null
        val askTime = doc.selectFirst("#question-header + div time")?.attr("datetime")
        return StackExchangePageMetadata(
            url = url,
            title = title,
            body = body,
            iconUrl = iconUrl,
            askTime = askTime?.let { Instant.parse(it, DATE_FORMAT).toEpochMilliseconds() },
        )
    }
}