package nand.apps.chat.embed.youtube.video

import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import nand.apps.chat.embed.EmbedPreviewData
import nand.apps.chat.embed.youtube.channel.YoutubeChannelMetadata
import nand.apps.chat.ui.embed.youtube.Renderer

const val THUMBNAIL_URL_ERROR = "http://s.ytimg.com/yts/img/meh7-vflGevej7.png"

data class YoutubeVideoMetadata(
    val url: String,
    val id: String,
    val title: String,
    val description: String? = null,
    val status: YoutubeVideoStatus = YoutubeVideoStatus.UNKNOWN,
    val lengthSecs: Int = 0,
    val viewCount: Long = 0,
    val uploadDate: Long? = null,
    val publishDate: Long? = null,
    val thumbnailUrl: String = THUMBNAIL_URL_ERROR,
    val channel: YoutubeChannelMetadata? = null,
    val isFamilySafe: Boolean = true,
    val isUnlisted: Boolean = false,
    val isShortsEligible: Boolean = false,
    val isUpcoming: Boolean = false,
    val isMembersOnly: Boolean = false,
    val liveBroadcast: YoutubeLiveMetadata? = null,
    val error: YoutubeVideoErrorMetadata? = null,
) : EmbedPreviewData {
    
    val isPlayable get() = status == YoutubeVideoStatus.OK
    val isAvailable get() = title.isNotEmpty()
    val isLiveContent get() = liveBroadcast != null
    val isLiveNow get() = liveBroadcast?.isLiveNow == true

    @Composable
    override fun GetComposable(modifier: Modifier) = Renderer(modifier)
}