package nand.apps.chat.engine

import kotlinx.coroutines.CoroutineScope
import nand.apps.chat.model.user.ChatProfileData
import nand.apps.chat.net.packet.ChatPacketRepo
import nand.apps.chat.repo.*
import org.koin.core.Koin

class DefaultChatEngineCallback(koin: Koin) : ChatEngineCallback {

    private val contactRepo: ContactRepo by koin.inject()
    private val selfProfileRepo: SelfProfileRepo by koin.inject()
    private val messageRepo: ChatMessageRepo by koin.inject()
    private val fileTransferRepo: FileTransferRepo by koin.inject()
    private val avatarRepo: ChatAvatarRepo by koin.inject()
    private val callRepo: CallRepo by koin.inject()
    private val voicemailRepo: VoicemailRepo by koin.inject()
    private val packetRepo: ChatPacketRepo by koin.inject()
    private val callbacks by lazy {
        listOf(fileTransferRepo, avatarRepo, contactRepo, selfProfileRepo, messageRepo, callRepo, voicemailRepo, packetRepo)
    }

    override suspend fun onEngineInit(profile: ChatProfileData) {
        callbacks.forEach { it.onEngineInit(profile) }
    }

    override fun onEngineStart(coroutineScope: CoroutineScope) {
        callbacks.forEach { it.onEngineStart(coroutineScope) }
    }

    override fun onEngineStop() {
        callbacks.forEach { it.onEngineStop() }
    }
}