package nand.apps.chat.engine.mock

import nand.apps.chat.model.file.ChatFileTransfer
import nand.apps.chat.model.group.ChatGroupInvite
import nand.apps.chat.model.group.ChatGroupRole
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.net.ConnectionProtocol
import nand.apps.chat.model.uid.ContactUid
import nand.apps.chat.model.uid.GroupUid
import nand.apps.chat.model.uid.MessageUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.FriendRequest
import nand.apps.chat.model.user.UserStatus

sealed class MockChatEvent(open val timestamp: Long)

data class FriendNameEvent(
    override val timestamp: Long,
    val friendUid: UserUid,
    val name: String,
) : MockChatEvent(timestamp)

data class FriendProtocolEvent(
    override val timestamp: Long,
    val friendUid: UserUid,
    val protocol: ConnectionProtocol,
) : MockChatEvent(timestamp)

data class FriendStatusEvent(
    override val timestamp: Long,
    val friendUid: UserUid,
    val status: UserStatus,
) : MockChatEvent(timestamp)

data class FriendStatusMessageEvent(
    override val timestamp: Long,
    val friendUid: UserUid,
    val message: String,
) : MockChatEvent(timestamp)

data class FriendRequestEvent(
    override val timestamp: Long,
    val request: FriendRequest,
) : MockChatEvent(timestamp)

data class GroupInviteEvent(
    override val timestamp: Long,
    val invite: ChatGroupInvite,
) : MockChatEvent(timestamp)

data class GroupTitleEvent(
    override val timestamp: Long,
    val groupUid: GroupUid,
    val peerUid: UserUid,
    val title: String,
) : MockChatEvent(timestamp)

data class GroupUsersChangedEvent(
    override val timestamp: Long,
    val groupUid: GroupUid,
) : MockChatEvent(timestamp)

data class GroupUserNameEvent(
    override val timestamp: Long,
    val groupUid: GroupUid,
    val peerUid: UserUid,
    val name: String,
) : MockChatEvent(timestamp)

data class GroupConnectedEvent(
    override val timestamp: Long,
    val groupUid: GroupUid,
) : MockChatEvent(timestamp)

data class GroupPeerKickedEvent(
    override val timestamp: Long,
    val groupUid: GroupUid,
    val sourceUid: UserUid,
    val targetUid: UserUid,
) : MockChatEvent(timestamp)

data class GroupPeerRoleChangedEvent(
    override val timestamp: Long,
    val groupUid: GroupUid,
    val sourceUid: UserUid,
    val targetUid: UserUid,
    val role: ChatGroupRole,
) : MockChatEvent(timestamp)

data class NewMessageEvent(
    override val timestamp: Long,
    val message: ChatMessage,
) : MockChatEvent(timestamp)

data class EditMessageEvent(
    override val timestamp: Long,
    val message: ChatMessage,
) : MockChatEvent(timestamp)

data class ReactMessageEvent(
    override val timestamp: Long,
    val senderUid: UserUid,
    val message: ChatMessage,
    val reactions: List<String>,
) : MockChatEvent(timestamp)

data class FriendReadReceiptEvent(
    override val timestamp: Long,
    val friendUid: UserUid,
    val messageUid: MessageUid,
) : MockChatEvent(timestamp)

data class ClearEvent(
    override val timestamp: Long,
    val conversationUid: ContactUid,
) : MockChatEvent(timestamp)

data class FileRequestEvent(
    override val timestamp: Long,
    val fileTransfer: ChatFileTransfer,
) : MockChatEvent(timestamp)

data class FileChunkEvent(
    override val timestamp: Long,
    val fileTransfer: ChatFileTransfer,
    val position: Long,
) : MockChatEvent(timestamp)

data class ReceiveCallEvent(
    override val timestamp: Long,
    val contactUid: ContactUid,
) : MockChatEvent(timestamp)