package nand.apps.chat.engine.tox

import nand.apps.chat.io.ChatEngineEncryption
import nand.apps.chat.io.ChatFileEncryption

class ToxChatEngineEncryption : ChatEngineEncryption {
    
    private var encryption: ToxFileEncryption? = null
        set(value) {
            if (value != field) {
                field?.close()
                field = value
            }
        }

    fun init(encryption: ToxFileEncryption?) {
        this.encryption = encryption
    }
    
    fun destroy() {
        encryption = null
    }
    
    override fun isEncryptionSupported() = encryption?.isEncryptionSupported() == true

    override fun isDecryptionSupported() = encryption?.isDecryptionSupported() == true

    override fun isEncrypted(data: ByteArray) = encryption?.isEncrypted(data) == true

    override fun encryptData(data: ByteArray) = encryption?.encryptData(data)

    override fun decryptData(data: ByteArray) = encryption?.decryptData(data)

    override fun getFileEncryption(key: String, cipher: ByteArray?): ChatFileEncryption {
        return ToxFileEncryption(
            password = key,
            cipher = cipher,
        )
    }
}