package nand.apps.chat.engine.tox

import nand.apps.chat.io.ChatDirectory
import nand.apps.chat.log.getLogger
import nand.apps.tox.ToxNative

class ToxChatEngineSaver {
    
    private var toxPtr = NULL_PTR
    private var encryption: ToxFileEncryption? = null
    private lateinit var profileDir: ChatDirectory
    private lateinit var profileName: String
    
    private val logger = getLogger("ToxChatEngine")
    
    fun init(
        toxPtr: Long,
        encryption: ToxFileEncryption?,
        profileDir: ChatDirectory,
        profileName: String,
    ) {
        this.toxPtr = toxPtr
        this.encryption = encryption
        this.profileDir = profileDir
        this.profileName = profileName
    }

    fun save() {
        var saveBytes = ToxNative.getSaveData(toxPtr)
        if (saveBytes.isEmpty()) {
            logger.error("Failed to get profile data")
            return
        }

        encryption?.run {
            saveBytes = encryptData(saveBytes) ?: run {
                logger.error("Failed to encrypt profile data")
                return
            }
        }

        if (profileDir.writeToFile(profileName, saveBytes)) {
            logger.debug("Saved profile data to $profileName (${saveBytes.size} bytes)")
        } else {
            logger.error("Failed to save profile data to $profileName")
        }
    }

    fun saveIfModified(modified: Boolean) {
        if (modified) save()
    }
}