package nand.apps.chat.engine.tox

import nand.apps.chat.engine.event.*
import nand.apps.chat.engine.tox.friend.ToxFriendSaver
import nand.apps.chat.engine.tox.group.ToxGroupSaver
import nand.apps.chat.repo.ContactRepo
import nand.apps.chat.util.clearAndIterate
import org.koin.core.Koin

class ToxChatEventHandler(koin: Koin) {

    private val contactRepo: ContactRepo by koin.inject()
    private val saver: ToxChatEngineSaver by koin.inject()
    private val friendSaver: ToxFriendSaver by koin.inject()
    private val groupSaver: ToxGroupSaver by koin.inject()
    private val events = mutableListOf<ChatEngineEvent>()

    fun handleEvents() {
        events.clearAndIterate { event ->
            when (event) {
                is SaveProfileEvent -> {
                    saver.save()
                }
                is SaveFriendEvent -> {
                    contactRepo.getFriend(event.friendUid)?.let { friendSaver.save(it) }
                }
                is SaveGroupEvent -> {
                    contactRepo.getGroup(event.groupUid)?.let { groupSaver.save(it) }
                }
                is GroupUsersChangedEvent -> {
                    contactRepo.onGroupUsersChanged(event.groupUid)
                }
                is JoinGroupEvent -> {
                    contactRepo.joinGroup(
                        groupUid = event.groupUid,
                        selfNickname = event.selfNickname,
                        password = event.password,
                    )
                }
            }
        }
    }
    
    operator fun plusAssign(event: ChatEngineEvent) {
        events += event
    }
}