package nand.apps.chat.engine.tox

import nand.apps.chat.io.ChatFileEncryption
import nand.apps.tox.ToxNative
import okio.Closeable

class ToxFileEncryption(
    password: String,
    cipher: ByteArray?,
) : ChatFileEncryption, Closeable {
    
    private var keyPtr = ToxNative.createPassKey(
        passPhrase = password.encodeToByteArray(),
        cipher = cipher?.takeIf { ToxNative.isEncrypted(it) },
    )
    
    override fun close() {
        if (keyPtr != NULL_PTR) {
            ToxNative.freePassKey(keyPtr)
            keyPtr = NULL_PTR
        }
    }
    
    fun finalize() {
        close()
    }

    override fun isEncryptionSupported() = keyPtr != NULL_PTR

    override fun isDecryptionSupported() = keyPtr != NULL_PTR

    override fun isEncrypted(data: ByteArray) = ToxNative.isEncrypted(data)

    override fun encryptData(data: ByteArray): ByteArray? {
        return if (isEncryptionSupported()) {
            ToxNative.encryptWithPassKey(
                keyPtr = keyPtr,
                data = data,
            )
        } else {
            null
        }
    }

    override fun decryptData(data: ByteArray): ByteArray? {
        return if (isDecryptionSupported()) {
            ToxNative.decryptWithPassKey(
                keyPtr = keyPtr,
                cipher = data,
            )
        } else {
            null
        }
    }
}