package nand.apps.chat.engine.tox.dev

import nand.apps.chat.dev.DevCommandLogger
import nand.apps.chat.dev.error
import nand.apps.chat.engine.ChatEngineCommandHandler
import nand.apps.chat.engine.tox.NULL_PTR
import nand.apps.tox.ToxNative

class ToxDevCommandHandler(
    private val logger: DevCommandLogger,
) : ChatEngineCommandHandler {
    
    private var toxPtr = NULL_PTR
    
    fun onEngineInit(toxPtr: Long) {
        this.toxPtr = toxPtr
    }
    
    fun onEngineStop() {
        toxPtr = NULL_PTR
    }
    
    override fun onCommand(command: String, args: List<String>, fullArgs: String): Boolean {
        if (toxPtr == NULL_PTR) return false
        val groupNum = when {
            command.startsWith("tox_group_") || command.startsWith("tox_conference_") -> {
                args.getOrNull(0)?.toIntOrNull()
            }
            else -> null
        }
        return when (command) {
            "tox_group_is_connected" -> {
                val isConnected = ToxNative.isGroupConnected(
                    toxPtr = toxPtr,
                    groupNum = groupNum ?: logGroupNumberError(),
                )
                logger.print("$command = $isConnected")
                true
            }
            "tox_group_disconnect" -> {
                val isSuccessful = ToxNative.disconnectGroup(
                    toxPtr = toxPtr,
                    groupNum = groupNum ?: logGroupNumberError(),
                )
                logResult(command, fullArgs, isSuccessful)
            }
            "tox_group_reconnect" -> {
                val isSuccessful = ToxNative.reconnectGroup(
                    toxPtr = toxPtr,
                    groupNum = groupNum ?: logGroupNumberError(),
                )
                logResult(command, fullArgs, isSuccessful)
            }
            "tox_conference_is_audio_enabled" -> {
                val isEnabled = ToxNative.isConferenceAudioEnabled(
                    toxPtr = toxPtr,
                    groupNum = groupNum ?: logGroupNumberError(),
                )
                logger.print("$command = $isEnabled")
                true
            }
            else -> false
        }
    }
    
    private fun logGroupNumberError(): Nothing =
        error("Please specify a group number as the first argument.")
    
    private fun logResult(command: String, fullArgs: String, result: Boolean): Boolean {
        if (result) {
            logger.print("$command $fullArgs succeeded")
        } else {
            logger.error("$command $fullArgs failed")
        }
        return true
    }
}