package nand.apps.chat.engine.tox.friend

import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import nand.apps.chat.engine.tox.ToxFileEncryption
import nand.apps.chat.io.ChatDirectory
import nand.apps.chat.io.ChatFileSystem
import nand.apps.chat.log.getLogger
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.PersistedUserData
import nand.apps.chat.model.user.UserData
import nand.apps.chat.model.user.toPersistedUserData
import nand.apps.chat.model.user.toUserData
import nand.apps.chat.repo.ChatAvatarRepo
import okio.Path

class ToxFriendSaver(
    private val fileSystem: ChatFileSystem,
    private val avatarRepo: ChatAvatarRepo,
) {

    private lateinit var directory: ChatDirectory
    private var encryption: ToxFileEncryption? = null
    private val json = Json { ignoreUnknownKeys = true }
    private val logger = getLogger("ToxFriendSaver")

    fun init(
        dirName: String,
        encryption: ToxFileEncryption?,
    ) {
        directory = fileSystem.getProfileDirectory("$dirName/friends")
        this.encryption = encryption
    }

    fun save(friend: UserData): Boolean {
        val data = try {
            json.encodeToString(friend.toPersistedUserData()).encodeToByteArray()
        } catch (e: Exception) {
            logger.error("Failed to save friend data", e)
            return false
        }
        return directory.writeToFile(
            fileName = friend.uid.value,
            data = data,
            encryption = encryption,
        )
    }

    fun remove(friendUid: UserUid) {
        directory.removeFile(friendUid.value)
    }

    fun loadFriends() = directory.listFiles().mapNotNull { load(it) }

    private fun load(friendPath: Path): UserData? {
        val data = directory.readFromFile(
            fileName = friendPath.name,
            encryption = encryption,
        ) ?: return null
        val friend = try {
            json.decodeFromString<PersistedUserData>(data.decodeToString())
        } catch (e: Exception) {
            logger.error("Failed to read friend data: $friendPath", e)
            return null
        }
        return friend.toUserData(avatarRepo)
    }
}