package nand.apps.chat.image

import coil3.ImageLoader
import coil3.Uri
import coil3.decode.DataSource
import coil3.decode.ImageSource
import coil3.fetch.FetchResult
import coil3.fetch.Fetcher
import coil3.fetch.SourceFetchResult
import coil3.request.Options
import nand.apps.chat.io.*
import nand.apps.chat.repo.AVATAR_CACHE_DIRECTORY
import okio.Buffer
import okio.Path
import org.koin.core.Koin

class ChatAvatarImageFetcher(
    private val path: Path,
    private val options: Options,
    private val directory: ChatDirectory,
    private val encryption: ChatFileEncryption?,
) : Fetcher {

    override suspend fun fetch(): FetchResult? {
        val data = directory.readFromFile(
            fileName = path.name,
            encryption = encryption,
        ) ?: return null
        return SourceFetchResult(
            source = ImageSource(
                source = Buffer().apply { write(data) },
                fileSystem = options.fileSystem,
            ),
            mimeType = null,
            dataSource = DataSource.MEMORY,
        )
    }

    class Factory(koin: Koin) : Fetcher.Factory<Uri> {

        private val encryption: ChatEngineEncryption by koin.inject()
        private val fileSystem: ChatFileSystem by koin.inject()
        private val profilesDirectory by lazy {
            fileSystem.getProfileDirectory()
        }
        private val avatarDirectory by lazy {
            fileSystem.getCacheDirectory(AVATAR_CACHE_DIRECTORY)
        }
        
        override fun create(data: Uri, options: Options, imageLoader: ImageLoader): Fetcher? {
            val path = data.toFilePath()?.takeIf {
                avatarDirectory.contains(it) || profilesDirectory.contains(it)
            } ?: return null
            val directory = fileSystem.getDirectoryForFile(path.toString()) ?: return null
            return ChatAvatarImageFetcher(
                path = path,
                options = options,
                directory = directory,
                encryption = encryption,
            )
        }
    }
}