package nand.apps.chat.image.qrcode

import coil3.ImageLoader
import coil3.decode.DataSource
import coil3.decode.ImageSource
import coil3.fetch.FetchResult
import coil3.fetch.Fetcher
import coil3.fetch.SourceFetchResult
import coil3.request.Options
import nand.apps.chat.image.sizePx
import nand.apps.chat.image.x
import okio.Buffer
import kotlin.math.min

class QrCodeFetcher(
    private val input: QrCodeImageData,
    private val options: Options,
) : Fetcher {

    override suspend fun fetch(): FetchResult {
        // Calculate the minimum square size, so we don't have to downscale a gigantic PNG
        val base = input.toQrCode()
        val numSquares = base.computedSize / base.squareSize
        val dstSize = options.sizePx(base.computedSize x base.computedSize)
        val dstSquareSize = min(dstSize.width, dstSize.height) / numSquares
        val qrCode = input.toQrCode(dstSquareSize + 1)
        return SourceFetchResult(
            source = ImageSource(
                source = Buffer().apply { write(qrCode.renderToBytes()) },
                fileSystem = options.fileSystem,
            ),
            mimeType = null,
            dataSource = DataSource.MEMORY,
        )
    }

    class Factory : Fetcher.Factory<QrCodeImageData> {
        override fun create(data: QrCodeImageData, options: Options, imageLoader: ImageLoader): Fetcher {
            return QrCodeFetcher(data, options)
        }
    }
}