package nand.apps.chat.io

import kotlinx.serialization.Serializable
import okio.FileSystem
import okio.Path

@Serializable
data class ChatFileMetadata(
    val name: String,
    val path: String,
    val extension: String,
    val size: Long,
    val lastModified: Long,
    val isDirectory: Boolean,
    val fileCount: Int,
) {
    val isHidden = name.startsWith('.')
}

fun FileSystem.getFileMetadata(path: Path): ChatFileMetadata? {
    val metadata = try {
        metadata(path)
    } catch (ignore: Exception) {
        return null
    }
    return ChatFileMetadata(
        name = path.name,
        path = path.toString(),
        extension = if (metadata.isDirectory) "" else path.extension,
        size = metadata.size ?: 0,
        lastModified = metadata.lastModifiedAtMillis ?: -1,
        isDirectory = metadata.isDirectory,
        fileCount = if (metadata.isDirectory) listOrNull(path)?.size ?: 0 else -1,
    )
}