package nand.apps.chat.io

enum class ChatFileSortType {
    NAME,
    SIZE,
    DATE,
    EXTENSION,
}

private val DIRECTORY_COMPARATOR = Comparator<ChatFileMetadata> { file1, file2 ->
    when {
        file1.isDirectory && !file2.isDirectory -> -1
        !file1.isDirectory && file2.isDirectory -> 1
        else -> 0
    }
}

private fun getFileComparator(
    isReversed: Boolean,
    comparator: (ChatFileMetadata, ChatFileMetadata) -> Int,
): Comparator<ChatFileMetadata> = Comparator { file1, file2 ->
    val dirComparator = DIRECTORY_COMPARATOR.compare(file1, file2)
    when {
        dirComparator != 0 -> dirComparator
        isReversed -> comparator(file2, file1)
        else -> comparator(file1, file2)
    }
}

fun getFileComparator(
    type: ChatFileSortType,
    isReversed: Boolean,
): Comparator<ChatFileMetadata> = when (type) {
    ChatFileSortType.NAME -> getFileComparator(isReversed) { file1, file2 ->
        file1.name.compareTo(file2.name)
    }
    ChatFileSortType.EXTENSION -> getFileComparator(isReversed) { file1, file2 ->
        file1.extension.compareTo(file2.extension)
    }
    ChatFileSortType.SIZE -> getFileComparator(isReversed) { file1, file2 ->
        if (file1.isDirectory) {
            file1.fileCount.compareTo(file2.fileCount)
        } else {
            file1.size.compareTo(file2.size)
        }
    }
    ChatFileSortType.DATE -> getFileComparator(isReversed) { file1, file2 ->
        file1.lastModified.compareTo(file2.lastModified)
    }
}