package nand.apps.chat.io

import nand.apps.chat.model.user.ChatProfileData
import okio.FileSystem
import okio.Path.Companion.toPath
import okio.openZip

class ChatFileSystem(
    private val fileSystem: FileSystem,
    val paths: ChatFilePaths,
) {

    fun getUserDataDirectory(path: String = "") = ChatDirectory(
        path = paths.userDataPath.resolve(path),
        fileSystem = fileSystem,
    )
    
    fun getCacheDirectory(path: String = "") = ChatDirectory(
        path = paths.cachePath.resolve(path),
        fileSystem = fileSystem,
    )

    fun getConfigDirectory(path: String = "") = ChatDirectory(
        path = paths.configPath.resolve(path),
        fileSystem = fileSystem,
    )
    
    fun getTempDirectory(path: String = "") = getCacheDirectory(".tmp/$path")
    
    fun getProfileDirectory(path: String = "") = getUserDataDirectory("profiles/$path")

    fun getProfileDirectory(profile: ChatProfileData, path: String = "") =
        getProfileDirectory("${profile.username}/$path")
    
    fun getDirectory(directoryPath: String) = ChatDirectory(
        path = directoryPath.toPath(),
        fileSystem = fileSystem,
    )
    
    fun getDirectoryForFile(filePath: String) = filePath.toPath().parent?.let {
        ChatDirectory(
            path = it,
            fileSystem = fileSystem,
        )
    }
    
    fun getDirectoryForZip(zipPath: String): ChatDirectory? {
        return try {
            val zipSystem = fileSystem.openZip(zipPath.toPath())
            ChatDirectory(
                path = "/".toPath(),
                fileSystem = zipSystem,
            )
        } catch (ignore: Throwable) {
            null
        }
    }
    
    fun exists(path: String) = fileSystem.exists(path.toPath())
    
    fun deleteFile(path: String): Boolean {
        return try {
            fileSystem.delete(path.toPath())
            true
        } catch (ignore: Exception) {
            false
        }
    }
    
    fun getSize(path: String) = fileSystem.metadataOrNull(path.toPath())?.size ?: 0L
    
    fun isDirectory(path: String) = fileSystem.isValidDirectory(path)
    
    fun makeDirectoriesForFile(path: String) = path.toPath().parent?.let {
        fileSystem.createDirectories(it)
    }
}