package nand.apps.chat.log

import kotlin.time.TimeSource.Monotonic.markNow

interface AppLogger {
    
    fun verbose(message: String)
    
    fun debug(message: String)
    
    fun warn(message: String, exception: Exception? = null)
    
    fun error(message: String, exception: Exception? = null)
}

expect fun getAppLogger(tag: String): AppLogger

fun getLogger(tag: String): AppLogger = ChatLogger(getAppLogger(tag))

fun <T> AppLogger.measureTime(label: String, block: () -> T): T {
    val mark = markNow()
    val result = block()
    val millis = mark.elapsedNow().inWholeMilliseconds
    debug("$label took $millis ms")
    return result
}