package nand.apps.chat.model.call

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.CallEnd
import androidx.compose.material.icons.filled.PhoneInTalk
import androidx.compose.material.icons.filled.PhonePaused
import nand.apps.chat.model.event.*
import nand.apps.chat.model.settings.shortcut.ShortcutAction
import nand.apps.chat.model.uid.ContactUid
import seers.composeapp.generated.resources.*

enum class CallAction {
    ANSWER,
    DECLINE,
    HANG_UP,
    PAUSE,
    RESUME,
    ;
    
    val label get() = when (this) {
        ANSWER -> Res.string.answer_call
        DECLINE -> Res.string.decline_call
        HANG_UP -> Res.string.end_call
        PAUSE -> Res.string.pause_call
        RESUME -> Res.string.resume_call
    }
    
    val shortLabel get() = when (this) {
        ANSWER -> Res.string.answer
        DECLINE -> Res.string.decline
        HANG_UP -> Res.string.hang_up
        PAUSE -> Res.string.pause
        RESUME -> Res.string.resume
    }
    
    val icon get() = when (this) {
        ANSWER,
        RESUME -> Icons.Default.PhoneInTalk
        DECLINE,
        HANG_UP -> Icons.Default.CallEnd
        PAUSE -> Icons.Default.PhonePaused
    }
    
    val shortcut get() = when (this) {
        ANSWER -> ShortcutAction.START_ANSWER_CALL
        RESUME,
        PAUSE -> ShortcutAction.PAUSE_RESUME_CALL
        DECLINE,
        HANG_UP -> ShortcutAction.END_CALL
    }
    
    fun getEvent(contactUid: ContactUid) = when (this) {
        ANSWER -> AnswerCallEvent(
            contactUid = contactUid,
            isAudioEnabled = true,
            isVideoEnabled = false, // TODO - Support video calling
        )
        DECLINE,
        HANG_UP -> EndCallEvent(contactUid)
        PAUSE -> PauseCallEvent(contactUid)
        RESUME -> ResumeCallEvent(contactUid)
    }
}

fun CallStatus.getActions() = when (this) {
    CallStatus.INCOMING,
    CallStatus.VOICEMAIL -> listOf(CallAction.ANSWER, CallAction.DECLINE)
    CallStatus.OUTGOING -> listOf(CallAction.HANG_UP)
    CallStatus.ACTIVE -> listOf(CallAction.PAUSE, CallAction.HANG_UP)
    CallStatus.PAUSED -> listOf(CallAction.RESUME, CallAction.HANG_UP)
    else -> emptyList()
}