package nand.apps.chat.model.event

import nand.apps.chat.model.contact.ChatContact
import nand.apps.chat.model.file.ChatFileControl
import nand.apps.chat.model.file.ChatFileTransfer
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.group.ChatGroupInvite
import nand.apps.chat.model.group.ChatGroupRole
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.settings.SettingsData
import nand.apps.chat.model.uid.ContactUid
import nand.apps.chat.model.user.FriendRequest
import nand.apps.chat.model.user.UserData
import okio.Path

sealed interface ChatEvent

data object AppLoadedEvent : ChatEvent

data class AppFocusChangedEvent(
    val isFocused: Boolean = false,
) : ChatEvent

data class RequestAppFocusEvent(
    val toggle: Boolean = false,
) : ChatEvent

data object LoginCompleteEvent : ChatEvent

data object LogoutEvent: ChatEvent

data class OpenConversationEvent(
    val contact: ChatContact,
) : ChatEvent

data object ExitConversationEvent : ChatEvent

data class SendMessageEvent(
    val conversationUid: ContactUid,
    val text: String,
) : ChatEvent

data class AcceptFriendRequestEvent(
    val request: FriendRequest,
) : ChatEvent

data class DeclineFriendRequestEvent(
    val request: FriendRequest,
) : ChatEvent

data class SendGroupInviteEvent(
    val friend: UserData,
    val group: ChatGroupData,
) : ChatEvent

data class AcceptGroupInviteEvent(
    val invite: ChatGroupInvite,
) : ChatEvent

data class DeclineGroupInviteEvent(
    val invite: ChatGroupInvite,
) : ChatEvent

data class RemoveFriendEvent(
    val friend: UserData,
) : ChatEvent

data class RemoveGroupEvent(
    val group: ChatGroupData,
    val partingMessage: String? = null,
) : ChatEvent

data class RenameGroupEvent(
    val group: ChatGroupData,
    val title: String,
) : ChatEvent

data class SetGroupDescriptionEvent(
    val group: ChatGroupData,
    val description: String,
) : ChatEvent

data class SetGroupPasswordEvent(
    val group: ChatGroupData,
    val password: String?,
) : ChatEvent

data class SetGroupNicknameEvent(
    val group: ChatGroupData,
    val nickname: String,
) : ChatEvent

data class SetGroupRoleEvent(
    val group: ChatGroupData,
    val peer: UserData,
    val role: ChatGroupRole,
) : ChatEvent

data class KickGroupPeerEvent(
    val group: ChatGroupData,
    val peer: UserData,
) : ChatEvent

data class SetGroupPeerBlockedEvent(
    val group: ChatGroupData,
    val peer: UserData,
    val block: Boolean,
) : ChatEvent

data class JoinGroupEvent(
    val group: ChatGroupData,
) : ChatEvent

data class FileTransferControlEvent(
    val fileTransfer: ChatFileTransfer,
    val control: ChatFileControl,
) : ChatEvent

data class OpenFileEvent(
    val fileTransfer: ChatFileTransfer,
) : ChatEvent

data object OpenFileBrowserEvent : ChatEvent

data object ReloadFileBrowserEvent : ChatEvent

data class AnswerCallEvent(
    val contactUid: ContactUid,
    val isAudioEnabled: Boolean,
    val isVideoEnabled: Boolean,
) : ChatEvent

data class EndCallEvent(
    val contactUid: ContactUid,
) : ChatEvent

data class PauseCallEvent(
    val contactUid: ContactUid,
) : ChatEvent

data class ResumeCallEvent(
    val contactUid: ContactUid,
) : ChatEvent

data class UpdateMessageEvent(
    val message: ChatMessage,
    val localOnly: Boolean = true,
) : ChatEvent

data class PromptEditMessageEvent(
    val message: ChatMessage,
) : ChatEvent

data class PromptReactMessageEvent(
    val message: ChatMessage,
) : ChatEvent

data class ReactMessageEvent(
    val message: ChatMessage,
    val reaction: String,
) : ChatEvent

data class DeleteMessageEvent(
    val message: ChatMessage,
    val localOnly: Boolean,
) : ChatEvent

data class ClearMessagesEvent(
    val conversationUid: ContactUid,
) : ChatEvent

data class MuteContactEvent(
    val contactUid: ContactUid,
    val isMuted: Boolean,
) : ChatEvent

data class UpdateSettingsEvent(
    val settings: SettingsData,
) : ChatEvent

data class TakePhoto(
    val path: Path,
) : ChatEvent

data class TakePhotoQrCode(
    val code: String,
) : ChatEvent

data class ErrorDialogEvent(
    val message: String,
) : ChatEvent