package nand.apps.chat.model.group

import nand.apps.chat.model.avatar.ChatGroupAvatar
import nand.apps.chat.model.contact.ChatContact
import nand.apps.chat.model.uid.GroupUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.UserData

data class ChatGroupData(
    override val uid: GroupUid,
    val selfUid: UserUid,
    val title: String = "",
    val description: String = "",
    val type: ChatGroupType = ChatGroupType.TEXT_ONLY,
    val visibility: ChatGroupVisibility = ChatGroupVisibility.PRIVATE,
    val password: String? = null,
    val isOnline: Boolean = false,
    val peers: Map<UserUid, UserData> = emptyMap(),
    val blocked: Set<UserUid> = emptySet(),
    val features: Set<ChatGroupFeature> = emptySet(),
    val error: ChatGroupErrorType? = null,
    override val isMuted: Boolean = false,
) : ChatContact {

    val numPeersOnline = peers.count { it.value.isOnline }
    val avatar = ChatGroupAvatar(title)
    
    val isTitleEditable = ChatGroupFeature.EDIT_TITLE in features
    val isDescEditable = ChatGroupFeature.EDIT_DESCRIPTION in features
    val hasRoles = ChatGroupFeature.ROLES in features
    
    val selfRole = peers[selfUid]?.role ?: ChatGroupRole.USER
    
    fun findSelf(fallback: UserData) = peers[selfUid] ?: fallback
    
    fun isPeerBlocked(peerUid: UserUid) = peerUid != selfUid && peerUid in blocked
    
    fun setPeerBlock(peerUid: UserUid, block: Boolean): ChatGroupData {
        if (peerUid == selfUid) return this
        return copy(
            blocked = when {
                block -> blocked + peerUid
                else -> blocked - peerUid
            },
        )
    }

    override fun toString() = "$uid $title"
}