package nand.apps.chat.model.group

import kotlinx.serialization.Serializable
import nand.apps.chat.io.ChatAvatarProvider
import nand.apps.chat.model.uid.GroupUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.PersistedUserData
import nand.apps.chat.model.user.toPersistedUserData
import nand.apps.chat.model.user.toUserData

@Serializable
data class PersistedChatGroupData(
    val uid: String,
    val selfUid: String,
    val title: String,
    val description: String,
    val type: ChatGroupType,
    val visibility: ChatGroupVisibility,
    val peers: List<PersistedUserData>,
    val password: String? = null,
    val blocked: List<String>? = null,
    val isMuted: Boolean = false,
)

fun ChatGroupData.toPersistedGroupData() = PersistedChatGroupData(
    uid = uid.value,
    selfUid = selfUid.value,
    title = title,
    description = description,
    type = type,
    visibility = visibility,
    password = password,
    peers = peers.values.map { it.toPersistedUserData() },
    blocked = blocked.takeIf { it.isNotEmpty() }?.map { it.value },
    isMuted = isMuted,
)

fun PersistedChatGroupData.toGroupData(
    avatarProvider: ChatAvatarProvider? = null,
) = ChatGroupData(
    uid = GroupUid(uid),
    selfUid = UserUid(selfUid),
    title = title,
    description = description,
    type = type,
    visibility = visibility,
    password = password,
    peers = peers.associate { UserUid(it.uid) to it.toUserData(avatarProvider) },
    blocked = blocked?.map { UserUid(it) }?.toSet().orEmpty(),
    isMuted = isMuted,
)