package nand.apps.chat.model.message

import kotlinx.serialization.Serializable
import nand.apps.chat.model.file.ChatFileTransfer
import nand.apps.chat.model.file.PersistedChatFileTransfer
import nand.apps.chat.model.file.toPersistedFileTransfer
import nand.apps.chat.model.uid.UserUid

@Serializable
data class PersistedChatMessage(
    val senderUid: String,
    val text: String,
    val timestamp: Long,
    val type: ChatMessageType = ChatMessageType.NORMAL,
    val isDelivered: Boolean = false,
    val isEdited: Boolean = false,
    val isUnread: Boolean = false,
    val reactions: Map<UserUid, Set<String>>? = null,
    val fileTransfer: PersistedChatFileTransfer? = null,
)

fun ChatMessage.toPersistedMessage(
    fileTransfer: ChatFileTransfer?,
) = PersistedChatMessage(
    senderUid = senderUid.value,
    text = text,
    timestamp = timestamp,
    type = type,
    isDelivered = isDelivered,
    isEdited = isEdited,
    isUnread = isUnread,
    reactions = reactions.takeIf { it.isNotEmpty() },
    fileTransfer = fileTransfer?.toPersistedFileTransfer(),
)