package nand.apps.chat.model.user

import kotlinx.serialization.Serializable
import nand.apps.chat.io.ChatAvatarProvider
import nand.apps.chat.model.group.ChatGroupRole
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.time.currentTimeMillis

@Serializable
data class PersistedUserData(
    val uid: String,
    val name: String,
    val statusMessage: String = "",
    val lastOnline: Long = -1L,
    val role: ChatGroupRole = ChatGroupRole.USER,
    val isMuted: Boolean = false,
)

fun UserData.toPersistedUserData() = PersistedUserData(
    uid = uid.value,
    name = name,
    statusMessage = statusMessage,
    lastOnline = if (isOnline) currentTimeMillis() else lastOnline,
    role = role,
    isMuted = isMuted,
)

fun PersistedUserData.toUserData(
    avatarProvider: ChatAvatarProvider? = null,
): UserData {
    val uid = UserUid(uid)
    return UserData(
        uid = uid,
        name = name,
        statusMessage = statusMessage,
        lastOnline = lastOnline,
        role = role,
        isMuted = isMuted,
        avatarPath = avatarProvider?.getAvatarPath(uid),
    )
}