package nand.apps.chat.model.user

import nand.apps.chat.model.avatar.ChatUserAvatar
import nand.apps.chat.model.contact.ChatContact
import nand.apps.chat.model.group.ChatGroupRole
import nand.apps.chat.model.net.ConnectionProtocol
import nand.apps.chat.model.uid.UserUid
import okio.Path

data class UserData(
    override val uid: UserUid,
    val name: String = "",
    val avatarPath: Path? = null,
    val status: UserStatus = UserStatus.DEFAULT,
    val statusMessage: String = "",
    val lastOnline: Long = -1L,
    val protocol: ConnectionProtocol = ConnectionProtocol.NONE,
    val role: ChatGroupRole = ChatGroupRole.USER,
    val clientVersion: ChatClientVersion? = null,
    override val isMuted: Boolean = false,
) : ChatContact {
    
    val displayName = name.ifBlank { uid.toString() }
    val isOnline = protocol != ConnectionProtocol.NONE
    val avatar = ChatUserAvatar(
        username = name,
        path = avatarPath,
    )
    
    fun hasAuthorityOver(other: UserData) = role.hasAuthorityOver(other.role)

    override fun toString() = "$uid $name ($protocol)"
    
    companion object {
        // Should only be used as the initial value of a mutable state
        val BLANK = UserData(uid = UserUid("TEMP"))
        
        val CONTACT_LIST_COMPARATOR = Comparator<UserData> { f1, f2 ->
            val nameComp = f1.displayName.compareTo(f2.displayName, ignoreCase = true)
            val onlineComp = f2.isOnline.compareTo(f1.isOnline)
            val lastOnlineComp = f2.lastOnline.compareTo(f1.lastOnline)
            when {
                onlineComp != 0 -> onlineComp
                !f1.isOnline && !f2.isOnline && lastOnlineComp != 0 -> lastOnlineComp
                else -> nameComp
            }
        }
    }
}