package nand.apps.chat.text

import androidx.compose.ui.text.input.OffsetMapping
import androidx.compose.ui.util.fastForEach

class IndexOffsetMapping(
    indices: List<Pair<Int, Int>>,
    indexMap: Map<Int, Int> = indices.toMap(),
    previousMapping: OffsetMapping = OffsetMapping.Identity,
) : OffsetMapping {

    private val srcToDst: Map<Int, Int>
    private val dstToSrc: Map<Int, Int>
    
    init {
        when {
            previousMapping is IndexOffsetMapping -> {
                srcToDst = LinkedHashMap(previousMapping.srcToDst)
                previousMapping.srcToDst.forEach { (src, dst) ->
                    srcToDst[src] = indexMap[dst] ?: missingMapping(dst)
                }
                dstToSrc = indices.associate {
                    it.second to (previousMapping.dstToSrc[it.first] ?: missingMapping(it.first))
                }
            }
            previousMapping === OffsetMapping.Identity -> {
                srcToDst = indexMap
                dstToSrc = indices.associate { it.second to it.first }
            }
            else -> {
                srcToDst = mutableMapOf()
                dstToSrc = mutableMapOf()
                indices.fastForEach { (_, dst) ->
                    val prevSrc = previousMapping.transformedToOriginal(dst)
                    srcToDst[prevSrc] = dst
                    dstToSrc[dst] = prevSrc
                }
            }
        }
    }

    override fun originalToTransformed(offset: Int) =
        srcToDst[offset] ?: missingOffset(offset)

    override fun transformedToOriginal(offset: Int) =
        dstToSrc[offset] ?: missingOffset(offset)
    
    private fun missingMapping(index: Int): Nothing = error("Missing mapping $index")
    
    private fun missingOffset(offset: Int): Nothing = error("Missing offset $offset")
}