package nand.apps.chat.text

import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.TransformedText
import androidx.compose.ui.text.style.TextDecoration
import nand.apps.chat.util.hasUrlAnnotations

// TODO - Support headers, masked links, lists, and code blocks
// TODO - Support tag preview in input

private val TAGS_FIRST_PASS = mapOf(
    "**" to SpanStyle(fontWeight = FontWeight.Bold),
    "__" to SpanStyle(textDecoration = TextDecoration.Underline),
    "~~" to SpanStyle(textDecoration = TextDecoration.LineThrough),
    "`" to SpanStyle(fontFamily = FontFamily.Monospace),
)

private val TAGS_SECOND_PASS = mapOf(
    "*" to SpanStyle(fontStyle = FontStyle.Italic),
    "_" to SpanStyle(fontStyle = FontStyle.Italic),
)

private data class MarkdownTag(
    val value: String,
    val index: Int,
)

fun TransformedText.addMarkdownStyles(): TransformedText {
    return applyMarkdown(TAGS_FIRST_PASS).applyMarkdown(TAGS_SECOND_PASS)
}

private fun TransformedText.applyMarkdown(
    tagMap: Map<String, SpanStyle>,
): TransformedText {
    val src = this
    val lastDelim = mutableMapOf<String, Int>()
    val tags = mutableListOf<MarkdownTag>()
    var start = 0
    while (start < text.length) {
        text.findAnyOf(tagMap.keys, start)?.let { (index, delimiter) ->
            val lastIndex = lastDelim.remove(delimiter)
            if (lastIndex != null) {
                if (index - lastIndex > delimiter.length && !text.hasUrlAnnotations(lastIndex, index)) {
                    tags += MarkdownTag(delimiter, lastIndex)
                    tags += MarkdownTag(delimiter, index)
                }
            } else {
                lastDelim[delimiter] = index
            }
            start = index + delimiter.length
        } ?: break
    }
    start = 0
    lastDelim.clear()
    return buildTransformedText(src) {
        val sorted = tags.sortedBy { it.index }
        for (tag in sorted) {
            val style = tagMap[tag.value] ?: continue
            appendSource(start, tag.index)

            val lastIndex = lastDelim.remove(tag.value)
            if (lastIndex != null) {
                addStyle(
                    style = style,
                    start = lastIndex,
                    end = tag.index + indexOffset,
                )
            } else {
                lastDelim[tag.value] = tag.index + indexOffset
            }

            val end = tag.index + tag.value.length
            start = end
        }
        appendSource(start, text.length)
    }
}