package nand.apps.chat.time

import kotlinx.datetime.LocalDateTime
import kotlinx.datetime.format
import kotlinx.datetime.format.DateTimeFormat
import kotlinx.datetime.format.DateTimeFormatBuilder.WithDateTime
import kotlinx.datetime.format.Padding
import kotlinx.datetime.format.char
import nand.apps.chat.repo.SettingsRepo

class DateTimeFormatter(
    private val settingsRepo: SettingsRepo,
) {
    
    private val settings get() = settingsRepo.settings.generalSettings
    
    private lateinit var timeFormat: DateTimeFormat<LocalDateTime>
    private lateinit var dateFormat: DateTimeFormat<LocalDateTime>
    
    private var timeFormatData: TimestampFormat? = null
    private var dateFormatData: DateFormat? = null

    fun formatDate(timestampMs: Long): String {
        if (settings.dateFormat != dateFormatData) {
            dateFormatData = settings.dateFormat
            dateFormat = LocalDateTime.Format { createDateFormatter(settings.dateFormat) }
        }
        return getLocalInstant(timestampMs).format(dateFormat)
    }
    
    fun formatTime(timestampMs: Long): String {
        if (settings.timeFormat != timeFormatData) {
            timeFormatData = settings.timeFormat
            timeFormat = LocalDateTime.Format {
                if (settings.timeFormat.includeAmPm) {
                    amPmHour(Padding.NONE)
                } else {
                    hour(Padding.NONE)
                }
                char(':')
                minute()
                if (settings.timeFormat.includeSeconds) {
                    char(':')
                    second()
                }
                if (settings.timeFormat.includeAmPm) {
                    char(' ')
                    amPmMarker("AM", "PM")
                }
            }
        }
        return getLocalInstant(timestampMs).format(timeFormat)
    }
    
    fun formatDateTime(timestampMs: Long) =
        formatDate(timestampMs) + " " + formatTime(timestampMs)
    
    private fun WithDateTime.createDateFormatter(
        dateFormat: DateFormat,
    ) {
        val separator = if (dateFormat.useDashSeparator) '-' else '/'
        when (dateFormat.order) {
            DateFormatOrder.YMD -> {
                year(dateFormat)
                char(separator)
                monthNumber()
                char(separator)
                dayOfMonth()
            }
            DateFormatOrder.MDY -> {
                monthNumber()
                char(separator)
                dayOfMonth()
                char(separator)
                year(dateFormat)
            }
            DateFormatOrder.DMY -> {
                dayOfMonth()
                char(separator)
                monthNumber()
                char(separator)
                year(dateFormat)
            }
        }
    }

    private fun WithDateTime.year(dateFormat: DateFormat) = if (dateFormat.useFullYear) {
        year(Padding.NONE)
    } else {
        yearTwoDigits(2000)
    }
}