package nand.apps.chat.time

import kotlinx.datetime.*
import nand.apps.chat.util.formatPlural

const val MINUTE_MILLIS = 60 * 1000L
const val HOUR_MILLIS = 60 * MINUTE_MILLIS
const val DAY_MILLIS = 24 * HOUR_MILLIS

fun currentTimeMillis() = Clock.System.now().toEpochMilliseconds()

fun getLocalInstant(timestampMs: Long) =
    Instant.fromEpochMilliseconds(timestampMs).toLocalDateTime(TimeZone.currentSystemDefault())

fun formatRelativeDate(timestampMs: Long): String {
    val curInstant = Clock.System.now()
    val tsInstant = Instant.fromEpochMilliseconds(timestampMs)
    val period = if (curInstant > tsInstant) {
        tsInstant.periodUntil(curInstant, TimeZone.currentSystemDefault())
    } else {
        curInstant.periodUntil(tsInstant, TimeZone.currentSystemDefault())
    }
    return when {
        period.seconds == 0 -> "Just now"
        else -> period.run {
            val weeks = days / 7
            when {
                years > 0 -> "$years " + formatPlural("year", years)
                months > 0 -> "$months " + formatPlural("month", months)
                weeks >= 2 -> "$weeks " + formatPlural("week", weeks)
                days > 0 -> "$days " + formatPlural("day", days)
                hours > 0 -> "$hours " + formatPlural("hour", hours)
                minutes > 0 -> "$minutes " + formatPlural("minute", minutes)
                else -> "$seconds " + formatPlural("second", seconds)
            } + " " + if (curInstant > tsInstant) "ago" else "from now"
        }
    }
}