package nand.apps.chat.ui.button

import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.size
import androidx.compose.material.Icon
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.input.pointer.PointerIcon
import androidx.compose.ui.input.pointer.pointerHoverIcon
import androidx.compose.ui.unit.Dp
import nand.apps.chat.ui.modifier.appendIf
import nand.apps.chat.ui.theme.ChatTheme
import androidx.compose.material.IconButton as CoreIconButton

@Composable
fun IconButton(
    icon: ImageVector,
    contentDescription: String?,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
    size: Dp = ChatTheme.dimens.iconSmall,
    color: Color = ChatTheme.colors.icon,
    isEnabled: Boolean = true,
    interactionSource: MutableInteractionSource = remember { MutableInteractionSource() },
) {
    CoreIconButton(
        onClick = onClick,
        modifier = modifier.appendIf(isEnabled) {
            pointerHoverIcon(PointerIcon.Hand)
        },
        enabled = isEnabled,
        interactionSource = interactionSource,
    ) {
        Icon(
            imageVector = icon,
            contentDescription = contentDescription,
            tint = color,
            modifier = Modifier.size(size),
        )
    }
}