package nand.apps.chat.ui.button

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.size
import androidx.compose.material.Button
import androidx.compose.material.ButtonDefaults
import androidx.compose.material.CircularProgressIndicator
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.pointer.PointerIcon
import androidx.compose.ui.input.pointer.pointerHoverIcon
import androidx.compose.ui.text.TextStyle
import nand.apps.chat.ui.modifier.appendIf
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun SimpleTextButton(
    text: String,
    onClick: () -> Unit,
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    isLoading: Boolean = false,
    style: TextStyle = ChatTheme.typography.body1,
    backgroundColor: Color = ChatTheme.colors.primary,
    textColor: Color = ChatTheme.colors.onPrimary,
) = Button(
    modifier = modifier.appendIf(isEnabled) {
        pointerHoverIcon(PointerIcon.Hand)
    },
    onClick = onClick,
    colors = ButtonDefaults.buttonColors(
        backgroundColor = backgroundColor,
        contentColor = textColor,
    ),
    enabled = isEnabled,
) {
    Box(
        contentAlignment = Alignment.Center,
    ) {
        TextLine(
            text = text,
            style = style,
            color = when {
                isLoading -> Color.Transparent
                isEnabled -> textColor
                else -> textColor.copy(alpha = 0.5f)
            },
        )
        if (isLoading) {
            CircularProgressIndicator(
                modifier = Modifier.size(ChatTheme.dimens.iconSmall),
            )
        }
    }
}