package nand.apps.chat.ui.chat.message

import androidx.compose.foundation.layout.Box
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.font.FontWeight
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.message.ChatMessageType
import nand.apps.chat.text.buildTransformedText
import nand.apps.chat.ui.provider.LocalDateTimeFormatProvider
import nand.apps.chat.ui.provider.isPortraitMode
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun GroupEventMessageRow(
    message: ChatMessage,
    displayName: String,
    modifier: Modifier = Modifier,
) {
    val dateTimeFormatter = LocalDateTimeFormatProvider.current
    val timestampFormatted = remember(message.timestamp) {
        dateTimeFormatter.formatTime(message.timestamp)
    }
    val textRes = stringResource(
        when (message.type) {
            ChatMessageType.PEER_ONLINE -> Res.string.group_event_peer_online
            ChatMessageType.PEER_OFFLINE -> Res.string.group_event_peer_offline
            else -> return
        }
    )
    val text = remember(displayName, message) {
        buildTransformedText(textRes) {
            val nameIdx = textRes.indexOf("%1\$s")
            appendSource(0, nameIdx)
            pushStyle(SpanStyle(fontWeight = FontWeight.Bold))
            appendOther(displayName)
            pop()
            appendSource(nameIdx + 4, textRes.length)
        }
    }
    val textStyle = if (isPortraitMode()) ChatTheme.typography.body2 else ChatTheme.typography.caption
    Box(
        modifier = modifier,
    ) {
        TextLine(
            text = text.text,
            style = textStyle,
            color = ChatTheme.colors.onBackground,
        )
        TextLine(
            text = timestampFormatted,
            style = ChatTheme.typography.caption,
            color = ChatTheme.colors.onBackground2,
            modifier = Modifier.align(Alignment.CenterEnd),
        )
    }
}