package nand.apps.chat.ui.chat.message

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.AddReaction
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.Edit
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import nand.apps.chat.model.event.*
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.group.ChatGroupFeature
import nand.apps.chat.model.group.ChatGroupRole
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.chat.screen.ChatScreenState
import nand.apps.chat.ui.dialog.DeleteChatMessageDialog
import nand.apps.chat.ui.menu.ActionMenuArea
import nand.apps.chat.ui.menu.ActionMenuItem
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.util.getStringBlocking
import seers.composeapp.generated.resources.*

@Composable
fun MessageActionMenu(
    message: ChatMessage,
    screenState: ChatScreenState,
    group: ChatGroupData?,
    user: UserData?,
    displayName: String,
    modifier: Modifier = Modifier,
    content: @Composable () -> Unit,
) {
    val eventHandler = LocalEventHandler.current
    val settings = LocalSettingsProvider.current.chatSettings
    var isDeleteDialogVisible by rememberSaveable { mutableStateOf(false) }

    val isReactionsEnabled = settings.isMessageReactionsEnabled && (group == null || ChatGroupFeature.REACT_MESSAGES in group.features)
    val isSelfObserver = group?.selfRole == ChatGroupRole.OBSERVER
    val isMessageEditable = settings.isMessageEditsEnabled &&
            message.isEditable() &&
            message.senderUid == screenState.selfUid &&
            (group == null || ChatGroupFeature.EDIT_MESSAGES in group.features && !isSelfObserver)
    
    ActionMenuArea(
        isEnabled = screenState.isConversationOnline && (isReactionsEnabled || isMessageEditable),
        items = {
            mutableListOf<ActionMenuItem>().apply {
                if (isReactionsEnabled) {
                    this += ActionMenuItem(
                        icon = Icons.Default.AddReaction,
                        label = getStringBlocking(Res.string.add_reaction),
                        onClick = { eventHandler += PromptReactMessageEvent(message) },
                    )
                }
                if (isMessageEditable) {
                    this += ActionMenuItem(
                        icon = Icons.Default.Edit,
                        label = getStringBlocking(Res.string.edit_message),
                        onClick = {
                            eventHandler += if (message.isEditable()) {
                                PromptEditMessageEvent(message)
                            } else {
                                ErrorDialogEvent(getStringBlocking(Res.string.message_edit_threshold_expired))
                            }
                        },
                    )
                    this += ActionMenuItem(
                        icon = Icons.Default.Delete,
                        label = getStringBlocking(Res.string.delete_message),
                        onClick = {
                            if (message.isEditable()) {
                                isDeleteDialogVisible = true
                            } else {
                                eventHandler += ErrorDialogEvent(getStringBlocking(Res.string.message_edit_threshold_expired))
                            }
                        },
                    )
                }
            }
        },
        modifier = modifier,
        content = content,
    )
    when {
        isDeleteDialogVisible && user != null -> DeleteChatMessageDialog(
            message = message,
            displayName = displayName,
            user = user,
            group = group,
            onDismiss = { isDeleteDialogVisible = false },
        )
    }
}