package nand.apps.chat.ui.chat.message

import androidx.compose.animation.*
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.text.style.TextAlign
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun MessageDateHeader(
    messageDate: String,
    currentDate: String,
    modifier: Modifier = Modifier,
) {
    val dateIndicatorShape = RoundedCornerShape(
        bottomStart = ChatTheme.dimens.spacingMedium,
        bottomEnd = ChatTheme.dimens.spacingMedium,
    )
    AnimatedVisibility(
        visible = messageDate != currentDate,
        enter = fadeIn() + slideInVertically(),
        exit = fadeOut() + slideOutVertically(),
        modifier = modifier
            .clip(dateIndicatorShape)
            .background(ChatTheme.colors.background),
    ) {
        TextLine(
            text = messageDate,
            style = ChatTheme.typography.body2,
            color = ChatTheme.colors.onBackground,
            textAlign = TextAlign.Center,
            modifier = Modifier
                .background(ChatTheme.colors.surface2)
                .padding(ChatTheme.dimens.spacingMedium)
        )
    }
}