package nand.apps.chat.ui.chat.message.reaction

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.padding
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.tooltip.TooltipArea
import nand.apps.chat.util.getStringBlocking
import seers.composeapp.generated.resources.*

@Composable
fun MessageReactionBadge(
    message: ChatMessage,
    reaction: String,
    count: Int,
    selfUid: UserUid,
    isLarge: Boolean,
    isSelfReaction: Boolean,
    isClickable: Boolean,
    onClick: () -> Unit,
    group: ChatGroupData? = null,
) = TooltipArea(
    isEnabled = group != null,
    tooltip = {
        val text = remember {
            var includesSelf = false
            val otherUids = message.reactions.mapNotNull { (uid, reactions) ->
                if (reaction in reactions) {
                    if (uid == selfUid) {
                        includesSelf = true
                        null
                    } else {
                        uid
                    }
                } else {
                    null
                }
            }
            val totalReactions = otherUids.size + if (includesSelf) 1 else 0
            val firstName = when {
                includesSelf -> getStringBlocking(Res.string.message_reaction_tooltip_self)
                else -> group.getName(otherUids.firstOrNull())
            }
            val secondName = when {
                includesSelf -> group.getName(otherUids.firstOrNull())
                else -> group.getName(otherUids.getOrNull(1))
            }
            when (totalReactions) {
                1 -> getStringBlocking(Res.string.message_reaction_tooltip_singular, firstName, reaction)
                2 -> getStringBlocking(Res.string.message_reaction_tooltip_couple, firstName, secondName, reaction)
                else -> getStringBlocking(Res.string.message_reaction_tooltip_multiple, firstName, totalReactions - 1, reaction)
            }
        }
        TextLine(
            text = text,
            style = ChatTheme.typography.caption,
            modifier = Modifier
                .background(
                    color = ChatTheme.colors.backgroundFloating,
                    shape = ChatTheme.shapes.small,
                )
                .padding(ChatTheme.dimens.spacingSmall),
        )
    },
) {
    Text(
        text = "$reaction $count",
        style = when {
            isLarge -> ChatTheme.typography.body1
            else -> ChatTheme.typography.body2
        },
        color = ChatTheme.colors.onBackground,
        modifier = Modifier
            .clip(ChatTheme.shapes.small)
            .background(if (isSelfReaction) ChatTheme.colors.primaryVariant else ChatTheme.colors.surface)
            .onClick(
                isEnabled = isClickable,
                onClick = onClick,
            )
            .padding(ChatTheme.dimens.spacingSmall),
    )
}

private fun ChatGroupData?.getName(uid: UserUid?) =
    this?.peers?.get(uid)?.displayName ?: getStringBlocking(Res.string.unknown_sender)