package nand.apps.chat.ui.chat.message.reaction

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.ExperimentalLayoutApi
import androidx.compose.foundation.layout.FlowRow
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import nand.apps.chat.model.event.ReactMessageEvent
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.message.MAX_REACTIONS_PER_USER
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.provider.isPortraitMode
import nand.apps.chat.ui.selection.DisableSelection
import nand.apps.chat.ui.theme.ChatTheme

@OptIn(ExperimentalLayoutApi::class)
@Composable
fun MessageReactionsRow(
    message: ChatMessage,
    selfUid: UserUid,
    modifier: Modifier = Modifier,
    group: ChatGroupData? = null,
    isLarge: Boolean = isPortraitMode(),
) = DisableSelection {
    val eventHandler = LocalEventHandler.current
    val reactions = remember(message.reactions) {
        mutableMapOf<String, Int>().apply {
            message.reactions.forEach { (_, reactions) ->
                reactions.forEach { this[it] = (this[it] ?: 0) + 1 }
            }
        }
    }
    val selfReactions = message.reactions[selfUid].orEmpty()
    val spacing = if (isLarge) ChatTheme.dimens.spacingMedium else ChatTheme.dimens.spacingSmall
    FlowRow(
        horizontalArrangement = Arrangement.spacedBy(spacing),
        verticalArrangement = Arrangement.spacedBy(spacing),
        modifier = modifier,
    ) {
        reactions.forEach { (reaction, count) ->
            MessageReactionBadge(
                message = message,
                group = group,
                reaction = reaction,
                count = count,
                selfUid = selfUid,
                isLarge = isLarge,
                isSelfReaction = reaction in selfReactions,
                isClickable = reaction in selfReactions || selfReactions.size < MAX_REACTIONS_PER_USER,
                onClick = {
                    eventHandler += ReactMessageEvent(
                        message = message,
                        reaction = reaction,
                    )
                },
            )
        }
    }
}