package nand.apps.chat.ui.chat.screen

import androidx.compose.animation.core.*
import androidx.compose.foundation.layout.*
import androidx.compose.material.Icon
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.WavingHand
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.text.style.TextAlign
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ChatScreenPlaceholder(
    state: ChatScreenState,
    modifier: Modifier = Modifier,
) {
    val isNewUser = state.friends.isEmpty() && state.groups.isEmpty()
    val infiniteTransition = rememberInfiniteTransition(label = "Waving hand")
    val rotation by infiniteTransition.animateFloat(
        initialValue = -10f,
        targetValue = 10f,
        animationSpec = infiniteRepeatable(tween(500, easing = LinearEasing), RepeatMode.Reverse),
        label = "Wave"
    )
    Box(
        contentAlignment = Alignment.Center,
        modifier = modifier,
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingSmall),
            modifier = Modifier.padding(ChatTheme.dimens.spacingLarge),
        ) {
            Icon(
                imageVector = Icons.Default.WavingHand,
                contentDescription = null,
                tint = ChatTheme.colors.icon,
                modifier = Modifier
                    .size(ChatTheme.dimens.iconXLarge)
                    .rotate(rotation),
            )
            Text(
                text = if (isNewUser) {
                    stringResource(Res.string.chat_screen_placeholder_title_new_user, APP_NAME)
                } else {
                    stringResource(Res.string.chat_screen_placeholder_title_welcome_back, state.self.name)
                },
                color = ChatTheme.colors.onSurface,
                style = ChatTheme.typography.h6,
                textAlign = TextAlign.Center,
            )
            Text(
                text = if (isNewUser) {
                    stringResource(Res.string.chat_screen_placeholder_add_contact)
                } else {
                    stringResource(Res.string.chat_screen_placeholder_select_contact)
                },
                color = ChatTheme.colors.onBackground,
                style = ChatTheme.typography.body2,
                textAlign = TextAlign.Center,
            )
        }
    }
}