package nand.apps.chat.ui.chat.screen

import nand.apps.chat.model.call.CallData
import nand.apps.chat.model.file.ChatFileTransfer
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.message.UnreadMessageCounts
import nand.apps.chat.model.uid.*
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.chat.message.MessageInputState
import nand.apps.chat.util.getStringBlocking
import seers.composeapp.generated.resources.*

data class ChatScreenState(
    val conversationUid: ContactUid? = null,
    val messages: List<ChatMessage> = emptyList(),
    val self: UserData = UserData.BLANK,
    val friends: Map<UserUid, UserData> = emptyMap(),
    val groups: Map<GroupUid, ChatGroupData> = emptyMap(),
    val unreadCounts: Map<ContactUid, UnreadMessageCounts> = emptyMap(),
    val fileTransfers: Map<FileTransferUid, ChatFileTransfer> = emptyMap(),
    val calls: Map<ContactUid, CallData> = emptyMap(),
    val callActivity: Map<ContactUid, Boolean> = emptyMap(),
    val inputState: MessageInputState = MessageInputState(),
) {
    
    val conversationUser = (conversationUid as? UserUid)?.let { friends[it] }
    val conversationGroup = (conversationUid as? GroupUid)?.let { groups[it] }

    val conversationTitle get() = when (conversationUid) {
        is UserUid -> conversationUser?.displayName
        is GroupUid -> conversationGroup?.title?.takeIf { it.isNotBlank() } ?: getStringBlocking(Res.string.group_untitled)
        is FriendRequestUid -> getStringBlocking(Res.string.friend_request)
        else -> null
    }
    
    val isConversationOnline = when (conversationUid) {
        is UserUid -> conversationUser?.isOnline == true
        is GroupUid -> conversationGroup?.isOnline == true
        else -> true
    }

    val selfUid get() = conversationGroup?.selfUid ?: self.uid

    val conversationCall = calls[conversationUid]

    fun getDisplayIndex(index: Int) = messages.size - index - 1
    
    fun findUser(message: ChatMessage) = when {
        message.isGroupMessage -> null
        message.senderUid == self.uid -> self
        message.conversationUid is GroupUid -> groups[message.conversationUid]?.let { group ->
            group.peers[message.senderUid]?.let { peer ->
                if (message.senderUid == group.selfUid) {
                    self
                } else {
                    friends[message.senderUid]
                }?.copy(
                    name = peer.name,
                    role = peer.role,
                    protocol = peer.protocol,
                ) ?: peer
            }
        }
        else -> friends[message.senderUid]
    }
    
    fun findGroup(message: ChatMessage) = when {
        message.conversationUid is GroupUid -> groups[message.conversationUid]
        else -> null
    }
}