package nand.apps.chat.ui.dialog

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.padding
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import nand.apps.chat.ui.text.SimpleTextCheckbox
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.Res
import seers.composeapp.generated.resources.dont_ask_again

@Composable
fun ConfirmationDialog(
    title: StringResource,
    message: String,
    onConfirm: (dontAskAgain: Boolean) -> Unit,
    onCancel: () -> Unit,
    modifier: Modifier = Modifier,
    isDontAskAgainEnabled: Boolean = false,
) {
    var dontAskAgain by rememberSaveable { mutableStateOf(false) }
    SimpleDialog(
        title = title,
        modifier = modifier,
        onConfirm = { onConfirm(dontAskAgain) },
        onCancel = onCancel,
    ) {
        Column {
            Text(
                text = message,
                style = ChatTheme.typography.body1,
                color = ChatTheme.colors.onBackground,
            )
            if (isDontAskAgainEnabled) {
                SimpleTextCheckbox(
                    text = stringResource(Res.string.dont_ask_again),
                    isChecked = dontAskAgain,
                    onCheckedChange = { dontAskAgain = it },
                    modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
                )
            }
        }
    }
}