package nand.apps.chat.ui.dialog

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.window.Dialog
import nand.apps.chat.model.user.ChatClientVersion
import nand.apps.chat.model.user.UserData
import nand.apps.chat.time.formatRelativeDate
import nand.apps.chat.ui.avatar.ChatAvatarBox
import nand.apps.chat.ui.avatar.ChatAvatarSize
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ContactProfileDialog(
    user: UserData,
    isSelf: Boolean,
    onDismiss: () -> Unit,
) = Dialog(
    onDismissRequest = onDismiss,
) {
    Column(
        horizontalAlignment = Alignment.CenterHorizontally,
        modifier = Modifier
            .background(
                color = ChatTheme.colors.background,
                shape = ChatTheme.shapes.small,
            )
            .padding(ChatTheme.dimens.spacingXLarge),
    ) {
        ChatAvatarBox(
            avatar = user.avatar,
            size = ChatAvatarSize.XLARGE,
            status = user.status,
            isOnline = user.isOnline,
        )
        TextLine(
            text = user.displayName,
            style = ChatTheme.typography.body1,
            textAlign = TextAlign.Center,
        )
        if (user.statusMessage.isNotBlank()) {
            TextLine(
                text = user.statusMessage,
                style = ChatTheme.typography.body2,
                color = ChatTheme.colors.onBackground,
                textAlign = TextAlign.Center,
            )
        }
        TextLine(
            text = when {
                isSelf -> ChatClientVersion().toString()
                user.isOnline -> user.clientVersion?.toString() ?: stringResource(Res.string.unknown_client)
                user.lastOnline > 0 -> stringResource(Res.string.last_online_timestamp, formatRelativeDate(user.lastOnline))
                else -> ""
            },
            style = ChatTheme.typography.caption,
            color = ChatTheme.colors.onBackground2,
            textAlign = TextAlign.Center,
            modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
        )
    }
}