package nand.apps.chat.ui.dialog

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.padding
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.text.SimpleTextCheckbox
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.formatFileSize
import okio.FileSystem
import okio.Path
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

@Composable
fun FileTransferConfirmationDialog(
    path: Path,
    onConfirm: () -> Unit,
    onCancel: () -> Unit,
) {
    val settings = LocalSettingsProvider.current.fileSettings
    val settingsRepo: SettingsRepo by getKoin().inject()
    val fileSystem: FileSystem by getKoin().inject()
    var dontAskAgain by rememberSaveable { mutableStateOf(false) }
    SimpleDialog(
        title = Res.string.file_transfer_confirmation_dialog_title,
        onConfirm = {
            if (dontAskAgain) {
                settingsRepo.update { settings.copy(showFileTransferConfirmation = false) }
            }
            onConfirm()
        },
        onCancel = onCancel,
    ) {
        Column {
            Column(
                modifier = Modifier.padding(start = ChatTheme.dimens.spacingMedium),
            ) {
                Text(
                    text = stringResource(Res.string.file_transfer_confirmation_dialog_message),
                    style = ChatTheme.typography.body2,
                    modifier = Modifier.padding(bottom = ChatTheme.dimens.spacingMedium),
                )
                TextLine(
                    text = path.name,
                    style = ChatTheme.typography.body1,
                    color = ChatTheme.colors.onSurface,
                )
                fileSystem.metadataOrNull(path)?.size?.let {
                    TextLine(
                        text = formatFileSize(it),
                        style = ChatTheme.typography.body2,
                        color = ChatTheme.colors.onSurface,
                    )
                }
            }
            SimpleTextCheckbox(
                text = stringResource(Res.string.dont_ask_again),
                isChecked = dontAskAgain,
                onCheckedChange = { dontAskAgain = it },
                modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
            )
        }
    }
}