package nand.apps.chat.ui.dialog

import androidx.compose.foundation.layout.*
import androidx.compose.material.Icon
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Error
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.style.TextAlign
import nand.apps.chat.model.event.*
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringArrayResource
import seers.composeapp.generated.resources.*

@Composable
fun GenericErrorDialog() {
    val eventHandler = LocalEventHandler.current
    var errorMessage by rememberSaveable { mutableStateOf<String?>(null) }
    LaunchedEffect(Unit) {
        eventHandler.onEvent.collect { event ->
            when (event) {
                is ErrorDialogEvent -> errorMessage = event.message
                else -> Unit
            }
        }
    }
    errorMessage?.let {
        SimpleDialog(
            title = null,
            confirmLabel = null,
            cancelLabel = Res.string.close,
            onCancel = { errorMessage = null },
        ) {
            Column(
                horizontalAlignment = Alignment.CenterHorizontally,
                verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingXLarge),
                modifier = Modifier.fillMaxWidth(),
            ) {
                Row(
                    horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
                    verticalAlignment = Alignment.CenterVertically,
                ) {
                    Icon(
                        imageVector = Icons.Default.Error,
                        contentDescription = null,
                        tint = ChatTheme.colors.error,
                        modifier = Modifier.size(ChatTheme.dimens.iconMedium),
                    )
                    TextLine(
                        text = stringArrayResource(Res.array.generic_error_titles).random(),
                        style = ChatTheme.typography.h5,
                        textAlign = TextAlign.Center,
                    )
                    Icon(
                        imageVector = Icons.Default.Error,
                        contentDescription = null,
                        tint = ChatTheme.colors.error,
                        modifier = Modifier.size(ChatTheme.dimens.iconMedium),
                    )
                }
                Text(
                    text = it,
                    style = ChatTheme.typography.body1,
                    color = ChatTheme.colors.onSurface,
                    textAlign = TextAlign.Center,
                )
            }
        }
    }
}