package nand.apps.chat.ui.dialog

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.material.Text
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import nand.apps.chat.ui.modifier.onEnter
import nand.apps.chat.ui.text.SimplePasswordField
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.takeIfUnderLength
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun PasswordDialog(
    title: StringResource,
    label: StringResource,
    maxLength: Int,
    initialPassword: String? = null,
    isOptional: Boolean = false,
    onConfirm: (String?) -> Unit,
    onCancel: () -> Unit,
) {
    var password by rememberTextFieldValue(initialPassword.orEmpty())
    val focusRequester = remember { FocusRequester() }
    val confirmPassword = { onConfirm(password.text.takeIf { it.isNotEmpty() }) }
    LaunchedEffect(Unit) {
        focusRequester.requestFocus()
    }
    SimpleDialog(
        title = title,
        onConfirm = confirmPassword,
        onCancel = onCancel,
    ) {
        Column(
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
            modifier = Modifier.fillMaxWidth(),
        ) {
            if (isOptional) {
                Text(
                    text = stringResource(Res.string.optional_password_message),
                    color = ChatTheme.colors.onBackground,
                    style = ChatTheme.typography.body2,
                )
            }
            SimplePasswordField(
                password = password,
                onValueChange = { password = it.takeIfUnderLength(maxLength, password) },
                label = label,
                modifier = Modifier.fillMaxWidth()
                    .focusRequester(focusRequester)
                    .onEnter(confirmPassword),
            )
        }
    }
}