package nand.apps.chat.ui.dialog

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.text.selection.DisableSelection
import androidx.compose.material.Surface
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.Dialog
import androidx.compose.ui.window.DialogProperties
import nand.apps.chat.ui.button.SimpleTextButton
import nand.apps.chat.ui.modifier.appendIf
import nand.apps.chat.ui.navigation.BackHandler
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun SimpleDialog(
    title: StringResource? = null,
    modifier: Modifier = Modifier,
    titleStyle: TextStyle = ChatTheme.typography.h6,
    confirmLabel: StringResource? = Res.string.confirm,
    cancelLabel: StringResource = Res.string.cancel,
    onConfirm: () -> Unit = {},
    onCancel: () -> Unit = {},
    isConfirmEnabled: Boolean = true,
    isCancelEnabled: Boolean = true,
    isCancellable: Boolean = true,
    isListContent: Boolean = false,
    titleEndContent: @Composable () -> Unit = {},
    content: @Composable BoxScope.() -> Unit,
) = DisableSelection {
    Dialog(
        onDismissRequest = {},
        properties = DialogProperties(dismissOnClickOutside = false),
    ) {
        Surface(
            shape = ChatTheme.shapes.small,
            color = ChatTheme.colors.background,
            modifier = modifier,
        ) {
            Column(
                modifier = Modifier.heightIn(max = if (isListContent) 500.dp else Dp.Unspecified)
            ) {
                title?.let {
                    Row(
                        verticalAlignment = Alignment.CenterVertically,
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(
                                start = ChatTheme.dimens.spacingXLarge,
                                end = ChatTheme.dimens.spacingMedium,
                                top = ChatTheme.dimens.spacingSmall,
                            ),
                    ) {
                        TextLine(
                            text = stringResource(title),
                            style = titleStyle,
                            modifier = Modifier
                                .padding(vertical = ChatTheme.dimens.spacingLarge)
                                .weight(1f),
                        )
                        titleEndContent()
                    }
                } ?: Spacer(modifier = Modifier.height(ChatTheme.dimens.spacingLarge))
                Box(
                    content = content,
                    modifier = Modifier
                        .fillMaxWidth()
                        .appendIf(isListContent) { weight(1f) }
                        .padding(horizontal = ChatTheme.dimens.spacingXLarge)
                )
                if (isCancellable) {
                    Row(
                        horizontalArrangement = Arrangement.SpaceAround,
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(ChatTheme.dimens.spacingLarge)
                    ) {
                        SimpleTextButton(
                            text = stringResource(cancelLabel),
                            onClick = onCancel,
                            isEnabled = isCancelEnabled,
                        )
                        confirmLabel?.let {
                            SimpleTextButton(
                                text = stringResource(confirmLabel),
                                onClick = onConfirm,
                                isEnabled = isConfirmEnabled,
                            )
                        }
                    }
                    BackHandler(onBack = onCancel)
                }
            }
        }
    }
}