package nand.apps.chat.ui.embed

import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import nand.apps.chat.embed.EmbedPreviewFactory
import nand.apps.chat.embed.file.ImageEmbedData
import nand.apps.chat.model.event.*
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.provider.LocalSettingsProvider
import org.koin.compose.getKoin

@Composable
fun EmbedContainer(
    url: String,
    modifier: Modifier = Modifier,
    message: ChatMessage? = null,
) {
    val factory: EmbedPreviewFactory by getKoin().inject()
    val settings = LocalSettingsProvider.current.embedSettings
    if (!settings.isEnabled || !factory.isHostAllowed(url)) return
    
    val eventHandler = LocalEventHandler.current
    
    var embedData by remember(url, message?.embedData) {
        mutableStateOf(message?.embedData ?: factory.getCachedEmbedData(url))
    }
    when {
        embedData == null -> {
            LaunchedEffect(url) {
                factory.getEmbedData(url)?.let { newEmbedData ->
                    if (message != null) {
                        eventHandler += UpdateMessageEvent(message.copy(embedData = newEmbedData))
                    } else {
                        embedData = newEmbedData
                    }
                }
            }
        }
        message != null && embedData != message.embedData -> {
            eventHandler += UpdateMessageEvent(message.copy(embedData = embedData))
            return
        }
    }
    embedData?.let { embed ->
        if (embed is ImageEmbedData && !settings.isImagesEnabled) return
        embed.GetComposable(modifier)
    }
}
