package nand.apps.chat.ui.embed

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.FilterQuality
import androidx.compose.ui.graphics.Shape
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import coil3.compose.AsyncImage
import nand.apps.chat.ui.dialog.image.StaticImagePreviewDialog
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun EmbedThumbnailRow(
    thumbnailUrl: String?,
    contentDescription: String?,
    modifier: Modifier = Modifier,
    thumbnailSize: Dp = ChatTheme.dimens.imageThumbnail,
    thumbnailShape: Shape = ChatTheme.shapes.small,
    thumbnailScale: ContentScale = ContentScale.Crop,
    clickable: Boolean = true,
    isImagePreviewSupported: Boolean = false,
    onClick: () -> Unit = {},
    content: @Composable ColumnScope.() -> Unit,
) = Row(
    verticalAlignment = Alignment.CenterVertically,
    modifier = modifier
        .fillMaxWidth()
        .clip(ChatTheme.shapes.medium)
        .background(ChatTheme.colors.surface2)
        .onClick(
            isEnabled = clickable,
            onClick = onClick,
        )
        .padding(ChatTheme.dimens.spacingMedium)
) {
    var isPreviewDialogVisible by rememberSaveable { mutableStateOf(false) }
    AsyncImage(
        model = thumbnailUrl,
        contentDescription = contentDescription,
        contentScale = thumbnailScale,
        filterQuality = FilterQuality.High,
        modifier = Modifier
            .size(thumbnailSize)
            .clip(thumbnailShape)
            .background(ChatTheme.colors.surface3)
            .padding(if (thumbnailScale == ContentScale.Fit) ChatTheme.dimens.spacingSmall else 0.dp)
            .onClick(isImagePreviewSupported && thumbnailUrl != null) { isPreviewDialogVisible = true },
    )
    Column(
        modifier = Modifier.padding(start = ChatTheme.dimens.spacingMedium),
        content = content,
    )
    if (isPreviewDialogVisible && thumbnailUrl != null) {
        StaticImagePreviewDialog(
            model = thumbnailUrl,
            contentDescription = thumbnailUrl,
            onDismiss = { isPreviewDialogVisible = false },
        )
    }
}