package nand.apps.chat.ui.embed.amazon

import androidx.compose.foundation.layout.padding
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.style.BaselineShift
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.text.withStyle
import androidx.compose.ui.unit.sp
import nand.apps.chat.embed.amazon.AmazonProductMetadata
import nand.apps.chat.ui.embed.EmbedThumbnailRow
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun AmazonProductMetadata.Renderer(
    modifier: Modifier = Modifier,
) = EmbedThumbnailRow(
    thumbnailUrl = imageUrl,
    contentDescription = name,
    isImagePreviewSupported = true,
    modifier = modifier,
) {
    TextLine(
        text = name,
        style = ChatTheme.typography.h5,
    )
    TextLine(
        text = buildAnnotatedString {
            withStyle(SpanStyle(fontSize = 12.sp, baselineShift = BaselineShift.Superscript)) {
                append(priceSymbol)
            }
            append(priceWhole)
            priceFraction?.let {
                withStyle(SpanStyle(fontSize = 12.sp, baselineShift = BaselineShift.Superscript)) {
                    append(it)
                }
            }
        },
        style = ChatTheme.typography.h6.copy(lineHeight = 14.sp),
    )
    features?.let {
        Text(
            text = features,
            color = ChatTheme.colors.onBackground,
            style = ChatTheme.typography.body2,
            maxLines = 3,
            overflow = TextOverflow.Ellipsis,
            modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall)
        )
    }
}