package nand.apps.chat.ui.embed.ebay

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.FilterQuality
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.withStyle
import coil3.compose.AsyncImage
import nand.apps.chat.embed.ebay.EbayListingMetadata
import nand.apps.chat.ui.embed.EmbedThumbnailRow
import nand.apps.chat.ui.provider.LocalUriLauncher
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun EbayListingMetadata.Renderer(
    modifier: Modifier = Modifier,
) {
    val uriLauncher = LocalUriLauncher.current
    EmbedThumbnailRow(
        thumbnailUrl = imageUrl,
        contentDescription = name,
        clickable = true,
        isImagePreviewSupported = true,
        modifier = modifier,
        onClick = { uriLauncher.launchUri(url) },
    ) {
        TextLine(
            text = name,
            style = ChatTheme.typography.h6,
        )
        TextLine(
            text = price,
            style = ChatTheme.typography.body1,
            color = Color(0xFFFFEECC),
        )
        condition?.let { condition ->
            TextLine(
                text = buildAnnotatedString {
                    append("Condition: ")
                    withStyle(SpanStyle(fontWeight = FontWeight.Bold)) {
                        append(condition)
                    }
                },
                style = ChatTheme.typography.body2,
                color = ChatTheme.colors.onBackground,
            )
        }
        seller?.let { seller ->
            Row(
                verticalAlignment = Alignment.CenterVertically,
                modifier = Modifier.padding(top = ChatTheme.dimens.spacingLarge),
            ) {
                seller.thumbnailUrl?.let {
                    AsyncImage(
                        model = seller.thumbnailUrl,
                        contentDescription = seller.name,
                        contentScale = ContentScale.Fit,
                        filterQuality = FilterQuality.High,
                        modifier = Modifier
                            .size(ChatTheme.dimens.avatarMedium)
                            .clip(CircleShape)
                            .background(ChatTheme.colors.surface3),
                    )
                }
                Column(
                    modifier = Modifier.padding(start = ChatTheme.dimens.spacingSmall)
                ) {
                    TextLine(
                        text = buildAnnotatedString {
                            append(seller.name)
                            seller.reviewCount?.let { reviewCount ->
                                append(" ")
                                withStyle(SpanStyle(color = ChatTheme.colors.onBackground)) {
                                    append("($reviewCount)")
                                }
                            }
                        },
                        style = ChatTheme.typography.caption,
                    )
                    seller.rating?.let { rating ->
                        TextLine(
                            text = "$rating positive",
                            style = ChatTheme.typography.caption,
                            color = ChatTheme.colors.onBackground,
                        )
                    }
                }
            }
        }
    }
}