package nand.apps.chat.ui.embed.image

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.size
import androidx.compose.material.Icon
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.BrokenImage
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.FilterQuality
import androidx.compose.ui.unit.Dp
import coil3.compose.AsyncImagePainter.State
import coil3.compose.SubcomposeAsyncImage
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.spinner.CubeGrid
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun StaticImageRenderer(
    model: Any,
    contentDescription: String?,
    modifier: Modifier = Modifier,
    onClick: (() -> Unit)? = null,
    placeholderSize: Dp = ChatTheme.dimens.avatarXLarge,
) {
    var imageState: State by remember { mutableStateOf(State.Empty) }
    SubcomposeAsyncImage(
        model = model,
        contentDescription = contentDescription,
        filterQuality = FilterQuality.High,
        onState = { imageState = it },
        modifier = modifier,
    ) {
        when (imageState) {
            is State.Success -> {
                Image(
                    painter = painter,
                    contentDescription = contentDescription,
                    modifier = Modifier.onClick(onClick != null) { onClick?.invoke() },
                )
            }
            is State.Loading -> {
                CubeGrid(size = placeholderSize)
            }
            is State.Error -> {
                Icon(
                    imageVector = Icons.Default.BrokenImage,
                    contentDescription = contentDescription,
                    tint = ChatTheme.colors.error,
                    modifier = Modifier.size(placeholderSize),
                )
            }
            else -> Unit
        }
    }
}