package nand.apps.chat.ui.embed.mediawiki

import androidx.compose.foundation.layout.*
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.text.style.TextOverflow
import nand.apps.chat.embed.mediawiki.MediaWikiPageMetadata
import nand.apps.chat.ui.embed.EmbedThumbnailRow
import nand.apps.chat.ui.provider.LocalUriLauncher
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.rememberHtmlAnnotatedString

@Composable
fun MediaWikiPageMetadata.Renderer(
    modifier: Modifier = Modifier,
) {
    val uriLauncher = LocalUriLauncher.current
    EmbedThumbnailRow(
        thumbnailUrl = thumbnail?.url,
        thumbnailScale = if (thumbnail?.shouldCrop == true) ContentScale.Crop else ContentScale.Fit,
        contentDescription = title,
        modifier = modifier,
        clickable = true,
        isImagePreviewSupported = true,
        onClick = { uriLauncher.launchUri(url) },
    ) {
        TextLine(
            text = title,
            style = ChatTheme.typography.h5,
        )

        summary?.let {
            Text(
                text = rememberHtmlAnnotatedString(summary),
                color = ChatTheme.colors.onBackground,
                style = ChatTheme.typography.body2,
                maxLines = 4,
                overflow = TextOverflow.Ellipsis,
                modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall)
            )
        }
    }
}