package nand.apps.chat.ui.embed.stackexchange

import androidx.compose.foundation.layout.*
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.text.style.TextOverflow
import nand.apps.chat.embed.stackexchange.StackExchangePageMetadata
import nand.apps.chat.time.formatRelativeDate
import nand.apps.chat.ui.embed.EmbedThumbnailRow
import nand.apps.chat.ui.provider.LocalUriLauncher
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.rememberHtmlAnnotatedString

@Composable
fun StackExchangePageMetadata.Renderer(
    modifier: Modifier = Modifier,
) {
    val uriLauncher = LocalUriLauncher.current
    EmbedThumbnailRow(
        thumbnailUrl = iconUrl,
        thumbnailScale = ContentScale.Crop,
        contentDescription = title,
        modifier = modifier,
        clickable = true,
        onClick = { uriLauncher.launchUri(url) },
    ) {
        TextLine(
            text = title,
            color = ChatTheme.colors.onSurface,
            style = ChatTheme.typography.h6,
        )
        Text(
            text = rememberHtmlAnnotatedString(body),
            color = ChatTheme.colors.onBackground,
            style = ChatTheme.typography.body2,
            maxLines = 3,
            overflow = TextOverflow.Ellipsis,
            modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
        )
        askTime?.let {
            TextLine(
                text = formatRelativeDate(it),
                color = ChatTheme.colors.onSurface,
                style = ChatTheme.typography.body2,
                modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
            )
        }
    }
}