package nand.apps.chat.ui.embed.web

import androidx.compose.foundation.layout.padding
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.text.style.TextOverflow
import nand.apps.chat.embed.web.WebPageEmbedData
import nand.apps.chat.ui.embed.EmbedThumbnailRow
import nand.apps.chat.ui.provider.LocalUriLauncher
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun WebPageEmbedData.Renderer(
    modifier: Modifier = Modifier,
) {
    val uriLauncher = LocalUriLauncher.current
    EmbedThumbnailRow(
        thumbnailUrl = imageUrl,
        thumbnailScale = ContentScale.Crop,
        contentDescription = title,
        modifier = modifier,
        clickable = true,
        onClick = { uriLauncher.launchUri(url) },
    ) {
        TextLine(
            text = title,
            color = ChatTheme.colors.onSurface,
            style = ChatTheme.typography.h6,
        )
        Text(
            text = description,
            color = ChatTheme.colors.onBackground,
            style = ChatTheme.typography.body2,
            maxLines = 3,
            overflow = TextOverflow.Ellipsis,
            modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
        )
        TextLine(
            text = domain,
            style = ChatTheme.typography.body2,
            modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
        )
    }
}