package nand.apps.chat.ui.file

import androidx.compose.foundation.text.ClickableText
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.withStyle
import androidx.compose.ui.util.fastForEachIndexed
import nand.apps.chat.platform.ChatPlatform
import nand.apps.chat.platform.getPlatform
import nand.apps.chat.ui.theme.ChatThemeScope
import okio.Path.Companion.DIRECTORY_SEPARATOR

@Composable
fun ChatFileDirectoryRow(
    directory: String,
    textStyle: TextStyle,
    onClick: (String) -> Unit,
    modifier: Modifier = Modifier,
) {
    var fullDirectory by rememberSaveable { mutableStateOf(directory) }
    if (!fullDirectory.startsWith(directory + DIRECTORY_SEPARATOR)) {
        fullDirectory = directory
    }
    val text = ChatThemeScope {
        remember(directory) {
            val parts = fullDirectory.split(DIRECTORY_SEPARATOR).filter { it.isNotEmpty() }
            buildAnnotatedString {
                val pathBuilder = StringBuilder(
                    if (getPlatform() != ChatPlatform.WINDOWS) DIRECTORY_SEPARATOR else ""
                )
                pushStyle(SpanStyle(color = colors.onBackground))
                parts.fastForEachIndexed { index, part ->
                    pathBuilder.append(part)
                    pathBuilder.append(DIRECTORY_SEPARATOR)
                    val fullPath = pathBuilder.toString()
                    pushStringAnnotation(
                        tag = "directory",
                        annotation = fullPath,
                    )
                    if (index > 0) append(' ')
                    if (fullPath == directory) {
                        withStyle(SpanStyle(color = colors.onSurface)) {
                            append(part)
                        }
                    } else {
                        append(part)
                    }
                    append(' ')
                    pop()
                    if (index < parts.lastIndex) append('»')
                }
            }
        }
    }
    ClickableText(
        text = text,
        style = textStyle,
        onClick = { index ->
            text.getStringAnnotations(index, index).firstOrNull()?.let { range ->
                onClick(range.item)
            }
        },
        modifier = modifier,
    )
}