package nand.apps.chat.ui.file

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.basicMarquee
import androidx.compose.foundation.layout.*
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.FilterQuality
import androidx.compose.ui.unit.dp
import coil3.compose.AsyncImage
import com.ashampoo.kim.Kim
import com.ashampoo.kim.common.convertToPhotoMetadata
import com.ashampoo.kim.kotlinx.readMetadata
import nand.apps.chat.io.ChatFileMetadata
import kotlinx.io.files.Path as KPath
import nand.apps.chat.io.IMAGE_EXTENSIONS
import nand.apps.chat.ui.provider.LocalDateTimeFormatProvider
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.formatFileSize

@OptIn(ExperimentalFoundationApi::class)
@Composable
fun ChatFilePreviewColumn(
    file: ChatFileMetadata,
    modifier: Modifier = Modifier,
) {
    if (file.extension !in IMAGE_EXTENSIONS) return
    
    val metadata = remember(file.path) { Kim.readMetadata(KPath(file.path))?.convertToPhotoMetadata() }
    Column(
        horizontalAlignment = Alignment.CenterHorizontally,
        modifier = modifier,
    ) {
        AsyncImage(
            model = file.path,
            contentDescription = file.name,
            filterQuality = FilterQuality.High,
            modifier = Modifier
                .clip(ChatTheme.shapes.small)
                .background(Color.Black)
                .fillMaxWidth()
                .aspectRatio(ratio = 1.6f, true),
        )
        Text(
            text = file.name,
            style = ChatTheme.typography.body2,
            maxLines = 1,
            color = ChatTheme.colors.onSurface,
            modifier = Modifier
                .padding(top = ChatTheme.dimens.spacingMedium)
                .basicMarquee(velocity = 60.dp),
        )
        FilePreviewTextLine(
            text = formatFileSize(file.size) + metadata?.run { " | $widthPx x $heightPx" }.orEmpty(),
        )
        FilePreviewTextLine(
            text = LocalDateTimeFormatProvider.current.formatDateTime(metadata?.takenDate ?: file.lastModified),
        )
        metadata?.run {
            location?.displayString?.let {
                FilePreviewTextLine(text = it)
            }
            gpsCoordinates?.displayString?.let {
                FilePreviewTextLine(text = it)
            }
        }
    }
}

@Composable
private fun FilePreviewTextLine(text: String) {
    TextLine(
        text = text,
        style = ChatTheme.typography.caption,
        color = ChatTheme.colors.onBackground,
        modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
    )
}