package nand.apps.chat.ui.file

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import nand.apps.chat.io.ChatFilePaths
import nand.apps.chat.io.getFileMetadata
import nand.apps.chat.ui.modifier.appendIf
import nand.apps.chat.ui.theme.ChatTheme
import okio.FileSystem
import org.koin.compose.getKoin

@Composable
fun ChatFileShortcutsColumn(
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    iconsOnly: Boolean = false,
    onSelectDirectory: (String) -> Unit = {},
) {
    val fileSystem: FileSystem by getKoin().inject()
    val filePaths: ChatFilePaths by getKoin().inject()
    val mainDirs = remember {
        val paths = mutableListOf(
            filePaths.homePath,
            filePaths.picturesPath,
            filePaths.videosPath,
            filePaths.musicPath,
            filePaths.downloadPath,
            filePaths.documentPath,
        )
        paths += filePaths.roots
        paths.mapNotNull { fileSystem.getFileMetadata(it) }
    }
    Column(modifier = modifier) {
        mainDirs.forEach { file ->
            ChatFileRow(
                file = file,
                color = ChatTheme.colors.icon,
                isTextVisible = !iconsOnly,
                iconSize = if (iconsOnly) ChatTheme.dimens.iconLarge else ChatTheme.dimens.iconMedium,
                modifier = Modifier
                    .appendIf(!iconsOnly) { fillMaxWidth() }
                    .clickable(isEnabled) { onSelectDirectory(file.path) }
                    .padding(
                        horizontal = ChatTheme.dimens.spacingMedium,
                        vertical = ChatTheme.dimens.spacingSmall,
                    ),
            )
        }
    }
}