package nand.apps.chat.ui.friends

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.PersonOff
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.font.FontWeight
import nand.apps.chat.model.event.*
import nand.apps.chat.model.user.FriendRequest
import nand.apps.chat.model.user.UserStatus
import nand.apps.chat.ui.contacts.ContactRow
import nand.apps.chat.ui.main.SidebarState
import nand.apps.chat.ui.menu.ActionMenuItem
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun FriendRequestRow(
    request: FriendRequest,
    sidebarState: SidebarState,
    modifier: Modifier = Modifier,
) {
    val eventHandler = LocalEventHandler.current
    val declineLabel = stringResource(Res.string.decline_friend_request)
    ContactRow(
        heading = stringResource(Res.string.friend_request),
        headingStyle = when (sidebarState) {
            SidebarState.FULL -> ChatTheme.typography.h6
            else -> ChatTheme.typography.body2
        }.copy(fontWeight = FontWeight.Bold),
        subHeading = request.message,
        avatar = request.avatar,
        status = UserStatus.DEFAULT,
        isOnline = true,
        sidebarState = sidebarState,
        onClick = { eventHandler += OpenConversationEvent(request) },
        isActionMenuEnabled = true,
        actionMenuItems = {
            listOf(
                ActionMenuItem(
                    icon = Icons.Default.PersonOff,
                    label = declineLabel,
                    onClick = { eventHandler += DeclineFriendRequestEvent(request) },
                )
            )
        },
        modifier = modifier,
    )
}