package nand.apps.chat.ui.groups

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.chat.screen.ChatScreenState
import nand.apps.chat.ui.main.SidebarState
import nand.apps.chat.ui.provider.isPortraitMode
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.toolbar.SimpleToolbar
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

private val PEERS_COLUMN_WIDTH = 160.dp

@OptIn(ExperimentalFoundationApi::class)
@Composable
fun GroupPeersColumn(
    state: ChatScreenState,
    group: ChatGroupData,
    sidebarState: SidebarState,
    onExit: () -> Unit,
    modifier: Modifier = Modifier,
) = Column(
    modifier = modifier.run {
        if (isPortraitMode()) fillMaxWidth() else width(PEERS_COLUMN_WIDTH)
    },
) {
    val title = stringResource(Res.string.group_chat_peers_online, group.numPeersOnline, group.peers.size)
    val selfPeer = remember(state, group) { group.findSelf(state.self) }
    val peers = remember(group) { group.peers.values.sortedWith(UserData.CONTACT_LIST_COMPARATOR) }
    if (isPortraitMode()) {
        SimpleToolbar(
            text = title,
            onNavClick = onExit,
            modifier = Modifier.padding(ChatTheme.dimens.spacingMedium),
        )
        Box(
            modifier = Modifier
                .background(ChatTheme.colors.onBackground2)
                .fillMaxWidth()
                .height(2.dp)
        )
    } else {
        TextLine(
            text = title,
            style = ChatTheme.typography.body1,
            modifier = Modifier
                .fillMaxWidth()
                .background(ChatTheme.colors.surface)
                .padding(horizontal = ChatTheme.dimens.spacingMedium)
                .padding(top = ChatTheme.dimens.spacingMedium),
        )
    }
    LazyColumn(
        contentPadding = PaddingValues(vertical = ChatTheme.dimens.spacingSmall),
        modifier = Modifier
            .background(ChatTheme.colors.surface)
            .fillMaxHeight(),
    ) {
        items(
            items = peers,
            key = { it.uid.value },
        ) { peer ->
            GroupPeerRow(
                group = group,
                peer = peer,
                selfPeer = selfPeer,
                sidebarState = sidebarState,
                chatState = state,
                modifier = Modifier
                    .fillMaxWidth()
                    .animateItemPlacement()
            )
        }
    }
}