package nand.apps.chat.ui.groups

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.Icon
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Check
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.avatar.ChatAvatarBox
import nand.apps.chat.ui.avatar.ChatAvatarSize
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun SendGroupInviteDialog(
    friend: UserData,
    groups: List<ChatGroupData>,
    onConfirm: (ChatGroupData) -> Unit,
    onCancel: () -> Unit,
) = SimpleDialog(
    confirmLabel = null,
    onCancel = onCancel,
) {
    Column {
        TextLine(
            text = stringResource(Res.string.invite_to_group_dialog_title, friend.displayName),
            style = ChatTheme.typography.body1,
            modifier = Modifier.padding(bottom = ChatTheme.dimens.spacingMedium),
        )
        LazyColumn {
            items(
                items = groups,
                key = { it.uid.value },
            ) { group ->
                val isInGroup = group.peers[friend.uid]?.isOnline == true
                Row(
                    verticalAlignment = Alignment.CenterVertically,
                    modifier = Modifier
                        .background(ChatTheme.colors.background)
                        .onClick(!isInGroup) { onConfirm(group) }
                        .fillMaxWidth()
                        .padding(ChatTheme.dimens.spacingLarge)
                        .alpha(if (isInGroup) 0.5f else 1f)
                ) {
                    ChatAvatarBox(
                        avatar = group.avatar,
                        size = ChatAvatarSize.LARGE,
                    )
                    Column(
                        modifier = Modifier
                            .weight(1f)
                            .padding(start = ChatTheme.dimens.spacingMedium),
                    ) {
                        TextLine(
                            text = group.title,
                            style = ChatTheme.typography.body1,
                        )
                        TextLine(
                            text = if (isInGroup) {
                                stringResource(Res.string.invite_to_group_already_member)
                            } else {
                                stringResource(
                                    Res.string.group_chat_peers_online,
                                    group.numPeersOnline,
                                    group.peers.size
                                )
                            },
                            style = ChatTheme.typography.body2,
                            color = ChatTheme.colors.onBackground,
                        )
                    }
                    if (isInGroup) {
                        Icon(
                            imageVector = Icons.Default.Check,
                            contentDescription = stringResource(Res.string.invite_to_group_already_member),
                            tint = ChatTheme.colors.onBackground2,
                            modifier = Modifier.size(ChatTheme.dimens.iconSmall),
                        )
                    }
                }
            }
        }
    }
}