package nand.apps.chat.ui.groups

import androidx.compose.foundation.layout.Column
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import nand.apps.chat.model.event.*
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.group.ChatGroupRole
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.text.SimpleTextRadioButton
import seers.composeapp.generated.resources.Res
import seers.composeapp.generated.resources.set_role

@Composable
fun SetGroupRoleDialog(
    group: ChatGroupData,
    peer: UserData,
    selfPeer: UserData,
    onDismiss: () -> Unit,
) {
    val eventHandler = LocalEventHandler.current
    var selectedRole by rememberSaveable { mutableStateOf(peer.role) }
    SimpleDialog(
        title = Res.string.set_role,
        onConfirm = {
            eventHandler += SetGroupRoleEvent(group, peer, selectedRole)
            onDismiss()
        },
        onCancel = onDismiss,
    ) {
        Column {
            ChatGroupRole.entries.forEach { role ->
                if (selfPeer.role.hasAuthorityOver(role)) {
                    SimpleTextRadioButton(
                        text = role.getDisplayString(),
                        selected = selectedRole == role,
                        onClick = { selectedRole = role },
                    )
                }
            }
        }
    }
}