package nand.apps.chat.ui.main

import androidx.compose.animation.animateContentSize
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import nand.apps.chat.model.event.*
import nand.apps.chat.ui.chat.screen.ChatScreenState
import nand.apps.chat.ui.contacts.ContactListColumn
import nand.apps.chat.ui.profile.SelfProfileRow
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.provider.isPortraitMode
import nand.apps.chat.ui.theme.ChatTheme

private val DEFAULT_SIDEBAR_WIDTH = 192.dp

@Composable
fun MainSidebar(
    chatState: ChatScreenState,
    modifier: Modifier = Modifier,
) {
    val eventHandler = LocalEventHandler.current
    val settings = LocalSettingsProvider.current
    val isContactsCollapsed = settings.generalSettings.isContactsCollapsed
    val sidebarState = when {
        isPortraitMode() -> SidebarState.FULL
        isContactsCollapsed -> SidebarState.COLLAPSED
        else -> SidebarState.DEFAULT
    }
    val widthModifier = when (sidebarState) {
        SidebarState.FULL -> Modifier.fillMaxWidth()
        SidebarState.COLLAPSED -> Modifier.animateContentSize()
        SidebarState.DEFAULT -> Modifier.animateContentSize().width(DEFAULT_SIDEBAR_WIDTH)
    }
    Column(
        modifier = modifier
            .fillMaxHeight()
            .then(widthModifier),
    ) {
        ContactListColumn(
            chatState = chatState,
            sidebarState = sidebarState,
            onToggleCollapsedState = {
                eventHandler += UpdateSettingsEvent(
                    settings = settings.copy(
                        generalSettings = settings.generalSettings.copy(
                            isContactsCollapsed = !isContactsCollapsed,
                        )
                    )
                )
            },
            modifier = Modifier
                .background(ChatTheme.colors.surface)
                .then(widthModifier)
                .weight(1f),
        )
        SelfProfileRow(
            sidebarState = sidebarState,
            modifier = Modifier
                .background(ChatTheme.colors.surface2)
                .then(widthModifier),
        )
    }
}