package nand.apps.chat.ui.menu

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.combinedClickable
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.hapticfeedback.HapticFeedbackType
import androidx.compose.ui.platform.LocalHapticFeedback
import androidx.compose.ui.window.Dialog
import nand.apps.chat.ui.theme.ChatTheme

@OptIn(ExperimentalFoundationApi::class)
@Composable
fun ActionMenuDialog(
    items: () -> List<ActionMenuItem>,
    onClick: (() -> Unit)?,
    isEnabled: Boolean,
    modifier: Modifier,
    content: @Composable () -> Unit,
) {
    val hapticFeedback = LocalHapticFeedback.current
    var isMenuVisible by rememberSaveable { mutableStateOf(false) }
    Box(
        modifier = modifier.run {
            when {
                isEnabled -> combinedClickable(
                    enabled = isEnabled,
                    onClick = onClick ?: {},
                    onLongClick = {
                        hapticFeedback.performHapticFeedback(HapticFeedbackType.LongPress)
                        isMenuVisible = true
                    },
                )
                onClick != null -> clickable(onClick = onClick)
                else -> this
            }
        },
        propagateMinConstraints = true,
    ) {
        content()
    }
    if (isMenuVisible) {
        Dialog(
            onDismissRequest = { isMenuVisible = false },
        ) {
            Column(
                modifier = Modifier
                    .background(
                        color = ChatTheme.colors.background,
                        shape = ChatTheme.shapes.small,
                    )
            ) {
                val menuItems = items()
                if (menuItems.isEmpty()) isMenuVisible = false
                menuItems.forEach { item ->
                    ActionMenuRow(
                        icon = item.icon,
                        label = item.label,
                        onClick = {
                            isMenuVisible = false
                            item.onClick()
                        }
                    )
                }
            }
        }
    }
}