package nand.apps.chat.ui.menu

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.material.Icon
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun ActionMenuRow(
    icon: ImageVector,
    label: String,
    onClick: () -> Unit,
) = Row(
    verticalAlignment = Alignment.CenterVertically,
    horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingLarge),
    modifier = Modifier
        .background(ChatTheme.colors.background)
        .fillMaxWidth()
        .onClick(onClick = onClick)
        .padding(ChatTheme.dimens.spacingXLarge),
) {
    Icon(
        imageVector = icon,
        contentDescription = label,
        tint = ChatTheme.colors.icon,
        modifier = Modifier.size(ChatTheme.dimens.iconMedium),
    )
    TextLine(
        text = label,
        style = ChatTheme.typography.h6,
        color = ChatTheme.colors.onSurface,
    )
}