package nand.apps.chat.ui.modifier

import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.pointer.isCtrlPressed
import nand.apps.chat.model.settings.DENSITY_RANGE
import nand.apps.chat.model.settings.shortcut.ShortcutAction
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.ui.shortcut.ShortcutHandler
import nand.apps.chat.util.clamp
import org.koin.compose.getKoin
import kotlin.math.sign

@Composable
fun Modifier.onDensityScale(): Modifier {
    val settingsRepo: SettingsRepo by getKoin().inject()
    ShortcutHandler { action ->
        when (action) {
            ShortcutAction.ZOOM_IN -> {
                settingsRepo.updateScale(1)
                true
            }
            ShortcutAction.ZOOM_OUT -> {
                settingsRepo.updateScale(-1)
                true
            }
            ShortcutAction.ZOOM_RESET -> {
                settingsRepo.updateScale(0)
                true
            }
            else -> false
        }
    }
    return onScrollWheel { amount, modifiers ->
        if (modifiers.isCtrlPressed) {
           settingsRepo.updateScale(-amount.sign.toInt())
        }
    }
}

private fun SettingsRepo.updateScale(direction: Int) {
    val settings = settings.generalSettings
    val scale = if (direction == 0) 1f else (settings.densityScale + (DENSITY_RANGE.start * direction)).clamp(DENSITY_RANGE)
    update { settings.copy(densityScale = scale) }
}