package nand.apps.chat.ui.modifier

import androidx.compose.ui.Modifier
import androidx.compose.ui.input.key.*
import nand.apps.chat.model.settings.shortcut.*

fun Modifier.onKeyDown(block: (KeyEvent) -> Boolean) = onPreviewKeyEvent { event ->
    if (event.type == KeyEventType.KeyDown) {
        block(event)
    } else {
        false
    }
}

fun Modifier.onEnter(block: () -> Unit) = onKeyDown { event ->
    if (event.key == Key.Enter || event.key == Key.NumPadEnter) {
        block()
        true
    } else {
        false
    }
}

fun Modifier.onPasteKey(block: () -> Boolean) = onKeyDown { event ->
    if (event.key == Key.Paste || event.key == Key.V && event.isCtrlPressed) {
        block()
    } else {
        false
    }
}

fun KeyEvent.toShortcutCombo(): ShortcutKeyCombo? {
    val label = key.label ?: return null
    return ShortcutKeyCombo(
        key = label,
        ctrl = isCtrlPressed,
        alt = isAltPressed,
        shift = isShiftPressed,
        meta = isMetaPressed,
    ).takeIf { it.isValid() }
}

fun KeyEvent.toShortcutAction(settings: ShortcutSettingsData): ShortcutAction? {
    val combo = toShortcutCombo() ?: return null
    val comboLabel = combo.toString()
    return settings.shortcuts.keys.find { settings.shortcuts[it] == comboLabel }
}