package nand.apps.chat.ui.modifier

import androidx.compose.foundation.clickable
import androidx.compose.foundation.gestures.awaitEachGesture
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.pointer.*
import androidx.compose.ui.util.fastForEach

inline fun Modifier.appendIf(condition: Boolean, block: Modifier.() -> Modifier) =
    if (condition) block() else this

fun Modifier.onClick(
    isEnabled: Boolean = true,
    onClick: () -> Unit,
) = clickable(
    enabled = isEnabled,
    onClick = onClick,
).appendIf(isEnabled) {
    pointerHoverIcon(PointerIcon.Hand)
}

fun Modifier.onScrollWheel(
    block: (Float, PointerKeyboardModifiers) -> Unit,
): Modifier {
    return pointerInput(Unit) {
        awaitEachGesture {
            val event = awaitPointerEvent()
            if (event.type == PointerEventType.Scroll) {
                event.changes.fastForEach { block(it.scrollDelta.y, event.keyboardModifiers) }
            }
        }
    }
}