package nand.apps.chat.ui.navigation

import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.Saver
import androidx.compose.runtime.saveable.SaverScope
import androidx.compose.runtime.saveable.rememberSaveable

class NavController {

    private var currentRoute: NavRoute? = null
        set(value) {
            if (value != field && value != null) {
                field = value
                onRouteChanged(value)
            }
        }
    
    private val backStack = mutableSetOf<NavRoute>()
    private var onRouteChanged: (NavRoute) -> Unit = {}
    val canNavigateBack get() = backStack.isNotEmpty()

    fun init(
        startRoute: NavRoute,
        onRouteChanged: (NavRoute) -> Unit,
    ) {
        this.onRouteChanged = onRouteChanged
        backStack.clear()
        currentRoute = startRoute
    }
    
    fun navigate(vararg routes: NavRoute, reset: Boolean = false) {
        if (reset) backStack.clear()
        for (route in routes) {
            when {
                route == currentRoute -> continue
                backStack.contains(route) -> {
                    val backRoutes = backStack.toList()
                    backStack.clear()
                    for (backRoute in backRoutes) {
                        if (backRoute != route) {
                            backStack.add(backRoute)
                        } else {
                            break
                        }
                    }
                }
                !reset -> currentRoute?.let { backStack.add(it) }
            }
            currentRoute = route
        }
    }

    // Function to handle the back
    fun navigateBack() {
        if (backStack.isNotEmpty()) {
            val toRoute = backStack.last()
            backStack.remove(toRoute)
            currentRoute = toRoute
        }
    }
    
    fun getRouteList() = (currentRoute?.let { backStack + it } ?: backStack).toList()
}

private object NavControllerSaver : Saver<NavController, List<String>> {

    override fun SaverScope.save(value: NavController): List<String> {
        return value.getRouteList().map { it.value }
    }

    override fun restore(value: List<String>): NavController {
        val navController = NavController()
        for (route in value) {
            navController.navigate(NavRoute(route))
        }
        return navController
    }
}

@Composable
fun rememberNavController(): NavController =
    rememberSaveable(saver = NavControllerSaver) { NavController() }