package nand.apps.chat.ui.navigation

import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.Saver
import androidx.compose.runtime.saveable.SaverScope
import androidx.compose.runtime.saveable.rememberSaveable

data class NavRoute(val value: String)

object NavRouteSaver : Saver<MutableState<NavRoute?>, String> {

    override fun SaverScope.save(value: MutableState<NavRoute?>): String {
        return value.value?.toString().orEmpty()
    }

    override fun restore(value: String): MutableState<NavRoute?> {
        return mutableStateOf(value.takeIf { it.isNotEmpty() }?.let { NavRoute(it) })
    }
}

@Composable
fun rememberMutableNavRoute(initialValue: NavRoute? = null) =
    rememberSaveable(saver = NavRouteSaver) { mutableStateOf(initialValue) }