package nand.apps.chat.ui.profile

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import nand.apps.chat.model.settings.theme.ChatThemeSetting
import nand.apps.chat.model.user.UserStatus
import nand.apps.chat.repo.SelfProfileRepo
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.ui.contacts.ContactDialogType
import nand.apps.chat.ui.util.collect

class SelfProfileRowViewModel(
    private val repo: SelfProfileRepo,
    private val settingsRepo: SettingsRepo,
) : ViewModel() {

    val stateData = MutableStateFlow(SelfProfileRowState())
    
    init {
        collect(repo.selfObserver) { self ->
            stateData.tryEmit(stateData.value.copy(self = self))
        }
    }
    
    fun toggleTheme(isLightMode: Boolean) {
        settingsRepo.update {
            generalSettings.copy(appTheme = if (isLightMode) ChatThemeSetting.DARK else ChatThemeSetting.LIGHT)
        }
    }
    
    fun openDialog(dialog: ContactDialogType) {
        stateData.tryEmit(stateData.value.copy(dialog = dialog))
    }
    
    fun dismissDialog() {
        stateData.tryEmit(stateData.value.copy(dialog = null))
    }

    fun setStatus(status: UserStatus) {
        repo.setSelfStatus(status)
    }
}