package nand.apps.chat.ui.profile

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Row
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.NonRestartableComposable
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.model.user.UserStatus
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun SelfProfileStatusSelector(
    status: UserStatus,
    onSelected: (UserStatus) -> Unit,
    modifier: Modifier = Modifier,
) = Row(
    horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
    modifier = modifier,
) {
    StatusIcon(
        icon = Icons.Default.AccountCircle,
        contentDescription = Res.string.status_online,
        color = ChatTheme.colors.statusOnline,
        status = UserStatus.DEFAULT,
        currentStatus = status,
        onSelected = onSelected,
    )
    StatusIcon(
        icon = Icons.Default.ModeStandby,
        contentDescription = Res.string.status_away,
        color = ChatTheme.colors.statusAway,
        status = UserStatus.AWAY,
        currentStatus = status,
        onSelected = onSelected,
    )
    StatusIcon(
        icon = Icons.Default.DoNotDisturbOnTotalSilence,
        contentDescription = Res.string.status_busy,
        color = ChatTheme.colors.statusBusy,
        status = UserStatus.BUSY,
        currentStatus = status,
        onSelected = onSelected,
    )
}

@Composable
@NonRestartableComposable
private fun StatusIcon(
    icon: ImageVector,
    contentDescription: StringResource,
    color: Color,
    status: UserStatus,
    currentStatus: UserStatus,
    onSelected: (UserStatus) -> Unit,
) = IconButton(
    icon = icon,
    contentDescription = stringResource(contentDescription),
    color = color,
    onClick = { onSelected(status) },
    modifier = Modifier
        .clip(ChatTheme.shapes.small)
        .background(if (status == currentStatus) ChatTheme.colors.surface2 else Color.Unspecified),
)