package nand.apps.chat.ui.profile.export

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import nand.apps.chat.io.extension
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.file.ChatFilePickerDialog
import nand.apps.chat.ui.file.ChatFilePickerOptions
import nand.apps.chat.ui.spinner.ChasingDots
import nand.apps.chat.ui.text.SimpleTextRadioButton
import nand.apps.chat.ui.viewmodel.composableViewModel
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ProfileExportDialog(
    onDismiss: () -> Unit,
    viewModel: ProfileExportDialogViewModel = composableViewModel(),
) {
    val state by viewModel.stateData.collectAsState()
    
    SimpleDialog(
        title = Res.string.profile_export_dialog_title,
        confirmLabel = Res.string.profile_export_button,
        cancelLabel = Res.string.close,
        onConfirm = viewModel::openFilePicker,
        onCancel = onDismiss,
        isConfirmEnabled = !state.isExportBusy,
    ) {
        Column(
            modifier = Modifier.fillMaxWidth(),
        ) {
            SimpleTextRadioButton(
                text = stringResource(Res.string.profile_export_profile_only),
                selected = state.exportType == ProfileExportType.PROFILE_ONLY,
                isEnabled = !state.isExportBusy,
                onClick = { viewModel.setExportType(ProfileExportType.PROFILE_ONLY) },
            )
            SimpleTextRadioButton(
                text = stringResource(Res.string.profile_export_profile_full),
                selected = state.exportType == ProfileExportType.PROFILE_FULL,
                isEnabled = !state.isExportBusy,
                onClick = { viewModel.setExportType(ProfileExportType.PROFILE_FULL) },
            )
        }
        if (state.isExportBusy) {
            Box(
                contentAlignment = Alignment.Center,
                modifier = Modifier.matchParentSize(),
            ) {
                ChasingDots()
            }
        }
    }
    
    state.exportPath?.let { path ->
        ChatFilePickerDialog(
            options = ChatFilePickerOptions(
                fileName = path.name,
                directory = path.parent?.toString(),
                extensions = setOf(path.extension),
                isOpenDialog = false,
            ),
            onConfirm = viewModel::exportProfile,
            onCancel = viewModel::closeFilePicker,
        )
    }
}