package nand.apps.chat.ui.profile.export

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.IO
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nand.apps.chat.io.ChatFilePaths
import nand.apps.chat.io.FULL_PROFILE_EXTENSION
import nand.apps.chat.io.extension
import nand.apps.chat.io.zipDirectory
import nand.apps.chat.repo.SelfProfileRepo
import okio.FileSystem
import okio.Path.Companion.toPath

class ProfileExportDialogViewModel(
    private val filePaths: ChatFilePaths,
    private val fileSystem: FileSystem,
    private val selfProfileRepo: SelfProfileRepo,
) : ViewModel() {
    
    val stateData = MutableStateFlow(ProfileExportDialogState())
    
    fun setExportType(exportType: ProfileExportType) {
        stateData.tryEmit(stateData.value.copy(exportType = exportType))
    }
    
    fun openFilePicker() {
        val profilePath = selfProfileRepo.profilePath
        val profileName = selfProfileRepo.profileName
        val fileName = when (stateData.value.exportType) {
            ProfileExportType.PROFILE_ONLY -> "$profileName.${profilePath.extension}"
            ProfileExportType.PROFILE_FULL -> "$profileName.$FULL_PROFILE_EXTENSION"
        }
        val exportPath = filePaths.documentPath.resolve(fileName)
        stateData.tryEmit(stateData.value.copy(exportPath = exportPath))
    }
    
    fun closeFilePicker() = stateData.tryEmit(stateData.value.copy(exportPath = null))
    
    fun exportProfile(outputPath: String) {
        val profilePath = selfProfileRepo.profilePath
        stateData.tryEmit(
            stateData.value.copy(
                exportPath = null,
                isExportBusy = true,
            )
        )
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                when (stateData.value.exportType) {
                    ProfileExportType.PROFILE_ONLY -> {
                        fileSystem.copy(profilePath, outputPath.toPath())
                    }
                    ProfileExportType.PROFILE_FULL -> {
                        zipDirectory(profilePath.parent!!, outputPath.toPath())
                    }
                }
            }
            stateData.tryEmit(stateData.value.copy(isExportBusy = false))
        }
    }
}