package nand.apps.chat.ui.profile.selector

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.style.TextAlign
import nand.apps.chat.io.PROFILE_EXTENSIONS
import nand.apps.chat.io.ZIP_EXTENSIONS
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.file.ChatFilePickerDialog
import nand.apps.chat.ui.file.ChatFilePickerMode
import nand.apps.chat.ui.file.ChatFilePickerOptions
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.scrollbar.VerticalScrollbar
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.viewmodel.composableViewModel
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ProfileSelectorDialog(
    onProfileSelected: (String) -> Unit,
    onDismiss: () -> Unit,
    viewModel: ProfileSelectorViewModel = composableViewModel(),
) {
    val state by viewModel.stateData.collectAsState()
    val listState = rememberLazyListState()
    LaunchedEffect(state.selectedProfile) {
        state.selectedProfile?.let {
            onProfileSelected(it.filePath)
        }
    }
    state.filePickerDirectory?.let { directory ->
        ChatFilePickerDialog(
            options = ChatFilePickerOptions(
                directory = directory,
                extensions = PROFILE_EXTENSIONS + ZIP_EXTENSIONS,
                mode = ChatFilePickerMode.FILES_AND_DIRECTORIES,
            ),
            onConfirm = onProfileSelected,
            onCancel = onDismiss,
        )
    } ?: SimpleDialog(
        title = Res.string.tutorial_profile_import_select,
        confirmLabel = null,
        onCancel = onDismiss,
        isListContent = true,
    ) {
        LazyColumn(
            horizontalAlignment = Alignment.CenterHorizontally,
            state = listState,
            modifier = Modifier.fillMaxWidth(),
        ) {
            items(
                items = state.rowData,
                key = { it.filePath },
            ) { row ->
                TextLine(
                    text = row.username,
                    style = ChatTheme.typography.body1,
                    textAlign = TextAlign.Center,
                    modifier = Modifier
                        .onClick { onProfileSelected(row.filePath) }
                        .fillMaxWidth()
                        .padding(ChatTheme.dimens.spacingLarge),
                )
            }
            item(key = "Browse") {
                TextLine(
                    text = stringResource(Res.string.tutorial_profile_import_select_browse),
                    style = ChatTheme.typography.body1,
                    textAlign = TextAlign.Center,
                    modifier = Modifier
                        .onClick { viewModel.openFilePicker() }
                        .fillMaxWidth()
                        .padding(ChatTheme.dimens.spacingLarge),
                )
            }
        }
        VerticalScrollbar(
            listState = listState,
            modifier = Modifier.align(Alignment.CenterEnd),
        )
    }
}