package nand.apps.chat.ui.profile.selector

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.launch
import nand.apps.chat.io.*
import nand.apps.chat.profile.ProfileBrowser
import nand.apps.chat.profile.getProfiles
import nand.apps.chat.profile.isProfileDirectory
import okio.Path

class ProfileSelectorViewModel(
    private val fileSystem: ChatFileSystem,
    private val profileBrowser: ProfileBrowser,
) : ViewModel() {

    val stateData = MutableStateFlow(ProfileSelectorState())

    private val externalProfileDirectory = fileSystem.getDirectory(fileSystem.paths.toxPath.toString())

    init {
        viewModelScope.launch {
            val externalProfiles = externalProfileDirectory.getProfiles().toRowData(isImported = false)
            val importedProfiles = fileSystem.getProfileDirectory().listFiles().filter {
                fileSystem.isProfileDirectory(it)
            }.toRowData(isImported = true)
            val profiles = importedProfiles + externalProfiles
            if (profiles.isNotEmpty()) {
                stateData.emit(stateData.value.copy(rowData = profiles))
            } else {
                openFilePicker()
            }
        }
    }

    fun openFilePicker() {
        if (profileBrowser.isSupported()) {
            profileBrowser.launch(
                onResult = { stateData.tryEmit(stateData.value.copy(selectedProfile = it)) }
            )
        } else {
            stateData.tryEmit(stateData.value.copy(
                filePickerDirectory = fileSystem.paths.documentPath.toString()),
            )
        }
    }

    private fun List<Path>.toRowData(isImported: Boolean) = map {
        ProfileSelectorRowData(
            username = it.name,
            filePath = it.toString(),
            isImported = isImported,
        )
    }
}