package nand.apps.chat.ui.settings

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.text.selection.SelectionContainer
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.Button
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.pointer.PointerIcon
import androidx.compose.ui.input.pointer.pointerHoverIcon
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.BuildConfig.APP_VERSION
import nand.apps.chat.BuildConfig.COMMIT_HASH
import nand.apps.chat.engine.ChatEngine
import nand.apps.chat.ui.provider.LocalUriLauncher
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.toolbar.SimpleNavToolbar
import nand.apps.chat.ui.util.rememberGridBrush
import org.jetbrains.compose.resources.painterResource
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

private const val GITLAB_URL = "https://gitlab.com/nand-apps/seers-lodge"

@Composable
fun AboutScreen(
    modifier: Modifier = Modifier,
) = Column(modifier = modifier) {
    
    val chatEngine: ChatEngine by getKoin().inject()
    val uriLauncher = LocalUriLauncher.current
    
    SimpleNavToolbar(
        text = Res.string.about_screen_title,
    )
    Box(
        contentAlignment = Alignment.Center,
        modifier = Modifier
            .background(
                brush = rememberGridBrush(
                    size = ChatTheme.dimens.iconXLarge,
                    backgroundColor = ChatTheme.colors.background,
                    foregroundColor = ChatTheme.colors.surface,
                )
            )
            .fillMaxSize()
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            modifier = Modifier.verticalScroll(rememberScrollState()),
        ) {
            Image(
                painter = painterResource(Res.drawable.ic_seers),
                contentDescription = APP_NAME,
                modifier = Modifier
                    .size(ChatTheme.dimens.avatarXLarge)
                    .padding(bottom = ChatTheme.dimens.spacingSmall),
            )
            TextLine(
                text = APP_NAME,
                style = ChatTheme.typography.h4,
                color = ChatTheme.colors.onSurface,
            )
            SelectionContainer(
                modifier = Modifier.padding(bottom = ChatTheme.dimens.spacingLarge),
            ) {
                Column(horizontalAlignment = Alignment.CenterHorizontally) {
                    TextLine(
                        text = stringResource(Res.string.about_screen_app_version, APP_VERSION, COMMIT_HASH),
                        style = ChatTheme.typography.body2,
                        color = ChatTheme.colors.onBackground,
                    )
                    TextLine(
                        text = chatEngine.version,
                        style = ChatTheme.typography.body2,
                        color = ChatTheme.colors.onBackground,
                    )
                    TextLine(
                        text = stringResource(Res.string.about_screen_author),
                        style = ChatTheme.typography.body2,
                        color = ChatTheme.colors.onBackground,
                    )
                }
            }
            Button(
                onClick = { uriLauncher.launchUri(GITLAB_URL) },
                modifier = Modifier.pointerHoverIcon(PointerIcon.Hand),
            ) {
                Image(
                    painter = painterResource(Res.drawable.ic_gitlab),
                    contentDescription = stringResource(Res.string.about_screen_gitlab),
                    modifier = Modifier
                        .size(ChatTheme.dimens.iconMedium)
                        .padding(end = ChatTheme.dimens.spacingMedium),
                )
                TextLine(
                    text = stringResource(Res.string.about_screen_gitlab),
                    style = ChatTheme.typography.h6,
                    color = ChatTheme.colors.onPrimary,
                )
            }
        }
    }
}