package nand.apps.chat.ui.settings

import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import nand.apps.chat.ui.navigation.NavRoute
import nand.apps.chat.ui.navigation.SettingsRoute
import nand.apps.chat.ui.settings.call.CallSettingsScreen
import nand.apps.chat.ui.settings.chat.ChatSettingsScreen
import nand.apps.chat.ui.settings.dev.DevSettingsScreen
import nand.apps.chat.ui.settings.embed.EmbedHostsScreen
import nand.apps.chat.ui.settings.embed.EmbedSettingsScreen
import nand.apps.chat.ui.settings.filetransfer.FileTransferAutoAcceptScreen
import nand.apps.chat.ui.settings.filetransfer.FileTransferSettingsScreen
import nand.apps.chat.ui.settings.filetransfer.FileTransferUploadConfigScreen
import nand.apps.chat.ui.settings.general.GeneralSettingsScreen
import nand.apps.chat.ui.settings.net.NetworkSettingsScreen
import nand.apps.chat.ui.settings.notifications.NotificationSettingsScreen
import nand.apps.chat.ui.settings.profile.ProfileSettingsScreen
import nand.apps.chat.ui.settings.shortcut.ShortcutSettingsScreen
import nand.apps.chat.ui.viewmodel.composableViewModel

@Composable
fun SettingsNavGraph(
    route: NavRoute,
    modifier: Modifier = Modifier,
) {
    val viewModel: SettingsViewModel = composableViewModel()
    when (route) {
        SettingsRoute -> SettingsScreen(
            modifier = modifier,
        )
        GeneralSettingsRoute -> GeneralSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        ChatSettingsRoute -> ChatSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        CallSettingsRoute -> CallSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        NotificationSettingsRoute -> NotificationSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        EmbedSettingsRoute -> EmbedSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        EmbedHostsRoute -> EmbedHostsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        FileTransferSettingsRoute -> FileTransferSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        FileTransferAutoAcceptRoute -> FileTransferAutoAcceptScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        FileTransferUploadConfigRoute -> FileTransferUploadConfigScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        ProfileSettingsRoute -> ProfileSettingsScreen(
            modifier = modifier,
        )
        NetworkSettingsRoute -> NetworkSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        ShortcutSettingsRoute -> ShortcutSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        DevSettingsRoute -> DevSettingsScreen(
            viewModel = viewModel,
            modifier = modifier,
        )
        AboutScreenRoute -> AboutScreen(
            modifier = modifier,
        )
    }
}