package nand.apps.chat.ui.settings

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.outlined.Chat
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import nand.apps.chat.ui.provider.LocalNavController
import nand.apps.chat.ui.provider.isLoggedIn
import nand.apps.chat.ui.settings.row.SettingsRow
import seers.composeapp.generated.resources.*

@Composable
fun SettingsScreen(
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings,
    modifier = modifier,
) {
    val navController = LocalNavController.current
    SettingsRow(
        title = Res.string.settings_general_title,
        description = Res.string.settings_general_desc,
        icon = Icons.Outlined.Settings,
        onClick = { navController.navigate(GeneralSettingsRoute) },
    )
    SettingsRow(
        title = Res.string.settings_chat_title,
        description = Res.string.settings_chat_desc,
        icon = Icons.AutoMirrored.Outlined.Chat,
        onClick = { navController.navigate(ChatSettingsRoute) },
    )
    SettingsRow(
        title = Res.string.settings_calls_title,
        description = Res.string.settings_calls_desc,
        icon = Icons.Outlined.SettingsPhone,
        onClick = { navController.navigate(CallSettingsRoute) },
    )
    SettingsRow(
        title = Res.string.settings_notifications_title,
        description = Res.string.settings_notifications_desc,
        icon = Icons.Outlined.Notifications,
        onClick = { navController.navigate(NotificationSettingsRoute) },
    )
    SettingsRow(
        title = Res.string.settings_file_transfer_title,
        description = Res.string.settings_file_transfer_desc,
        icon = Icons.Outlined.MultipleStop,
        onClick = { navController.navigate(FileTransferSettingsRoute) },
    )
    SettingsRow(
        title = Res.string.settings_embed_title,
        description = Res.string.settings_embed_desc,
        icon = Icons.Outlined.SmartDisplay,
        onClick = { navController.navigate(EmbedSettingsRoute) },
    )
    if (isLoggedIn()) {
        SettingsRow(
            title = Res.string.settings_profile_title,
            description = Res.string.settings_profile_desc,
            icon = Icons.Outlined.AccountCircle,
            onClick = { navController.navigate(ProfileSettingsRoute) },
        )
    }
    SettingsRow(
        title = Res.string.settings_shortcuts_title,
        description = Res.string.settings_shortcuts_desc,
        icon = Icons.Outlined.Keyboard,
        onClick = { navController.navigate(ShortcutSettingsRoute) },
    )
    SettingsRow(
        title = Res.string.settings_network_title,
        description = Res.string.settings_network_desc,
        icon = Icons.Outlined.Wifi,
        onClick = { navController.navigate(NetworkSettingsRoute) },
    )
    SettingsRow(
        title = Res.string.settings_dev_title,
        description = Res.string.settings_dev_desc,
        icon = Icons.Outlined.Code,
        onClick = { navController.navigate(DevSettingsRoute) },
    )
    SettingsRow(
        title = Res.string.about_screen_title,
        description = null,
        icon = Icons.Outlined.Info,
        onClick = { navController.navigate(AboutScreenRoute) },
    )
}