package nand.apps.chat.ui.settings.chat

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.outlined.SpeakerNotes
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import nand.apps.chat.time.MINUTE_MILLIS
import nand.apps.chat.ui.settings.SettingsColumn
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.row.SettingsCheckboxRow
import nand.apps.chat.ui.settings.row.SettingsInputDialogRow
import nand.apps.chat.ui.settings.row.SettingsSeparator
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.toIntOrZero
import seers.composeapp.generated.resources.*

@Composable
fun ChatSettingsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_chat_title,
    modifier = modifier,
) {
    val state = viewModel.settings.chatSettings
    SettingsCheckboxRow(
        title = Res.string.settings_chat_text_style_title,
        description = Res.string.settings_chat_text_style_desc,
        icon = Icons.Outlined.TextFields,
        isChecked = state.isTextStylesEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isTextStylesEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_text_fx_title,
        description = Res.string.settings_chat_text_fx_desc,
        icon = Icons.Outlined.Reviews,
        isChecked = state.isTextEffectsEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isTextEffectsEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_avatars_title,
        description = Res.string.settings_chat_avatars_desc,
        icon = Icons.Outlined.AccountBox,
        isChecked = state.isAvatarsEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isAvatarsEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_message_edits_title,
        description = Res.string.settings_chat_message_edits_desc,
        icon = Icons.Outlined.EditNote,
        isChecked = state.isMessageEditsEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isMessageEditsEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_message_reactions_title,
        description = Res.string.settings_chat_message_reactions_desc,
        icon = Icons.Outlined.AddReaction,
        isChecked = state.isMessageReactionsEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isMessageReactionsEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_animated_emojis_title,
        description = Res.string.settings_chat_animated_emojis_desc,
        icon = Icons.Outlined.EmojiObjects,
        isChecked = state.isAnimatedEmojisEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isAnimatedEmojisEnabled = checked))
        }
    )
    SettingsSeparator(
        text = Res.string.settings_chat_groups_section,
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_peer_name_messages_title,
        description = Res.string.settings_chat_peer_name_messages_desc,
        icon = Icons.Outlined.Badge,
        isChecked = state.isPeerNameMessagesEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isPeerNameMessagesEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_peer_connect_messages_title,
        description = Res.string.settings_chat_peer_connect_messages_desc,
        icon = Icons.Outlined.ConnectWithoutContact,
        isChecked = state.isPeerConnectMessagesEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isPeerConnectMessagesEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_mentions_enabled_title,
        description = Res.string.settings_chat_mentions_enabled_desc,
        icon = Icons.Outlined.AlternateEmail,
        isChecked = state.isMentionsEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isMentionsEnabled = checked))
        }
    )
    SettingsSeparator(
        text = Res.string.settings_chat_input_section,
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_text_input_style_title,
        description = Res.string.settings_chat_text_input_style_desc,
        icon = Icons.Outlined.FormatItalic,
        isChecked = state.isInputTextStylesEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isInputTextStylesEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_text_input_style_tag_title,
        description = Res.string.settings_chat_text_input_style_tag_desc,
        icon = Icons.Outlined.AttachMoney,
        isChecked = state.isInputTagPreviewEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isInputTagPreviewEnabled = checked))
        }
    )
    SettingsSeparator(
        text = Res.string.settings_advanced,
    )
    SettingsCheckboxRow(
        title = Res.string.settings_chat_message_history_title,
        description = Res.string.settings_chat_message_history_desc,
        icon = Icons.Outlined.History,
        isChecked = state.isMessageHistoryEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isMessageHistoryEnabled = checked))
        }
    )
    SettingsInputDialogRow(
        title = Res.string.settings_chat_max_messages_title,
        description = Res.string.settings_chat_max_messages_desc,
        label = Res.string.settings_chat_max_messages_label,
        icon = Icons.Outlined.FormatListNumbered,
        value = state.maxMessages.toString(),
        regex = REGEX_NON_DIGITS,
        onConfirm = { maxMessages ->
            viewModel.update(state.copy(maxMessages = maxMessages.toIntOrZero().coerceAtLeast(50)))
        }
    )
    SettingsInputDialogRow(
        title = Res.string.settings_chat_message_split_title,
        description = Res.string.settings_chat_message_split_desc,
        label = Res.string.settings_chat_message_split_label,
        icon = Icons.AutoMirrored.Outlined.SpeakerNotes,
        value = (state.messageMergeThreshold / MINUTE_MILLIS).toString(),
        regex = REGEX_NON_DIGITS,
        onConfirm = { minutes ->
            viewModel.update(state.copy(messageMergeThreshold = minutes.toIntOrZero() * MINUTE_MILLIS))
        }
    )
    SettingsInputDialogRow(
        title = Res.string.settings_chat_max_text_fx_title,
        description = Res.string.settings_chat_max_text_fx_desc,
        label = Res.string.settings_chat_max_text_fx_label,
        icon = Icons.Outlined.Reviews,
        value = state.maxTextEffects.toString(),
        regex = REGEX_NON_DIGITS,
        isEnabled = state.isTextEffectsEnabled,
        onConfirm = { maxEffects ->
            viewModel.update(state.copy(maxTextEffects = maxEffects.toIntOrZero()))
        }
    )
}