package nand.apps.chat.ui.settings.dev

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.Icon
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Warning
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import nand.apps.chat.model.settings.shortcut.ShortcutAction
import nand.apps.chat.platform.getPlatform
import nand.apps.chat.ui.settings.SettingsColumn
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.dialog.DevLogLevelDialog
import nand.apps.chat.ui.settings.row.SettingsCheckboxRow
import nand.apps.chat.ui.settings.row.SettingsDialogRow
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun DevSettingsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_dev_title,
    modifier = modifier,
) {
    val state = viewModel.settings.devSettings
    Row(
        horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
        verticalAlignment = Alignment.CenterVertically,
        modifier = Modifier
            .fillMaxWidth()
            .padding(ChatTheme.dimens.spacingMedium),
    ) {
        Icon(
            imageVector = Icons.Default.Warning,
            contentDescription = stringResource(Res.string.settings_dev_log_level_warning),
            tint = ChatTheme.colors.warning,
        )
        Text(
            text = stringResource(Res.string.settings_dev_disclaimer),
            style = ChatTheme.typography.body2,
            color = ChatTheme.colors.warning,
        )
    }
    if (getPlatform().isDesktop) {
        val devConsoleShortcut = viewModel.settings.shortcutSettings.shortcuts[ShortcutAction.TOGGLE_DEV_CONSOLE]
        SettingsCheckboxRow(
            title = Res.string.settings_dev_console_enabled_title,
            description = Res.string.settings_dev_console_enabled_desc,
            icon = Icons.Outlined.LogoDev,
            isChecked = state.isDevConsoleEnabled,
            onCheckedChange = { checked ->
                viewModel.update(state.copy(isDevConsoleEnabled = checked))
            },
            endContent = {
                devConsoleShortcut?.let {
                    TextLine(
                        text = it,
                        style = ChatTheme.typography.caption,
                        color = ChatTheme.colors.onBackground2,
                    )
                }
            }
        )
    }
    SettingsDialogRow(
        title = Res.string.settings_dev_log_level_title,
        description = Res.string.settings_dev_log_level_desc,
        icon = Icons.Outlined.EditNote,
    ) { onConfirm ->
        DevLogLevelDialog(
            viewModel = viewModel,
            onConfirm = onConfirm,
        )
    }
}