package nand.apps.chat.ui.settings.dialog

import androidx.compose.foundation.layout.Column
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.text.SimpleTextRadioButton
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringArrayResource
import seers.composeapp.generated.resources.*

@Composable
fun BitRateSettingDialog(
    title: StringResource,
    kbpsOptions: List<Int>,
    defaultKbps: Int,
    onConfirm: (Int) -> Unit,
    onCancel: () -> Unit,
) {
    var selectedIndex by rememberSaveable { mutableStateOf(kbpsOptions.indexOf(defaultKbps)) }
    SimpleDialog(
        title = title,
        titleStyle = ChatTheme.typography.body1,
        onConfirm = {
            kbpsOptions.getOrNull(selectedIndex)?.let { onConfirm(it) } ?: onCancel()
        },
        onCancel = onCancel,
    ) {
        Column {
            stringArrayResource(Res.array.settings_bit_rate_options).forEachIndexed { index, option ->
                SimpleTextRadioButton(
                    text = "$option (${kbpsOptions[index]} kbps)",
                    selected = index == selectedIndex,
                    onClick = { selectedIndex = index },
                )
            }
        }
    }
}