package nand.apps.chat.ui.settings.dialog

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.Dialog
import nand.apps.chat.model.settings.notifications.ChatWidgetPlacement
import nand.apps.chat.model.settings.notifications.ChatWidgetSize
import nand.apps.chat.ui.button.SimpleTextButton
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.text.SimpleTextRadioButton
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun ChatWidgetPlacementDialog(
    viewModel: SettingsViewModel,
    onConfirm: () -> Unit,
) {
    val state = viewModel.settings.notificationSettings.widgetSettings
    Dialog(
        onDismissRequest = onConfirm,
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingSmall),
            modifier = Modifier
                .background(
                    color = ChatTheme.colors.background,
                    shape = ChatTheme.shapes.small,
                )
                .padding(ChatTheme.dimens.spacingMedium)
                .fillMaxWidth(),
        ) {
            Box(
                modifier = Modifier
                    .clip(ChatTheme.shapes.medium)
                    .background(ChatTheme.colors.surface)
                    .size(
                        width = 320.dp,
                        height = 180.dp,
                    )
            ) {
                ChatWidgetPlacement.entries.forEach { placement ->
                    Box(
                        modifier = Modifier
                            .align(placement.toAlignment())
                            .clip(ChatTheme.shapes.medium)
                            .background(
                                when {
                                    state.placement == placement -> ChatTheme.colors.primary
                                    else -> ChatTheme.colors.surface3
                                }
                            )
                            .size(ChatTheme.dimens.iconSmall)
                            .onClick { viewModel.update(state.copy(placement = placement)) },
                    )
                }
            }
            Row {
                ChatWidgetSize.entries.forEach { size ->
                    SimpleTextRadioButton(
                        text = stringResource(
                            when (size) {
                                ChatWidgetSize.SMALL -> Res.string.chat_widget_size_small
                                ChatWidgetSize.MEDIUM -> Res.string.chat_widget_size_medium
                                ChatWidgetSize.LARGE -> Res.string.chat_widget_size_large
                            }
                        ),
                        selected = state.size == size,
                        onClick = { viewModel.update(state.copy(size = size)) },
                    )
                }
            }
            SimpleTextButton(
                text = stringResource(Res.string.confirm),
                onClick = onConfirm,
            )
        }
    }
}