package nand.apps.chat.ui.settings.dialog

import androidx.compose.foundation.layout.Column
import androidx.compose.material.Slider
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import nand.apps.chat.model.settings.DENSITY_RANGE
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.clamp
import kotlin.math.roundToInt
import seers.composeapp.generated.resources.*

@Composable
fun DensitySettingDialog(
    viewModel: SettingsViewModel,
    onConfirm: () -> Unit,
) {
    val state = viewModel.settings.generalSettings
    var density by rememberSaveable { mutableStateOf(state.densityScale) }
    SimpleDialog(
        title = Res.string.settings_general_density_title,
        titleStyle = ChatTheme.typography.body1,
        confirmLabel = null,
        cancelLabel = Res.string.confirm,
        onCancel = { onConfirm() },
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            TextLine(
                text = "${(density.roundToNearest() * 100).roundToInt()}%",
                style = ChatTheme.typography.body1,
            )
            Slider(
                value = density,
                valueRange = DENSITY_RANGE,
                steps = 6,
                onValueChange = { value -> density = value },
                onValueChangeFinished = {
                    viewModel.update(state.copy(densityScale = density.roundToNearest()))
                }
            )
        }
    }
}

private fun Float.roundToNearest() = ((this * 4).roundToInt() / 4f).clamp(DENSITY_RANGE)