package nand.apps.chat.ui.settings.dialog

import androidx.compose.foundation.layout.*
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import nand.apps.chat.log.ChatLogLevel
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.text.SimpleTextRadioButton
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun DevLogLevelDialog(
    viewModel: SettingsViewModel,
    onConfirm: () -> Unit,
) {
    val settings = viewModel.settings.devSettings
    val logLevel = rememberSaveable(settings) { mutableStateOf(settings.logLevel) }
    SimpleDialog(
        title = Res.string.settings_dev_log_level_title,
        titleStyle = ChatTheme.typography.body1,
        onConfirm = {
            viewModel.update(settings.copy(logLevel = logLevel.value))
            onConfirm()
        },
        onCancel = onConfirm,
    ) {
        Column {
            LevelButton(
                text = Res.string.settings_dev_log_level_verbose,
                level = ChatLogLevel.VERBOSE,
                state = logLevel,
            )
            LevelButton(
                text = Res.string.settings_dev_log_level_debug,
                level = ChatLogLevel.DEBUG,
                state = logLevel,
            )
            LevelButton(
                text = Res.string.settings_dev_log_level_warning,
                level = ChatLogLevel.WARNING,
                state = logLevel,
            )
            LevelButton(
                text = Res.string.settings_dev_log_level_error,
                level = ChatLogLevel.ERROR,
                state = logLevel,
            )
        }
    }
}

@Composable
private fun LevelButton(
    text: StringResource,
    state: MutableState<ChatLogLevel>,
    level: ChatLogLevel,
) = SimpleTextRadioButton(
    text = stringResource(text),
    selected = state.value == level,
    onClick = { state.value = level },
)