package nand.apps.chat.ui.settings.dialog

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.BasicText
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import nand.apps.chat.locale.ChatLocale
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.theme.ChatTheme
import seers.composeapp.generated.resources.*

@Composable
fun LocaleSettingDialog(
    onConfirm: (ChatLocale) -> Unit,
    onCancel: () -> Unit,
) {
    SimpleDialog(
        title = Res.string.settings_general_locale_title,
        titleStyle = ChatTheme.typography.body1,
        confirmLabel = null,
        onCancel = onCancel,
    ) {
        Row(
            horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
            modifier = Modifier.align(Alignment.Center),
        ) {
            ChatLocale.entries.forEach { locale ->
                BasicText(
                    text = locale.emoji,
                    style = ChatTheme.typography.h3,
                    modifier = Modifier
                        .clip(ChatTheme.shapes.small)
                        .background(ChatTheme.colors.background)
                        .onClick { onConfirm(locale) }
                        .padding(ChatTheme.dimens.spacingSmall),
                )
            }
        }
    }
}