package nand.apps.chat.ui.settings.dialog

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import androidx.compose.ui.text.input.KeyboardType
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.modifier.onEnter
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.REGEX_NON_DIGITS
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource

@Composable
fun SettingsInputDialog(
    title: StringResource,
    label: StringResource,
    initialValue: String,
    onConfirm: (String) -> Unit,
    onCancel: () -> Unit,
    regex: Regex? = null,
) {
    var input by rememberTextFieldValue(initialValue)
    val focusRequester = remember { FocusRequester() }
    LaunchedEffect(Unit) {
        focusRequester.requestFocus()
    }
    SimpleDialog(
        title = title,
        titleStyle = ChatTheme.typography.body1,
        onConfirm = { onConfirm(input.text) },
        onCancel = onCancel,
    ) {
        SimpleTextField(
            value = input,
            onValueChange = { value ->
                input = regex?.let { value.copy(text = value.text.replace(regex, "")) } ?: value
            },
            labelText = stringResource(label),
            keyboardOptions = KeyboardOptions(
                autoCorrect = false,
                keyboardType = if (regex === REGEX_NON_DIGITS) KeyboardType.Number else KeyboardType.Text,
            ),
            keyboardActions = KeyboardActions(
                onDone = { onConfirm(input.text) },
            ),
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = ChatTheme.dimens.spacingMedium)
                .focusRequester(focusRequester)
                .onEnter { onConfirm(input.text) },
        )
    }
}