package nand.apps.chat.ui.settings.embed

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import nand.apps.chat.ui.provider.LocalNavController
import nand.apps.chat.ui.settings.*
import nand.apps.chat.ui.settings.row.SettingsCheckboxRow
import nand.apps.chat.ui.settings.row.SettingsRow
import seers.composeapp.generated.resources.*

@Composable
fun EmbedSettingsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_embed_title,
    modifier = modifier,
) {
    val state = viewModel.settings.embedSettings
    val navController = LocalNavController.current
    SettingsCheckboxRow(
        title = Res.string.settings_embed_enabled_title,
        description = Res.string.settings_embed_enabled_desc,
        icon = Icons.Outlined.SmartDisplay,
        isChecked = state.isEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_embed_images_title,
        description = Res.string.settings_embed_images_desc,
        icon = Icons.Outlined.PhotoLibrary,
        isChecked = state.isImagesEnabled,
        isEnabled = state.isEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isImagesEnabled = checked))
        }
    )
    SettingsRow(
        title = Res.string.settings_embed_hosts_title,
        description = Res.string.settings_embed_hosts_desc,
        icon = Icons.Outlined.Language,
        isEnabled = state.isEnabled,
        onClick = { navController.navigate(EmbedHostsRoute) },
    )
}