package nand.apps.chat.ui.settings.filetransfer

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.filled.Delete
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.settings.*
import nand.apps.chat.ui.text.SimpleTextFieldButton
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun FileTransferAutoAcceptScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_file_transfer_auto_accept_title,
    modifier = modifier,
    isScrollable = false,
) {
    val state = viewModel.settings.fileSettings
    var input by rememberTextFieldValue()
    
    Text(
        text = stringResource(Res.string.settings_file_transfer_auto_accept_header),
        style = ChatTheme.typography.body1,
        color = ChatTheme.colors.onBackground,
    )
    
    SimpleTextFieldButton(
        value = input,
        hintText = stringResource(Res.string.settings_file_transfer_auto_accept_hint),
        buttonIcon = Icons.Filled.Add,
        buttonDescription = stringResource(Res.string.settings_file_transfer_auto_accept_add_rule),
        onValueChange = { input = it },
        onSubmit = {
            val extension = it.text.trimStart('.', '*', ' ').lowercase()
            if (extension.isNotBlank() && extension !in state.autoAcceptFileTypes) {
                viewModel.update(state.copy(autoAcceptFileTypes = state.autoAcceptFileTypes + extension))
            }
        },
        modifier = Modifier
            .fillMaxWidth()
            .padding(top = ChatTheme.dimens.spacingMedium),
    )

    val fileTypes = remember(state.autoAcceptFileTypes) {
        state.autoAcceptFileTypes.toList()
    }
    LazyColumn(
        verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
        modifier = Modifier
            .padding(top = ChatTheme.dimens.spacingMedium)
            .clip(ChatTheme.shapes.medium)
            .background(ChatTheme.colors.surface)
            .fillMaxSize()
            .padding(ChatTheme.dimens.spacingMedium),
    ) {
        items(fileTypes) { type ->
            Row(
                modifier = Modifier.fillMaxWidth()
            ) {
                TextLine(
                    text = "*.$type",
                    style = ChatTheme.typography.body2,
                    modifier = Modifier.weight(1f),
                )
                IconButton(
                    icon = Icons.Filled.Delete,
                    contentDescription = stringResource(Res.string.settings_file_transfer_auto_accept_remove_rule),
                    onClick = {
                        viewModel.update(state.copy(autoAcceptFileTypes = state.autoAcceptFileTypes - type))
                    },
                )
            }
        }
    }
}