package nand.apps.chat.ui.settings.filetransfer

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import nand.apps.chat.platform.getPlatform
import nand.apps.chat.time.HOUR_MILLIS
import nand.apps.chat.ui.provider.LocalNavController
import nand.apps.chat.ui.settings.*
import nand.apps.chat.ui.settings.row.*
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.toIntOrZero
import seers.composeapp.generated.resources.*

@Composable
fun FileTransferSettingsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_file_transfer_title,
    modifier = modifier,
) {
    val state = viewModel.settings.fileSettings
    val navController = LocalNavController.current
    SettingsFilePickerRow(
        title = Res.string.settings_file_transfer_download_directory_title,
        description = Res.string.settings_file_transfer_download_directory_desc,
        icon = Icons.Outlined.SystemUpdateAlt,
        path = state.downloadDirectory,
        onConfirm = { viewModel.update(state.copy(downloadDirectory = it)) },
    )
    SettingsCheckboxRow(
        title = Res.string.settings_file_transfer_avatars_title,
        description = Res.string.settings_file_transfer_avatars_desc,
        icon = Icons.Outlined.ContactMail,
        isChecked = state.isAvatarTransferEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isAvatarTransferEnabled = checked))
        }
    )
    if (getPlatform().isDesktop) {
        SettingsCheckboxRow(
            title = Res.string.settings_file_transfer_drag_title,
            description = Res.string.settings_file_transfer_drag_desc,
            icon = Icons.Outlined.PanToolAlt,
            isChecked = state.isDragDropEnabled,
            onCheckedChange = { checked ->
                viewModel.update(state.copy(isDragDropEnabled = checked))
            }
        )
        SettingsCheckboxRow(
            title = Res.string.settings_file_transfer_paste_title,
            description = Res.string.settings_file_transfer_paste_desc,
            icon = Icons.Outlined.ContentPasteGo,
            isChecked = state.isCopyPasteEnabled,
            onCheckedChange = { checked ->
                viewModel.update(state.copy(isCopyPasteEnabled = checked))
            }
        )
        SettingsCheckboxRow(
            title = Res.string.settings_file_transfer_group_upload_title,
            description = Res.string.settings_file_transfer_group_upload_desc,
            icon = Icons.Outlined.CloudUpload,
            isChecked = state.isGroupUploadEnabled,
            onCheckedChange = { checked ->
                viewModel.update(state.copy(isGroupUploadEnabled = checked))
            }
        )
    }
    SettingsRow(
        title = Res.string.settings_file_transfer_auto_accept_title,
        description = Res.string.settings_file_transfer_auto_accept_desc,
        icon = Icons.Outlined.Description,
        onClick = { navController.navigate(FileTransferAutoAcceptRoute) },
    )
    SettingsSeparator(
        text = Res.string.settings_advanced,
    )
    SettingsRow(
        title = Res.string.settings_file_transfer_upload_config_title,
        description = Res.string.settings_file_transfer_upload_config_desc,
        icon = Icons.Outlined.CloudSync,
        onClick = { navController.navigate(FileTransferUploadConfigRoute) },
    )
    SettingsInputDialogRow(
        title = Res.string.settings_file_transfer_avatar_expiration_title,
        description = Res.string.settings_file_transfer_avatar_expiration_desc,
        label = Res.string.settings_file_transfer_avatar_expiration_label,
        icon = Icons.Outlined.PersonPin,
        value = (state.avatarExpireTime / HOUR_MILLIS).toString(),
        regex = REGEX_NON_DIGITS,
        onConfirm = { hours ->
            viewModel.update(state.copy(avatarExpireTime = hours.toIntOrZero() * HOUR_MILLIS))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_file_transfer_confirmation_enabled_title,
        description = Res.string.settings_file_transfer_confirmation_enabled_desc,
        icon = Icons.Outlined.UploadFile,
        isChecked = state.showFileTransferConfirmation,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(showFileTransferConfirmation = checked))
        }
    )
}