package nand.apps.chat.ui.settings.general

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.DarkMode
import androidx.compose.material.icons.outlined.LightMode
import androidx.compose.material.icons.outlined.ModeStandby
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import nand.apps.chat.locale.isAppLocaleSupported
import nand.apps.chat.model.settings.theme.getDefaultChatTheme
import nand.apps.chat.time.MINUTE_MILLIS
import nand.apps.chat.ui.settings.SettingsColumn
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.row.*
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.toIntOrZero
import seers.composeapp.generated.resources.*

@Composable
fun GeneralSettingsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_general_title,
    modifier = modifier,
) {
    val state = viewModel.settings.generalSettings
    if (isAppLocaleSupported()) {
        LocaleSettingRow(viewModel)
    }
    TimeSettingRow(
        viewModel = viewModel,
    )
    SettingsInputDialogRow(
        title = Res.string.settings_general_auto_away_title,
        description = Res.string.settings_general_auto_away_desc,
        label = Res.string.settings_general_auto_away_label,
        icon = Icons.Outlined.ModeStandby,
        value = (state.autoAwayTime / MINUTE_MILLIS).toString(),
        regex = REGEX_NON_DIGITS,
        onConfirm = { hours ->
            viewModel.update(state.copy(autoAwayTime = hours.toIntOrZero() * MINUTE_MILLIS))
        }
    )
    DensitySettingRow(
        viewModel = viewModel,
    )
    SettingsSeparator(
        text = Res.string.settings_general_theme_section,
    )
    ColorSettingRow(
        viewModel = viewModel,
    )
    SettingsRow(
        title = Res.string.settings_general_reset_theme_title,
        description = Res.string.settings_general_reset_theme_desc,
        icon = if (ChatTheme.colors.isLight) Icons.Outlined.LightMode else Icons.Outlined.DarkMode,
        onClick = {
            viewModel.update(
                state.copy(
                    appTheme = getDefaultChatTheme(),
                    appThemeHue = -1f,
                )
            )
        },
    )
}