package nand.apps.chat.ui.settings.net

import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import nand.apps.chat.ui.settings.SettingsColumn
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.dialog.NetworkProxySettingDialog
import nand.apps.chat.ui.settings.row.SettingsCheckboxRow
import nand.apps.chat.ui.settings.row.SettingsDialogRow
import seers.composeapp.generated.resources.*

@Composable
fun NetworkSettingsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_network_title,
    modifier = modifier,
) {
    val state = viewModel.settings.networkSettings
    SettingsCheckboxRow(
        title = Res.string.settings_network_udp_enabled_title,
        description = Res.string.settings_network_udp_enabled_desc,
        icon = Icons.Outlined.LeakAdd,
        isChecked = state.isUdpEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isUdpEnabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_network_ipv6_enabled_title,
        description = Res.string.settings_network_ipv6_enabled_desc,
        icon = Icons.Outlined.Domain,
        isChecked = state.isIPv6Enabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isIPv6Enabled = checked))
        }
    )
    SettingsCheckboxRow(
        title = Res.string.settings_network_lan_enabled_title,
        description = Res.string.settings_network_lan_enabled_desc,
        icon = Icons.Outlined.Lan,
        isChecked = state.isLanEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isLanEnabled = checked))
        }
    )
    SettingsDialogRow(
        title = Res.string.settings_network_proxy_title,
        description = Res.string.settings_network_proxy_desc,
        icon = Icons.Outlined.VpnLock,
    ) { onConfirm ->
        NetworkProxySettingDialog(
            onUpdateProxy = { viewModel.update(state.copy(proxy = it)) },
            onDismiss = onConfirm,
        )
    }
}