package nand.apps.chat.ui.settings.row

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Language
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import nand.apps.chat.locale.setAppLocale
import nand.apps.chat.ui.provider.LocalNavController
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.dialog.LocaleSettingDialog
import seers.composeapp.generated.resources.*

@Composable
fun LocaleSettingRow(viewModel: SettingsViewModel) {
    val settings = viewModel.settings.generalSettings
    var isDialogVisible by rememberSaveable { mutableStateOf(false) }
    val navController = LocalNavController.current
    SettingsRow(
        title = Res.string.settings_general_locale_title,
        description = Res.string.settings_general_locale_desc,
        icon = Icons.Outlined.Language,
        onClick = { isDialogVisible = true },
        modifier = Modifier.fillMaxWidth(),
        endContent = {
            if (isDialogVisible) {
                LocaleSettingDialog(
                    onConfirm = { locale ->
                        setAppLocale(locale)
                        viewModel.update(settings.copy(locale = locale))
                        if (navController.canNavigateBack) {
                            // Workaround to force complete re-composition so the text updates take effect
                            navController.navigateBack()
                        }
                    },
                    onCancel = { isDialogVisible = false },
                )
            }
        },
    )
}