package nand.apps.chat.ui.settings.row

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.io.ChatFilePaths
import nand.apps.chat.ui.file.ChatFilePickerDialog
import nand.apps.chat.ui.file.ChatFilePickerMode
import nand.apps.chat.ui.file.ChatFilePickerOptions
import org.jetbrains.compose.resources.StringResource
import org.koin.compose.getKoin

@Composable
fun SettingsFilePickerRow(
    title: StringResource,
    description: StringResource,
    icon: ImageVector,
    path: String?,
    onConfirm: (String) -> Unit,
    modifier: Modifier = Modifier.fillMaxWidth(),
) {
    val filePaths: ChatFilePaths by getKoin().inject()
    var isDialogVisible by rememberSaveable { mutableStateOf(false) }
    SettingsRow(
        title = title,
        description = description,
        icon = icon,
        onClick = { isDialogVisible = true },
        modifier = modifier,
        endContent = {
            if (isDialogVisible) {
                ChatFilePickerDialog(
                    options = ChatFilePickerOptions(
                        directory = path ?: filePaths.downloadPath.toString(),
                        mode = ChatFilePickerMode.DIRECTORIES_ONLY,
                    ),
                    onConfirm = {
                        isDialogVisible = false
                        onConfirm(it)
                    },
                    onCancel = { isDialogVisible = false },
                )
            }
        },
    )
}