package nand.apps.chat.ui.settings.row

import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.ui.settings.dialog.SettingsInputDialog
import org.jetbrains.compose.resources.StringResource

@Composable
fun SettingsInputDialogRow(
    title: StringResource,
    description: StringResource,
    label: StringResource,
    icon: ImageVector,
    value: String,
    onConfirm: (String) -> Unit,
    modifier: Modifier = Modifier.fillMaxWidth(),
    regex: Regex? = null,
    isEnabled: Boolean = true,
) {
    var isDialogVisible by rememberSaveable { mutableStateOf(false) }
    SettingsRow(
        title = title,
        description = description,
        icon = icon,
        onClick = { isDialogVisible = true },
        isEnabled = isEnabled,
        modifier = modifier,
        endContent = {
            if (isDialogVisible) {
                SettingsInputDialog(
                    title = title,
                    label = label,
                    initialValue = value,
                    regex = regex,
                    onConfirm = { value ->
                        isDialogVisible = false
                        onConfirm(value)
                    },
                    onCancel = { isDialogVisible = false },
                )
            }
        },
    )
}