package nand.apps.chat.ui.shortcut

import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberUpdatedState
import androidx.compose.ui.input.key.KeyEvent
import nand.apps.chat.ui.dispatcher.DisposableEventHandler

@Composable
fun KeyEventHandler(
    isEnabled: Boolean = true,
    onKeyEvent: (KeyEvent) -> Boolean,
) {
    val currentOnKeyEvent by rememberUpdatedState(onKeyEvent)
    val keyEventCallback = remember {
        object : KeyEventCallback(isEnabled) {
            override fun onKeyEvent(event: KeyEvent): Boolean {
                return if (this.isEnabled) {
                    currentOnKeyEvent(event)
                } else {
                    false
                }
            }
        }
    }
    DisposableEventHandler(
        eventCallback = keyEventCallback,
        eventDispatcher = LocalKeyEventDispatcher.current,
        isEnabled = isEnabled,
    )
}