package nand.apps.chat.ui.spinner

import androidx.compose.animation.core.InfiniteTransition
import androidx.compose.animation.core.LinearEasing
import androidx.compose.animation.core.RepeatMode
import androidx.compose.animation.core.rememberInfiniteTransition
import androidx.compose.foundation.Canvas
import androidx.compose.foundation.layout.size
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.Dp
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.toRadians
import kotlin.math.*

/**
 * Adapted from https://github.com/commandiron/ComposeLoading/blob/master/compose-loading/src/main/java/com/commandiron/compose_loading/ChasingDots.kt
 */
@Composable
fun ChasingDots(
    modifier: Modifier = Modifier,
    durationMillis: Int = 2000,
    delayBetweenDotsMillis: Int = 50,
    size: Dp = ChatTheme.dimens.iconLarge,
    color: Color = ChatTheme.colors.primary,
    circleRatio: Float = 0.25f
) {
    val transition = rememberInfiniteTransition()
    val pathAngleTransitions = Array(6) { i ->
        transition.pathAngleTransition(
            offset = i,
            durationMillis = durationMillis * 4,
            delayBetweenDotsMillis = delayBetweenDotsMillis,
        )
    }
    val circleRadiusInitialValues = arrayOf(0.512f, 0.64f, 0.8f, 1f)
    val circleRadiusTransitions = Array(4) { i ->
        transition.circleRadiusTransition(
            initialValue = circleRadiusInitialValues[i],
            circleRatio = circleRatio,
            durationMillis = durationMillis / 2,
        )
    }
    Canvas(modifier = modifier.size(size)) {

        val pathRadius = (this.size.height / 2)
        val radius = this.size.height / 5
        val radiusCommon = radius * circleRatio

        for (i in 0..<6) {
            val angle = (pathAngleTransitions[i].value * 360.0).toRadians()
            val offsetX = -(pathRadius * sin(angle)).toFloat() + (this.size.width / 2)
            val offsetY = (pathRadius * cos(angle)).toFloat() + (this.size.height / 2)
            drawCircle(
                color = color,
                radius = if (i < 2) radiusCommon else circleRadiusTransitions[i - 2].value * radius,
                center = Offset(offsetX,  offsetY)
            )
        }
    }
}

@Composable
private fun InfiniteTransition.pathAngleTransition(
    durationMillis: Int,
    delayBetweenDotsMillis: Int,
    offset: Int,
) = fractionTransition(
    initialValue = 0f,
    targetValue = 7f,
    fraction = 4,
    durationMillis = durationMillis,
    offsetMillis = delayBetweenDotsMillis * offset,
    easing = EaseInOut
)

@Composable
private fun InfiniteTransition.circleRadiusTransition(
    initialValue: Float,
    circleRatio: Float,
    durationMillis: Int,
) = fractionTransition(
    initialValue = initialValue,
    targetValue = circleRatio,
    durationMillis = durationMillis,
    repeatMode = RepeatMode.Reverse,
    easing = LinearEasing
)