package nand.apps.chat.ui.spinner

import androidx.compose.animation.core.RepeatMode
import androidx.compose.animation.core.rememberInfiniteTransition
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.State
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.Dp
import nand.apps.chat.ui.theme.ChatTheme

/**
 * Adapted from https://github.com/commandiron/ComposeLoading/blob/master/compose-loading/src/main/java/com/commandiron/compose_loading/CubeGrid.kt
 */
@Composable
fun CubeGrid(
    modifier: Modifier = Modifier,
    size: Dp = ChatTheme.dimens.iconLarge,
    rows: Int = 3,
    durationMillis: Int = 1000,
    color: Color = ChatTheme.colors.primary,
) {
    val transition = rememberInfiniteTransition()
    val durationPerFraction = durationMillis / 2
    val delayPerFraction = durationMillis / 4
    val offsetPerFraction = durationPerFraction / 4
    val rectSizeMultipliers = Array((rows * 2) - 1) { i ->
        transition.fractionTransition(
            initialValue = 1f,
            targetValue = 0f,
            durationMillis = durationPerFraction,
            delayMillis = delayPerFraction,
            offsetMillis = offsetPerFraction * i,
            repeatMode = RepeatMode.Reverse,
            easing = EaseInOut
        )
    }
    
    val boxSize = size / rows
    
    Column(
        modifier = modifier,
        horizontalAlignment = Alignment.CenterHorizontally,
        verticalArrangement = Arrangement.Center
    ) {
        for (y in 0..<rows) {
            val start = rows - y - 1
            Row {
                for (x in 0..<rows) {
                    NestedBox(boxSize, rectSizeMultipliers[start + x], color)
                }
            }
        }
    }
}

@Composable
private fun NestedBox(
    size: Dp,
    multiplier: State<Float>,
    color: Color,
) {
    Box(modifier = Modifier.size(size), contentAlignment = Alignment.Center) {
        Box(
            modifier = Modifier
                .background(color)
                .size(size * multiplier.value),
        )
    }
}