package nand.apps.chat.ui.text

import androidx.compose.foundation.gestures.awaitEachGesture
import androidx.compose.foundation.text.ClickableText
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.pointer.PointerEventType
import androidx.compose.ui.input.pointer.PointerIcon
import androidx.compose.ui.input.pointer.pointerHoverIcon
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.ExperimentalTextApi
import androidx.compose.ui.text.TextLayoutResult
import androidx.compose.ui.text.TextStyle
import nand.apps.chat.ui.provider.LocalUriLauncher

@OptIn(ExperimentalTextApi::class)
@Composable
fun ClickableMessageText(
    text: AnnotatedString,
    modifier: Modifier = Modifier,
    style: TextStyle = TextStyle.Default,
    color: Color = Color.Unspecified,
    onClick: (Int) -> Unit = {},
    showUrlHoverIcon: Boolean = true,
) {
    val uriLauncher = LocalUriLauncher.current
    val layoutResult = remember { mutableStateOf<TextLayoutResult?>(null) }
    var isHoveringUrl by remember { mutableStateOf(false) }
    ClickableText(
        text = text,
        style = style.run {
            if (color != Color.Unspecified) copy(color = color) else this
        },
        modifier = modifier
            .pointerInput(showUrlHoverIcon) {
                if (showUrlHoverIcon) {
                    awaitEachGesture {
                        val event = awaitPointerEvent()
                        when (event.type) {
                            PointerEventType.Enter,
                            PointerEventType.Move -> {
                                val pos = event.changes.firstOrNull()?.position
                                val layout = layoutResult.value
                                if (pos != null && layout != null) {
                                    val index = layout.getOffsetForPosition(pos)
                                    isHoveringUrl = text.getUrlAnnotations(index, index).isNotEmpty()
                                }
                            }
                            PointerEventType.Exit -> isHoveringUrl = false
                        }
                    }
                }
            }
            .pointerHoverIcon(
                when {
                    isHoveringUrl -> PointerIcon.Hand
                    else -> PointerIcon.Default
                }
            ),
        onClick = { index ->
            onClick(index)
            text.getUrlAnnotations(index, index).firstOrNull()?.let { range ->
                uriLauncher.launchUri(range.item.url)
            }
        },
        onTextLayout = { layoutResult.value = it },
    )
}