package nand.apps.chat.ui.text

import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Visibility
import androidx.compose.material.icons.filled.VisibilityOff
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.text.input.TextFieldValue
import androidx.compose.ui.text.input.VisualTransformation
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.Res
import seers.composeapp.generated.resources.hide_password
import seers.composeapp.generated.resources.password
import seers.composeapp.generated.resources.show_password

@Composable
fun SimplePasswordField(
    password: TextFieldValue,
    modifier: Modifier = Modifier,
    label: StringResource = Res.string.password,
    isEnabled: Boolean = true,
    isError: Boolean = false,
    keyboardActions: KeyboardActions = KeyboardActions(),
    onValueChange: (TextFieldValue) -> Unit = {},
    onToggleVisibility: () -> Unit = {},
    onSubmit: (TextFieldValue) -> Unit = {},
) {
    var isPasswordVisible by rememberSaveable { mutableStateOf(false) }
    SimpleTextField(
        value = password,
        onValueChange = onValueChange,
        labelText = stringResource(label),
        visualTransformation = if (isPasswordVisible) VisualTransformation.None else PasswordVisualTransformation(),
        trailingIcon = {
            if (isPasswordVisible) {
                nand.apps.chat.ui.button.IconButton(
                    icon = Icons.Filled.Visibility,
                    contentDescription = stringResource(Res.string.hide_password),
                    onClick = {
                        isPasswordVisible = false
                        onToggleVisibility()
                    },
                )
            } else {
                nand.apps.chat.ui.button.IconButton(
                    icon = Icons.Filled.VisibilityOff,
                    contentDescription = stringResource(Res.string.show_password),
                    color = ChatTheme.colors.onBackground2,
                    onClick = {
                        isPasswordVisible = true
                        onToggleVisibility()
                    },
                )
            }
        },
        onSubmit = onSubmit,
        keyboardOptions = KeyboardOptions(
            autoCorrect = false,
            keyboardType = KeyboardType.Password,
        ),
        keyboardActions = keyboardActions,
        isEnabled = isEnabled,
        isError = isError,
        modifier = modifier,
    )
}