package nand.apps.chat.ui.text

import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.padding
import androidx.compose.material.Checkbox
import androidx.compose.material.CheckboxDefaults
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.TextStyle
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.theme.ChatTheme

@Composable
fun SimpleTextCheckbox(
    text: String,
    isChecked: Boolean,
    onCheckedChange: (Boolean) -> Unit,
    modifier: Modifier = Modifier,
    isEnabled: Boolean = true,
    textColor: Color = ChatTheme.colors.onSurface,
    textStyle: TextStyle = ChatTheme.typography.body2,
) = Row(
    verticalAlignment = Alignment.CenterVertically,
    modifier = modifier.onClick { onCheckedChange(!isChecked) },
) {
    Checkbox(
        checked = isChecked,
        enabled = isEnabled,
        onCheckedChange = onCheckedChange,
        colors = CheckboxDefaults.colors(
            checkmarkColor = ChatTheme.colors.onSecondary,
            checkedColor = ChatTheme.colors.surface3,
        ),
    )
    TextLine(
        text = text,
        style = textStyle,
        color = textColor,
        modifier = Modifier.padding(end = ChatTheme.dimens.spacingMedium),
    )
}