package nand.apps.chat.ui.theme

import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.*
import androidx.compose.runtime.*
import androidx.compose.ui.unit.dp

private val LocalColors = staticCompositionLocalOf { DarkChatColors }
private val LocalDimensions = staticCompositionLocalOf { ChatThemeDimensions() }

object ChatTheme {

    val colors: ChatThemeColors
        @Composable
        @ReadOnlyComposable
        get() = LocalColors.current

    val typography: Typography
        @Composable
        @ReadOnlyComposable
        get() = MaterialTheme.typography
    
    val shapes: Shapes
        @Composable
        @ReadOnlyComposable
        get() = MaterialTheme.shapes
    
    val dimens: ChatThemeDimensions
        @Composable
        @ReadOnlyComposable
        get() = LocalDimensions.current
}

class ChatThemeScope(
    val colors: ChatThemeColors,
    val typography: Typography,
    val shapes: Shapes,
    val dimens: ChatThemeDimensions,
)

@Composable
fun ChatTheme(
    colors: ChatThemeColors = DarkChatColors,
    content: @Composable () -> Unit,
) {
    val providers = getChatThemeProviders(colors) + (LocalColors provides colors)
    CompositionLocalProvider(*providers.toTypedArray()) {
        MaterialTheme(
            colors = colors.material,
            shapes = Shapes(
                small = RoundedCornerShape(6.dp),
            ),
            content = content,
        )
    }
}

@Composable
fun <T> ChatThemeScope(content: @Composable ChatThemeScope.() -> T) = ChatThemeScope(
    colors = ChatTheme.colors,
    typography = ChatTheme.typography,
    shapes = ChatTheme.shapes,
    dimens = ChatTheme.dimens,
).content()

expect fun getChatThemeProviders(colors: ChatThemeColors): List<ProvidedValue<*>>