package nand.apps.chat.ui.theme

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import nand.apps.chat.ui.text.TextLine

@Composable
fun ThemePreview(
    modifier: Modifier = Modifier,
) {
    Column(
        modifier = modifier.background(ChatTheme.colors.background),
    ) {
        Row(modifier = Modifier.weight(1f)) {
            ColorBox(
                label = "background",
                textColor = ChatTheme.colors.onBackground,
                bgColor = ChatTheme.colors.background,
            )
            ColorBox(
                label = "onBackground2",
                textColor = ChatTheme.colors.onBackground2,
                bgColor = ChatTheme.colors.background,
            )
        }
        Row(modifier = Modifier.weight(1f)) {
            ColorBox(
                label = "surface",
                textColor = ChatTheme.colors.onSurface,
                bgColor = ChatTheme.colors.surface,
            )
            ColorBox(
                label = "surface2",
                textColor = ChatTheme.colors.onSurface,
                bgColor = ChatTheme.colors.surface2,
            )
            ColorBox(
                label = "surface3",
                textColor = ChatTheme.colors.onSurface,
                bgColor = ChatTheme.colors.surface3,
            )
        }
        Row(modifier = Modifier.weight(1f)) {
            ColorBox(
                label = "statusOnline",
                textColor = ChatTheme.colors.statusOnline,
                bgColor = ChatTheme.colors.surface,
            )
            ColorBox(
                label = "statusAway",
                textColor = ChatTheme.colors.statusAway,
                bgColor = ChatTheme.colors.surface,
            )
            ColorBox(
                label = "statusBusy",
                textColor = ChatTheme.colors.statusBusy,
                bgColor = ChatTheme.colors.surface,
            )
            ColorBox(
                label = "statusOffline",
                textColor = ChatTheme.colors.statusOffline,
                bgColor = ChatTheme.colors.surface,
            )
        }
        Row(modifier = Modifier.weight(1f)) {
            ColorBox(
                label = "primary",
                textColor = ChatTheme.colors.onPrimary,
                bgColor = ChatTheme.colors.primary,
            )
            ColorBox(
                label = "primaryVariant",
                textColor = ChatTheme.colors.onPrimary,
                bgColor = ChatTheme.colors.primaryVariant,
            )
        }
        Row(modifier = Modifier.weight(1f)) {
            ColorBox(
                label = "secondary",
                textColor = ChatTheme.colors.onSecondary,
                bgColor = ChatTheme.colors.secondary,
            )
            ColorBox(
                label = "secondaryVariant",
                textColor = ChatTheme.colors.onSecondary,
                bgColor = ChatTheme.colors.secondaryVariant,
            )
        }
        Row(modifier = Modifier.weight(1f)) {
            ColorBox(
                label = "error",
                textColor = ChatTheme.colors.onError,
                bgColor = ChatTheme.colors.error,
            )
            ColorBox(
                label = "quote",
                textColor = ChatTheme.colors.quote,
                bgColor = ChatTheme.colors.background,
            )
            ColorBox(
                label = "spoiler",
                textColor = ChatTheme.colors.spoiler,
                bgColor = ChatTheme.colors.background,
                textModifier = Modifier.background(ChatTheme.colors.spoiler)
            )
        }
    }
}

@Composable
private fun RowScope.ColorBox(
    label: String,
    textColor: Color,
    bgColor: Color,
    textModifier: Modifier = Modifier,
) {
    Box(
        modifier = Modifier
            .background(bgColor)
            .weight(1f)
            .fillMaxHeight()
            .padding(ChatTheme.dimens.spacingLarge)
    ) {
        TextLine(
            text = label,
            style = ChatTheme.typography.body1,
            color = textColor,
            modifier = textModifier,
        )
    }
}