package nand.apps.chat.ui.toolbar

import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.ArrowBackIosNew
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun SimpleToolbar(
    text: String,
    modifier: Modifier = Modifier,
    navIcon: ImageVector = Icons.Outlined.ArrowBackIosNew,
    isNavButtonEnabled: Boolean = true,
    onNavClick: () -> Unit = {},
    endContent: @Composable () -> Unit = {},
) = Row(
    verticalAlignment = Alignment.CenterVertically,
    modifier = modifier.fillMaxWidth(),
) {
    IconButton(
        icon = navIcon,
        contentDescription = stringResource(Res.string.back),
        isEnabled = isNavButtonEnabled,
        onClick = onNavClick,
        size = ChatTheme.dimens.iconMedium,
    )
    TextLine(
        text = text,
        style = ChatTheme.typography.h6,
        modifier = Modifier.weight(1f),
    )
    endContent()
}