package nand.apps.chat.ui.tutorial

import androidx.compose.foundation.layout.*
import androidx.compose.material.RadioButton
import androidx.compose.material.Text
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import nand.apps.chat.ui.button.SimpleTextButton
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.provider.LocalNavController
import nand.apps.chat.ui.provider.isPortraitMode
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun TutorialAppSettingsScreen(
    viewModel: TutorialViewModel,
) {
    val navController = LocalNavController.current
    val setting = rememberSaveable { mutableStateOf(TutorialAppSetting.MAXIMUM) }
    TutorialStepColumn(
        title = stringResource(Res.string.tutorial_app_settings_title),
    ) {
        Column(
            modifier = Modifier.run { if (isPortraitMode()) fillMaxWidth() else width(480.dp) },
        ) {
            AppSettingRow(
                title = Res.string.tutorial_app_settings_maximum_label,
                description = Res.string.tutorial_app_settings_maximum_desc,
                setting = TutorialAppSetting.MAXIMUM,
                selected = setting,
            )
            AppSettingRow(
                title = Res.string.tutorial_app_settings_compatibility_label,
                description = Res.string.tutorial_app_settings_compatibility_desc,
                setting = TutorialAppSetting.COMPATIBILITY,
                selected = setting,
            )
            AppSettingRow(
                title = Res.string.tutorial_app_settings_security_label,
                description = Res.string.tutorial_app_settings_security_desc,
                setting = TutorialAppSetting.SECURITY,
                selected = setting,
            )
            AppSettingRow(
                title = Res.string.tutorial_app_settings_minimal_label,
                description = Res.string.tutorial_app_settings_minimal_desc,
                setting = TutorialAppSetting.MINIMAL,
                selected = setting,
            )
        }
        SimpleTextButton(
            text = stringResource(Res.string.tutorial_continue),
            onClick = {
                viewModel.updateAppSettings(setting.value)
                navController.navigate(TutorialNetworkSettingsRoute)
            },
        )
    }
}

@Composable
private fun AppSettingRow(
    title: StringResource,
    description: StringResource,
    setting: TutorialAppSetting,
    selected: MutableState<TutorialAppSetting>,
) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = Modifier
            .fillMaxWidth()
            .onClick { selected.value = setting }
            .padding(vertical = ChatTheme.dimens.spacingMedium),
    ) {
        RadioButton(
            selected = setting == selected.value,
            onClick = { selected.value = setting },
        )
        Column(
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingSmall),
            modifier = Modifier.padding(end = ChatTheme.dimens.spacingMedium),
        ) {
            TextLine(
                text = stringResource(title),
                style = ChatTheme.typography.body1,
                color = ChatTheme.colors.onSurface,
            )
            Text(
                text = stringResource(description),
                style = ChatTheme.typography.body2,
                color = ChatTheme.colors.onBackground,
            )
        }
    }
}