package nand.apps.chat.ui.tutorial

import androidx.compose.animation.*
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.padding
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import com.hoc081098.kmp.viewmodel.koin.compose.koinKmpViewModel
import nand.apps.chat.platform.ChatPlatform
import nand.apps.chat.platform.getPlatform
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.navigation.NavRoute
import nand.apps.chat.ui.navigation.TutorialRoute
import nand.apps.chat.ui.provider.LocalNavController
import nand.apps.chat.ui.provider.LocalNavRoute
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun TutorialNavGraph(
    modifier: Modifier = Modifier,
) {
    val viewModel: TutorialViewModel = koinKmpViewModel()
    Box(
        contentAlignment = Alignment.Center,
        modifier = modifier.background(Color.Black),
    ) {
        TutorialPage(TutorialRoute, enter = EnterTransition.None) {
            TutorialIntroScreen()
        }
        TutorialPage(TutorialAppUsageRoute) {
            TutorialAppUsageScreen(viewModel)
        }
        TutorialPage(TutorialToxUsageRoute) {
            TutorialToxUsageScreen()
        }
        TutorialPage(TutorialExistingProfileRoute) {
            TutorialExistingProfileScreen()
        }
        TutorialPage(TutorialImportProfileRoute) {
            TutorialImportProfileScreen(viewModel)
        }
        TutorialPage(TutorialUsernameRoute) {
            TutorialUsernameScreen(viewModel)
        }
        TutorialPage(TutorialPasswordRoute) {
            TutorialPasswordScreen(viewModel)
        }
        TutorialPage(TutorialAppSettingsRoute) {
            TutorialAppSettingsScreen(viewModel)
        }
        TutorialPage(TutorialNetworkSettingsRoute) {
            TutorialNetworkSettingsScreen(viewModel)
        }
        // Back button (desktop and iOS only)
        val navRoute = LocalNavRoute.current
        val isBackButtonVisible = getPlatform() != ChatPlatform.ANDROID &&
                navRoute != TutorialRoute &&
                navRoute != TutorialAppSettingsRoute
        AnimatedVisibility(
            visible = isBackButtonVisible,
            enter = fadeIn() + slideInHorizontally(),
            exit = fadeOut() + slideOutHorizontally(),
            modifier = Modifier.align(Alignment.BottomStart),
        ) {
            val navController = LocalNavController.current
            TextLine(
                text = stringResource(Res.string.back),
                style = ChatTheme.typography.body1,
                color = ChatTheme.colors.onBackground,
                modifier = Modifier
                    .onClick { navController.navigateBack() }
                    .padding(ChatTheme.dimens.spacingXLarge),
            )
        }
    }
}

@Composable
private fun TutorialPage(
    route: NavRoute,
    enter: EnterTransition = fadeIn() + slideInHorizontally { it / 2 },
    exit: ExitTransition = fadeOut() + slideOutHorizontally(),
    content: @Composable AnimatedVisibilityScope.() -> Unit,
) = AnimatedVisibility(
    visible = LocalNavRoute.current == route,
    enter = enter,
    exit = exit,
    content = content,
)