package nand.apps.chat.ui.tutorial

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.Text
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.focus.focusRequester
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.style.TextAlign
import nand.apps.chat.io.isValidFileName
import nand.apps.chat.platform.getPlatform
import nand.apps.chat.ui.button.SimpleTextButton
import nand.apps.chat.ui.provider.LocalNavController
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun TutorialUsernameScreen(viewModel: TutorialViewModel) {
    val navController = LocalNavController.current
    var username by rememberTextFieldValue(viewModel.username)
    val usernameFocus = remember { FocusRequester() }
    TutorialStepColumn(
        title = stringResource(Res.string.tutorial_enter_username),
    ) {
        val isUsernameValid = username.text.isValidFileName()
        LaunchedEffect(Unit) { if (getPlatform().isDesktop) usernameFocus.requestFocus() }
        SimpleTextField(
            value = username,
            onValueChange = { username = it },
            labelText = stringResource(Res.string.username),
            isError = username.text.isNotEmpty() && !isUsernameValid,
            onSubmit = {
                if (isUsernameValid) {
                    viewModel.updateState(
                        username = username.text,
                        isNewAccount = true,
                    )
                    navController.navigate(TutorialPasswordRoute)
                }
            },
            keyboardOptions = KeyboardOptions(
                autoCorrect = false,
                imeAction = ImeAction.Done,
            ),
            modifier = Modifier.focusRequester(usernameFocus),
        )
        Text(
            text = stringResource(Res.string.tutorial_enter_username_desc),
            color = ChatTheme.colors.onBackground,
            style = ChatTheme.typography.body2,
            textAlign = TextAlign.Center,
            modifier = Modifier.padding(
                vertical = ChatTheme.dimens.spacingMedium,
                horizontal = ChatTheme.dimens.spacingXLarge,
            ),
        )
        SimpleTextButton(
            text = stringResource(Res.string.tutorial_continue),
            isEnabled = isUsernameValid,
            onClick = {
                viewModel.updateState(
                    username = username.text,
                    isNewAccount = true,
                )
                navController.navigate(TutorialPasswordRoute)
            },
        )
    }
}