package nand.apps.chat.ui.util

import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.graphics.*
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.unit.Dp

@Composable
fun rememberGridBrush(
    size: Dp,
    backgroundColor: Color,
    foregroundColor: Color,
): ShaderBrush {
    val sizePx = LocalDensity.run { size.toPx() }
    return remember(sizePx, backgroundColor, foregroundColor) {
        val half = sizePx / 2f
        val bitmap = ImageBitmap(sizePx.toInt(), sizePx.toInt())
        val paint = Paint()
        val canvas = Canvas(bitmap)
        paint.color = backgroundColor
        canvas.drawRect(0f, 0f, sizePx, sizePx, paint)
        paint.color = foregroundColor
        canvas.drawRect(0f, 0f, half, half, paint)
        canvas.drawRect(half, half, sizePx, sizePx, paint)
        ShaderBrush(ImageShader(bitmap, TileMode.Repeated, TileMode.Repeated))
    }
}