package nand.apps.chat.ui.viewmodel

import androidx.compose.runtime.Composable
import androidx.compose.runtime.NonRestartableComposable
import com.hoc081098.kmp.viewmodel.*
import com.hoc081098.kmp.viewmodel.compose.LocalSavedStateHandleFactory
import com.hoc081098.kmp.viewmodel.compose.defaultPlatformCreationExtras
import com.hoc081098.kmp.viewmodel.compose.internal.kClassOf
import com.hoc081098.kmp.viewmodel.koin.compose.koinKmpViewModel
import org.koin.compose.currentKoinScope
import org.koin.core.parameter.ParametersDefinition
import org.koin.core.qualifier.Qualifier
import org.koin.core.scope.Scope

@Suppress("LongParameterList")
@OptIn(InternalKmpViewModelApi::class)
@MainThread
@Composable
@NonRestartableComposable
inline fun <reified VM : ViewModel> composableViewModel(
    viewModelStoreOwner: ViewModelStoreOwner = getComposableViewModelStoreOwner(),
    key: String? = null,
    extras: CreationExtras = defaultPlatformCreationExtras(),
    savedStateHandleFactory: SavedStateHandleFactory? = LocalSavedStateHandleFactory.current,
    qualifier: Qualifier? = null,
    scope: Scope = currentKoinScope(),
    noinline parameters: ParametersDefinition? = null,
): VM =
    koinKmpViewModel(
        viewModelClass = kClassOf<VM>(),
        viewModelStoreOwner = viewModelStoreOwner,
        key = key,
        extras = extras,
        savedStateHandleFactory = savedStateHandleFactory,
        qualifier = qualifier,
        scope = scope,
        parameters = parameters,
    )

@Composable
expect fun getComposableViewModelStoreOwner(): ViewModelStoreOwner