package nand.apps.chat.util

import kotlinx.serialization.json.*

fun JsonElement?.jsonObjectSafe() = this as? JsonObject ?: JsonObject(emptyMap())

fun JsonElement?.jsonArraySafe() = this as? JsonArray ?: JsonArray(emptyList())

fun JsonElement?.getInt() = this?.jsonPrimitive?.intOrNull ?: 0

fun JsonElement?.getLong() = this?.jsonPrimitive?.longOrNull ?: 0L

fun JsonElement?.getBoolean() = this?.jsonPrimitive?.booleanOrNull == true

fun JsonElement?.getString() = this?.jsonPrimitive?.contentOrNull

fun JsonElement.find(vararg keys: String): JsonElement? {
    var root: JsonElement? = this
    for (key in keys) {
        root = when (root) {
            is JsonObject -> root[key]
            is JsonArray -> root.find { it is JsonObject && it.containsKey(key) }?.jsonObject?.get(key)
            else -> return null
        }
    }
    return root
}