package nand.apps.chat.util

import kotlin.math.PI

fun Double.toRadians() = (this * PI) / 180.0

fun Float.clamp(range: ClosedFloatingPointRange<Float>) =
    this.coerceAtLeast(range.start).coerceAtMost(range.endInclusive)

fun ByteArray.toIntLittleEndian() =
    (this[0] ushl 24) or (this[1] ushl 16) or (this[2] ushl 8) or (this[3] ushl 0)

infix fun Byte.ushl(shift: Int) =
    (toInt() + if (this < 0) 256 else 0) shl shift

fun Int.hasBits(bits: Int) = (this and bits) == bits

fun Int.clamp(min: Int, max: Int) = when {
    this < min -> min
    this > max -> max
    else -> this
}