package nand.apps.opus

import nand.apps.chat.audio.DEFAULT_AUDIO_CHANNELS
import nand.apps.chat.audio.DEFAULT_AUDIO_SAMPLE_RATE
import kotlin.math.roundToLong

class OpusFileReader private constructor() : OpusFileStream(
    sampleRate = DEFAULT_AUDIO_SAMPLE_RATE,
    channels = DEFAULT_AUDIO_CHANNELS,
) {
    
    val sampleCount by lazy {
        if (isOpen) OpusNative.getSampleCount(ptr) else -1
    }
    
    val timestamp get() = if (isOpen) (OpusNative.getSamplePosition(ptr).toDouble() / sampleRate) else 0.0
    
    val duration by lazy {
        if (isOpen) sampleCount.toDouble() / sampleRate else 0.0
    }

    constructor(path: String) : this() {
        ptr = OpusNative.openFileReader(path)
    }
    
    constructor(data: ByteArray) : this() {
        ptr = OpusNative.openMemoryReader(data)
    }
    
    fun seekTo(timestamp: Double): Boolean {
        return if (isOpen) OpusNative.seekTo(ptr, (timestamp * sampleRate).roundToLong()) else false
    }
    
    fun read(buffer: ShortArray, offset: Int, length: Int): Int {
        return when {
            isOpen -> OpusNative.readSamples(
                ptr = ptr,
                buffer = buffer,
                offset = offset,
                length = length,
            )
            else -> 0
        }
    }
    
    fun read(buffer: ShortArray) = read(
        buffer = buffer,
        offset = 0,
        length = buffer.size,
    )

    override fun closeImpl() {
        OpusNative.closeReader(ptr)
    }
}