package nand.apps.opus

class OpusFileWriter(
    val path: String,
    sampleRate: Int,
    channels: Int,
    val bitRate: Int,
) : OpusFileStream(
    sampleRate = sampleRate,
    channels = channels,
) {
    
    init {
        ptr = OpusNative.openFileWriter(
            path = path,
            sampleRate = sampleRate,
            channels = channels,
            bitRate = bitRate,
        )
    }

    fun write(buffer: ShortArray, offset: Int, length: Int): Int {
        return when {
            isOpen -> OpusNative.writeSamples(
                ptr = ptr,
                buffer = buffer,
                offset = offset,
                samplesPerChannel = length / channels,
            )
            else -> 0
        }
    }

    fun write(buffer: ShortArray) = write(
        buffer = buffer,
        offset = 0,
        length = buffer.size,
    )

    override fun closeImpl() {
        OpusNative.closeWriter(ptr)
    }
}