package nand.apps.tox

import nand.apps.chat.log.getLogger

private const val LOGGER_LEVEL_TRACE = 0
private const val LOGGER_LEVEL_DEBUG = 1
private const val LOGGER_LEVEL_INFO = 2
private const val LOGGER_LEVEL_WARNING = 3
private const val LOGGER_LEVEL_ERROR = 4

class DefaultToxLogger : ToxLogger {
    
    private val logger = getLogger("ToxNative")
    
    override fun onLogMessage(level: Int, file: ByteArray, line: Int, func: ByteArray, message: ByteArray) {
        val fileName = getFileName(file)
        val formatted = "$fileName[$line] ${func.decodeToString()}: ${message.decodeToString()}"
        when (level) {
            LOGGER_LEVEL_WARNING -> logger.warn(formatted)
            LOGGER_LEVEL_ERROR -> logger.error(formatted)
            LOGGER_LEVEL_TRACE -> logger.verbose(formatted)
            else -> logger.debug(formatted)
        }
    }
    
    private fun getFileName(file: ByteArray): String {
        val path = file.decodeToString().replace('\\', '/')
        val fileName = path.substringAfterLast('/')
        return when {
            path.contains("native/jni/tox/") -> "toxjni/$fileName"
            else -> fileName
        }
    }
}