package nand.apps.tox

import nand.apps.tox.callback.ToxCallback

object ToxNative {
    
    // Primary Tox functionality
    external fun initTox(saveData: ByteArray?, options: ToxInitOptions?, logger: ToxLogger): ToxInitResult
    external fun destroyTox(toxPtr: Long)
    external fun bootstrap(toxPtr: Long, host: String, udpPort: Int, tcpPort: Int, publicKey: ByteArray): Boolean
    external fun registerCallbacks(toxPtr: Long, avPtr: Long, cb: ToxCallback)
    external fun iterate(toxPtr: Long): Int
    external fun getSaveData(toxPtr: Long): ByteArray
    external fun getConstants(): ToxConstants
    
    // Self state
    external fun getSelfUserData(toxPtr: Long): ToxUserData
    external fun setSelfUserData(toxPtr: Long, name: ByteArray, statusMessage: ByteArray): Boolean
    external fun setSelfStatus(toxPtr: Long, status: Int)
    external fun getSelfAddress(toxPtr: Long): ByteArray
    external fun setSelfNoSpam(toxPtr: Long, noSpam: Int)
    
    // Friends
    external fun getFriendList(toxPtr: Long): Array<ToxUserData>
    external fun sendFriendRequest(toxPtr: Long, address: ByteArray, message: ByteArray): Int
    external fun acceptFriendRequest(toxPtr: Long, publicKey: ByteArray): Int
    external fun sendFriendMessage(toxPtr: Long, friendNum: Int, message: ByteArray, messageType: Int): Int
    external fun sendFriendPacket(toxPtr: Long, friendNum: Int, packet: ByteArray): Boolean
    external fun removeFriend(toxPtr: Long, friendNum: Int): Boolean
    
    // Simple groups (conferences)
    external fun getConferenceList(toxPtr: Long): Array<ToxGroupData?>
    external fun getConferencePeerList(toxPtr: Long, groupNum: Int): Array<ToxUserData?>
    external fun createConference(toxPtr: Long, title: ByteArray, type: Int): ToxGroupData?
    external fun setConferenceTitle(toxPtr: Long, groupNum: Int, title: ByteArray): Boolean
    external fun setConferenceAudioEnabled(toxPtr: Long, groupNum: Int, isEnabled: Boolean): Boolean
    external fun isConferenceAudioEnabled(toxPtr: Long, groupNum: Int): Boolean
    external fun sendConferenceMessage(toxPtr: Long, groupNum: Int, message: ByteArray, messageType: Int)
    external fun sendConferenceAudioFrame(toxPtr: Long, groupNum: Int, samples: ShortArray, channels: Int, sampleRate: Int): Boolean
    external fun sendConferenceInvite(toxPtr: Long, friendNum: Int, groupNum: Int): Boolean
    external fun acceptConferenceInvite(toxPtr: Long, friendNum: Int, groupType: Int, cookie: ByteArray): ToxGroupData?
    external fun removeConference(toxPtr: Long, groupNum: Int): Boolean
    
    // Advanced groups (DHT-based)
    external fun getGroupList(toxPtr: Long): Array<ToxGroupData?>
    external fun createGroup(toxPtr: Long, privacyState: Int, title: ByteArray, selfNickname: ByteArray): ToxGroupData?
    external fun joinGroup(toxPtr: Long, uid: ByteArray, selfNickname: ByteArray?, password: ByteArray?): Int
    external fun getGroupSelfPeerData(toxPtr: Long, groupNum: Int): ToxUserData?
    external fun getGroupPeerData(toxPtr: Long, groupNum: Int, peerIds: IntArray): Array<ToxUserData?>
    external fun setGroupTopic(toxPtr: Long, groupNum: Int, topic: ByteArray): Boolean
    external fun setGroupPassword(toxPtr: Long, groupNum: Int, password: ByteArray?): Boolean
    external fun setGroupNickname(toxPtr: Long, groupNum: Int, name: ByteArray): Boolean
    external fun setGroupRole(toxPtr: Long, groupNum: Int, peerId: Int, role: Int): Boolean
    external fun kickGroupPeer(toxPtr: Long, groupNum: Int, peerId: Int): Boolean
    external fun setGroupPeerIgnore(toxPtr: Long, groupNum: Int, peerId: Int, ignore: Boolean): Boolean
    external fun sendGroupInvite(toxPtr: Long, friendNum: Int, groupNum: Int): Boolean
    external fun acceptGroupInvite(toxPtr: Long, friendNum: Int, inviteData: ByteArray, peerName: ByteArray, password: ByteArray?): ToxGroupData?
    external fun sendGroupMessage(toxPtr: Long, groupNum: Int, message: ByteArray, messageType: Int)
    external fun sendGroupPacket(toxPtr: Long, groupNum: Int, packet: ByteArray): Boolean
    external fun sendPeerPacket(toxPtr: Long, groupNum: Int, peerId: Int, packet: ByteArray): Boolean
    external fun reconnectGroup(toxPtr: Long, groupNum: Int): Boolean
    external fun disconnectGroup(toxPtr: Long, groupNum: Int): Boolean
    external fun isGroupConnected(toxPtr: Long, groupNum: Int): Boolean
    external fun removeGroup(toxPtr: Long, groupNum: Int, partingMessage: ByteArray?): Boolean

    // File transfers
    external fun sendFile(toxPtr: Long, friendNum: Int, kind: Int, fileSize: Long, fileName: ByteArray, fileId: ByteArray?): Int
    external fun sendFileChunk(toxPtr: Long, friendNum: Int, fileNum: Int, position: Long, data: ByteArray): Boolean
    external fun sendFileControl(toxPtr: Long, friendNum: Int, fileNum: Int, control: Int): Int
    
    // Audio/video calls
    external fun initAv(toxPtr: Long): Long
    external fun destroyAv(avPtr: Long)
    external fun iterateAudio(avPtr: Long): Int
    external fun startCall(avPtr: Long, friendNum: Int, audioBitRate: Int, videoBitRate: Int): Boolean
    external fun answerCall(avPtr: Long, friendNum: Int, audioBitRate: Int, videoBitRate: Int): Boolean
    external fun sendCallControl(avPtr: Long, friendNum: Int, control: Int): Boolean
    external fun sendAudioFrame(avPtr: Long, friendNum: Int, samples: ShortArray, channels: Int, sampleRate: Int): Boolean
    external fun sendAudioBitRate(avPtr: Long, friendNum: Int, bitRate: Int): Boolean
    
    // Hashing and encryption
    external fun hash(data: ByteArray): ByteArray?
    external fun createPassKey(passPhrase: ByteArray, cipher: ByteArray?): Long
    external fun freePassKey(keyPtr: Long)
    external fun encryptWithPassKey(keyPtr: Long, data: ByteArray): ByteArray?
    external fun decryptWithPassKey(keyPtr: Long, cipher: ByteArray): ByteArray?
    external fun isEncrypted(data: ByteArray): Boolean
}