package nand.apps.chat.di

import nand.apps.chat.engine.ChatEngine
import nand.apps.chat.model.engine.ChatEngineConstraints
import nand.apps.chat.engine.mock.getMockKoinModule
import nand.apps.chat.io.*
import nand.apps.chat.net.TestUriLauncher
import nand.apps.chat.net.UriLauncher
import nand.apps.chat.net.packet.ChatPacketConverter
import nand.apps.chat.net.packet.ChatPacketRepo
import nand.apps.chat.repo.NotificationRepo
import nand.apps.chat.repo.*
import nand.apps.chat.text.emoji.EmojiRepo
import nand.apps.chat.event.ChatEventHandler
import nand.apps.chat.platform.AppStateProvider
import nand.apps.chat.platform.TestAppStateProvider
import okio.FileSystem
import okio.fakefilesystem.FakeFileSystem
import org.koin.core.Koin
import org.koin.core.KoinApplication
import org.koin.core.context.startKoin
import org.koin.core.module.dsl.singleOf
import org.koin.dsl.module

fun initKoinTest(): KoinApplication {
    val registeredModules = module {
        // Global
        single<Koin> { getKoin() }
        single<FileSystem> { FakeFileSystem() }
        single<AppStateProvider> { TestAppStateProvider() }
        single<ChatFilePaths> { TestFilePaths() }
        single<UriLauncher> { TestUriLauncher() }
        single<ChatBackgroundWorker> { TestBackgroundWorker() }
        singleOf(::ChatFileSystem)
        singleOf(::ChatPacketConverter)
        
        // Core chat engine
        single<ChatEngineConstraints> {
            val engine: ChatEngine by getKoin().inject()
            engine.constraints
        }
        
        // Repos
        singleOf(::SettingsRepo)
        singleOf(::FileTransferRepo)
        singleOf(::ChatAvatarRepo)
        singleOf(::ChatMessageRepo)
        singleOf(::ContactRepo)
        singleOf(::SelfProfileRepo)
        singleOf(::ChatPacketRepo)
        singleOf(::NotificationRepo)
        singleOf(::EmojiRepo)
        
        // Other
        singleOf(::ChatEventHandler)
    }
    return startKoin {
        modules(registeredModules)
        modules(getMockKoinModule())
    }
}