package nand.apps.chat.io

import okio.Path.Companion.toPath
import okio.fakefilesystem.FakeFileSystem
import kotlin.test.BeforeTest
import kotlin.test.Test
import kotlin.test.assertEquals

class DuplicateFileTests {
    
    private val fileSystem = FakeFileSystem()
    private val directory = ChatDirectory(".".toPath(), fileSystem)
    
    @BeforeTest
    fun setup() {
        fileSystem.emulateUnix()
        fileSystem.workingDirectory = "/some_directory".toPath()
        fileSystem.createDirectory(fileSystem.workingDirectory)
    }
    
    @Test
    fun `Ensure no duplicate files`() {
        createFile("1st.txt")
        createFile("2nd.txt")
        assertEquals(
            expected = "3rd.txt",
            actual = directory.resolveDuplicateFileName("3rd.txt"),
        )
    }

    @Test
    fun `Duplicate file name has appended number`() {
        createFile("test.txt")
        assertEquals(
            expected = "test_1.txt",
            actual = directory.resolveDuplicateFileName("test.txt"),
        )
    }

    @Test
    fun `Multiple duplicate file names have incrementing numbers`() {
        createFile("test.txt")
        for (i in 1 until 10) {
            val actual = directory.resolveDuplicateFileName("test.txt")
            assertEquals(
                expected = "test_$i.txt",
                actual = actual,
            )
            createFile(actual)
        }
        assertEquals(
            expected = "test_10.txt",
            actual = directory.resolveDuplicateFileName("test_5.txt"),
        )
        assertEquals(
            expected = "test_20.txt",
            actual = directory.resolveDuplicateFileName("test_20.txt"),
        )
    }
    
    private fun createFile(name: String) = fileSystem.write(name.toPath()) {}
}