package nand.apps.chat.shortcut

import nand.apps.chat.model.settings.shortcut.ShortcutKeyCombo
import nand.apps.chat.model.settings.shortcut.toShortcutKeyCombo
import nand.apps.chat.platform.ChatPlatform
import kotlin.test.Test
import kotlin.test.assertEquals
import kotlin.test.assertTrue

class ShortcutKeyTests {
    
    @Test
    fun `Shortcut serialization and deserialization`() {
        assertEquals(
            expected = ShortcutKeyCombo("Escape"),
            actual = "Escape".toShortcutKeyCombo(),
        )
        assertEquals(
            expected = ShortcutKeyCombo("Enter", ctrl = true),
            actual = "Ctrl+Enter".toShortcutKeyCombo(),
        )
        assertEquals(
            expected = ShortcutKeyCombo("C", ctrl = true, shift = true, alt = true),
            actual = "Ctrl+Alt+Shift+C".toShortcutKeyCombo(),
        )
    }
    
    @Test
    fun `Shortcut platform localization`() {
        assertEquals(
            expected = "⌃⌥⌦",
            actual = ShortcutKeyCombo("Delete", ctrl = true, alt = true).toPlatformString(ChatPlatform.MAC),
        )
        assertEquals(
            expected = "⇧⌘A",
            actual = ShortcutKeyCombo("A", shift = true, meta = true).toPlatformString(ChatPlatform.MAC),
        )
        assertEquals(
            expected = "Ctrl+Shift+D",
            actual = ShortcutKeyCombo("D", ctrl = true, shift = true).toPlatformString(ChatPlatform.LINUX),
        )
    }
    
    @Test
    fun `Shortcut is valid or invalid`() {
        val validKeys = listOf(
            ShortcutKeyCombo("Escape"),
            ShortcutKeyCombo("C", ctrl = true),
            ShortcutKeyCombo("Back", shift = true),
        )
        val invalidKeys = listOf(
            ShortcutKeyCombo("C"),
            ShortcutKeyCombo("Back"),
        )
        validKeys.forEach { assertTrue(it.isValid()) }
        invalidKeys.forEach { assertTrue(it.isValid().not()) }
    }
}