package nand.apps.chat.text.emoji

import kotlin.test.Test
import kotlin.test.assertEquals

class TextEmojiTests {
    
    @Test
    fun `Split string of regular emojis`() {
        assertSplitEmojis("\uD83D\uDE00", "\uD83E\uDD75", "\uD83D\uDC96")
    }

    @Test
    fun `Split string of skin tone emojis`() {
        assertSplitEmojis("\uD83E\uDD18\uD83C\uDFFB", "\uD83D\uDC49\uD83C\uDFFD", "\uD83E\uDEF5")
    }
    
    @Test
    fun `Split string of regional flag emojis`() {
        assertSplitEmojis("\uD83C\uDDEE\uD83C\uDDF3", "\uD83C\uDDEC\uD83C\uDDF3", "\uD83C\uDDFF\uD83C\uDDE6")
    }
    
    @Test
    fun `Split string of all types of emojis`() {
        assertSplitEmojis(
            "\uD83D\uDC49\uD83C\uDFFD",
            "\uD83C\uDDF5\uD83C\uDDF7",
            "\uD83D\uDC69\uD83C\uDFFF\u200D❤\uFE0F\u200D\uD83D\uDC69\uD83C\uDFFE",
            "\uD83C\uDFF4\uDB40\uDC67\uDB40\uDC62\uDB40\uDC65\uDB40\uDC6E\uDB40\uDC67\uDB40\uDC7F",
            "\uD83C\uDFF3\uFE0F\u200D\uD83C\uDF08",
            "\uD83C\uDFF3\uFE0F\u200D⚧\uFE0F",
        )
    }

    @Test
    fun `Unicode font is not affected`() {
        assertSplitEmojis("\uD835\uDD04\uD835\uDD05ℭ\uD835\uDD07\uD835\uDD08\uD835\uDD09\uD835\uDD0A")
    }
    
    private fun assertSplitEmojis(vararg segments: String) {
        assertEquals(
            expected = segments.toList(),
            actual = segments.joinToString(separator = "").splitByEmoji(),
        )
    }
}