package nand.apps.chat.text.markdown

import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.font.FontWeight
import nand.apps.chat.text.assertEquals
import nand.apps.chat.text.buildMarkdownText
import kotlin.test.Test

class BoldTextTests {
    
    @Test
    fun `Entirely bold string`() = assertEquals(
        expected = buildAnnotatedString {
            append("Bold text")
            addStyle(SpanStyle(fontWeight = FontWeight.Bold), 0, 9)
        },
        actual = buildMarkdownText("**Bold text**"),
    )
    
    @Test
    fun `Partially bold string`() = assertEquals(
        expected = buildAnnotatedString {
            append("This is some bold text")
            addStyle(SpanStyle(fontWeight = FontWeight.Bold), 13, 17)
        },
        actual = buildMarkdownText("This is some **bold** text"),
    )

    @Test
    fun `Incomplete bold tags`() {
        assertEquals(
            expected = AnnotatedString("**Incomplete tag content"),
            actual = buildMarkdownText("**Incomplete tag content"),
        )
        assertEquals(
            expected = AnnotatedString("Incomplete tag content**"),
            actual = buildMarkdownText("Incomplete tag content**"),
        )
    }
    
    @Test
    fun `Style not applied when nothing to style`() = assertEquals(
        expected = AnnotatedString("*****"),
        actual = buildMarkdownText("*****"),
    )
}