package nand.apps.chat.text.markdown

import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.font.FontStyle
import nand.apps.chat.text.assertEquals
import nand.apps.chat.text.buildMarkdownText
import kotlin.test.Test

class ItalicTextTests {
    
    @Test
    fun `Entirely italic string using asterisks`() = assertEquals(
        expected = buildAnnotatedString {
            append("Italic text")
            addStyle(SpanStyle(fontStyle = FontStyle.Italic), 0, 11)
        },
        actual = buildMarkdownText("*Italic text*"),
    )
    
    @Test
    fun `Entirely italic string using underscores`() = assertEquals(
        expected = buildAnnotatedString {
            append("Italic text")
            addStyle(SpanStyle(fontStyle = FontStyle.Italic), 0, 11)
        },
        actual = buildMarkdownText("_Italic text_"),
    )
    
    @Test
    fun `Partially italic string`() = assertEquals(
        expected = buildAnnotatedString {
            append("This is some italic text")
            addStyle(SpanStyle(fontStyle = FontStyle.Italic), 13, 19)
        },
        actual = buildMarkdownText("This is some *italic* text"),
    )
    
    @Test
    fun `Partially italic text using both tags`() = assertEquals(
        expected = buildAnnotatedString {
            append("Both tags set")
            addStyle(SpanStyle(fontStyle = FontStyle.Italic), 5, 9)
            addStyle(SpanStyle(fontStyle = FontStyle.Italic), 5, 9)
        },
        actual = buildMarkdownText("Both _*tags*_ set"),
    )
    
    @Test
    fun `Partially italic text using both tags interleaved`() = assertEquals(
        expected = buildAnnotatedString {
            append("Both tags set")
            addStyle(SpanStyle(fontStyle = FontStyle.Italic), 5, 9)
            addStyle(SpanStyle(fontStyle = FontStyle.Italic), 5, 9)
        },
        actual = buildMarkdownText("Both _*tags_* set"),
    )
}