package nand.apps.chat.text.message

import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.buildAnnotatedString
import nand.apps.chat.text.assertEquals
import nand.apps.chat.text.parseStyleTags
import kotlin.test.Test

class ColorTagTests {
    
    @Test
    fun `Red and blue text`() = assertEquals(
        expected = buildAnnotatedString {
            append("This text is red and blue")
            addStyle(SpanStyle(color = Color.Red), 13, 16)
            addStyle(SpanStyle(color = Color.Blue), 21, 25)
        },
        actual = parseStyleTags("This text is \$red\$red\$end\$ and \$blu\$blue\$end\$"),
    )

    @Test
    fun `Yellow text with open tag`() = assertEquals(
        expected = buildAnnotatedString {
            append("This text is yellow")
            addStyle(SpanStyle(color = Color.Yellow), 13, 19)
        },
        actual = parseStyleTags("This text is \$yel\$yellow"),
    )

    @Test
    fun `Overlapping color tags`() = assertEquals(
        expected = buildAnnotatedString {
            append("This text is 2 different colors")
            addStyle(SpanStyle(color = Color.Cyan), 13, 31)
            addStyle(SpanStyle(color = Color.Magenta), 15, 24)
        },
        actual = parseStyleTags("This text is \$cya\$2 \$mag\$different\$end\$ colors"),
    )

    @Test
    fun `Hidden text`() = assertEquals(
        expected = buildAnnotatedString {
            append("This text is hidden")
            addStyle(SpanStyle(color = Color.Transparent), 0, 19)
        },
        actual = parseStyleTags("\$hide\$This text is hidden"),
    )

    @Test
    fun `Invalid tag isn't replaced`() = assertEquals(
        expected = AnnotatedString("This \$tag\$ is invalid"),
        actual = parseStyleTags("This \$tag\$ is invalid"),
    )
}