package nand.apps.chat.text.message

import androidx.compose.ui.text.buildAnnotatedString
import nand.apps.chat.text.*
import kotlin.test.Test

class FontTagTests {

    @Test
    fun `Multiple fonts overlapping`() = assertEquals(
        expected = buildAnnotatedString {
            append("This \uD835\uDD23\uD835\uDD2C\uD835\uDD2B\uD835\uDD31 \uD835\uDD26\uD835\uDD30 \uD835\uDD32\uD835\uDD30\uD835\uDD22\uD835\uDD21 \uD835\uDCB8\uD835\uDCCA\uD835\uDCC7\uD835\uDCC8\uD835\uDCBE\uD835\uDCCB\uD835\uDC52\uD835\uDCC1\uD835\uDCCE \uD835\uDCB6\uD835\uDCC3\uD835\uDCB9 \uD835\uDD2B\uD835\uDD2C\uD835\uDD31 \uD835\uDD20\uD835\uDD1E\uD835\uDD30\uD835\uDD32\uD835\uDD1E\uD835\uDD29\uD835\uDD29\uD835\uDD36, I swear.")
            addFontAnnotation(UnicodeFont.FRAKTUR, 5, 79)
            addFontAnnotation(UnicodeFont.CURSIVE, 28, 53)
        },
        actual = buildFontTagText("This \$fra\$font is used \$cur\$cursively and\$end\$ not casually, \$end\$I swear."),
    )
    
    @Test
    fun `Fraktur font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD835\uDD16\uD835\uDD2D\uD835\uDD25\uD835\uDD26\uD835\uDD2B\uD835\uDD35 \uD835\uDD2C\uD835\uDD23 \uD835\uDD1F\uD835\uDD29\uD835\uDD1E\uD835\uDD20\uD835\uDD28 \uD835\uDD2E\uD835\uDD32\uD835\uDD1E\uD835\uDD2F\uD835\uDD31\uD835\uDD37, \uD835\uDD27\uD835\uDD32\uD835\uDD21\uD835\uDD24\uD835\uDD22 \uD835\uDD2A\uD835\uDD36 \uD835\uDD33\uD835\uDD2C\uD835\uDD34.")
            addFontAnnotation(UnicodeFont.FRAKTUR, 0, 66)
        },
        actual = buildFontTagText("\$fra\$Sphinx of black quartz, judge my vow."),
    )

    @Test
    fun `Fraktur Bold font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD835\uDD73\uD835\uDD94\uD835\uDD9C \uD835\uDD9B\uD835\uDD8A\uD835\uDD9D\uD835\uDD8E\uD835\uDD93\uD835\uDD8C\uD835\uDD91\uD835\uDD9E \uD835\uDD96\uD835\uDD9A\uD835\uDD8E\uD835\uDD88\uD835\uDD90 \uD835\uDD89\uD835\uDD86\uD835\uDD8B\uD835\uDD99 \uD835\uDD9F\uD835\uDD8A\uD835\uDD87\uD835\uDD97\uD835\uDD86\uD835\uDD98 \uD835\uDD8F\uD835\uDD9A\uD835\uDD92\uD835\uDD95!")
            addFontAnnotation(UnicodeFont.FRAKTUR_BOLD, 0, 66)
        },
        actual = buildFontTagText("\$frab\$How vexingly quick daft zebras jump!"),
    )

    @Test
    fun `Cursive font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD835\uDCB2\uD835\uDCB6\uD835\uDCC1\uD835\uDCC9\uD835\uDCCF, \uD835\uDCB7\uD835\uDCB6\uD835\uDCB9 \uD835\uDCC3\uD835\uDCCE\uD835\uDCC2\uD835\uDCC5\uD835\uDCBD, \uD835\uDCBB\uD835\uDC5C\uD835\uDCC7 \uD835\uDCC6\uD835\uDCCA\uD835\uDCBE\uD835\uDCB8\uD835\uDCC0 \uD835\uDCBF\uD835\uDCBE\uD835\uDC54\uD835\uDCC8 \uD835\uDCCB\uD835\uDC52\uD835\uDCCD.")
            addFontAnnotation(UnicodeFont.CURSIVE, 0, 65)
        },
        actual = buildFontTagText("\$cur\$Waltz, bad nymph, for quick jigs vex."),
    )

    @Test
    fun `Cursive alt font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD835\uDCE6\uD835\uDCEA\uD835\uDCF5\uD835\uDCFD\uD835\uDD03, \uD835\uDCEB\uD835\uDCEA\uD835\uDCED \uD835\uDCF7\uD835\uDD02\uD835\uDCF6\uD835\uDCF9\uD835\uDCF1, \uD835\uDCEF\uD835\uDCF8\uD835\uDCFB \uD835\uDCFA\uD835\uDCFE\uD835\uDCF2\uD835\uDCEC\uD835\uDCF4 \uD835\uDCF3\uD835\uDCF2\uD835\uDCF0\uD835\uDCFC \uD835\uDCFF\uD835\uDCEE\uD835\uDD01.")
            addFontAnnotation(UnicodeFont.CURSIVE_ALT, 0, 65)
        },
        actual = buildFontTagText("\$cur2\$Waltz, bad nymph, for quick jigs vex."),
    )

    @Test
    fun `Double struck font`() = assertEquals(
        expected = buildAnnotatedString {
            append("ℙ\uD835\uDD52\uD835\uDD54\uD835\uDD5C \uD835\uDD5E\uD835\uDD6A \uD835\uDD53\uD835\uDD60\uD835\uDD69 \uD835\uDD68\uD835\uDD5A\uD835\uDD65\uD835\uDD59 \uD835\uDD57\uD835\uDD5A\uD835\uDD67\uD835\uDD56 \uD835\uDD55\uD835\uDD60\uD835\uDD6B\uD835\uDD56\uD835\uDD5F \uD835\uDD5D\uD835\uDD5A\uD835\uDD62\uD835\uDD66\uD835\uDD60\uD835\uDD63 \uD835\uDD5B\uD835\uDD66\uD835\uDD58\uD835\uDD64.")
            addFontAnnotation(UnicodeFont.DOUBLE_STRUCK, 0, 71)
        },
        actual = buildFontTagText("\$dbs\$Pack my box with five dozen liquor jugs."),
    )

    @Test
    fun `Circle font`() = assertEquals(
        expected = buildAnnotatedString {
            append("Ⓜⓡ. Ⓙⓞⓒⓚ, ⓉⓋ ⓠⓤⓘⓩ ⓅⓗⒹ, ⓑⓐⓖⓢ ⓕⓔⓦ ⓛⓨⓝⓧ.")
            addFontAnnotation(UnicodeFont.CIRCLED, 0, 37)
        },
        actual = buildFontTagText("\$cir\$Mr. Jock, TV quiz PhD, bags few lynx."),
    )

    @Test
    fun `Inverse circle font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD83C\uDD5C\uD83C\uDD61. \uD83C\uDD59\uD83C\uDD5E\uD83C\uDD52\uD83C\uDD5A, \uD83C\uDD63\uD83C\uDD65 \uD83C\uDD60\uD83C\uDD64\uD83C\uDD58\uD83C\uDD69 \uD83C\uDD5F\uD83C\uDD57\uD83C\uDD53, \uD83C\uDD51\uD83C\uDD50\uD83C\uDD56\uD83C\uDD62 \uD83C\uDD55\uD83C\uDD54\uD83C\uDD66 \uD83C\uDD5B\uD83C\uDD68\uD83C\uDD5D\uD83C\uDD67.")
            addFontAnnotation(UnicodeFont.CIRCLED_INVERSE, 0, 63)
        },
        actual = buildFontTagText("\$ciri\$Mr. Jock, TV quiz PhD, bags few lynx."),
    )

    @Test
    fun `Square font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD83C\uDD3C\uD83C\uDD41. \uD83C\uDD39\uD83C\uDD3E\uD83C\uDD32\uD83C\uDD3A, \uD83C\uDD43\uD83C\uDD45 \uD83C\uDD40\uD83C\uDD44\uD83C\uDD38\uD83C\uDD49 \uD83C\uDD3F\uD83C\uDD37\uD83C\uDD33, \uD83C\uDD31\uD83C\uDD30\uD83C\uDD36\uD83C\uDD42 \uD83C\uDD35\uD83C\uDD34\uD83C\uDD46 \uD83C\uDD3B\uD83C\uDD48\uD83C\uDD3D\uD83C\uDD47.")
            addFontAnnotation(UnicodeFont.SQUARED, 0, 63)
        },
        actual = buildFontTagText("\$sqr\$Mr. Jock, TV quiz PhD, bags few lynx."),
    )

    @Test
    fun `Inverse square font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD83C\uDD7C\uD83C\uDD81. \uD83C\uDD79\uD83C\uDD7E\uD83C\uDD72\uD83C\uDD7A, \uD83C\uDD83\uD83C\uDD85 \uD83C\uDD80\uD83C\uDD84\uD83C\uDD78\uD83C\uDD89 \uD83C\uDD7F\uD83C\uDD77\uD83C\uDD73, \uD83C\uDD71\uD83C\uDD70\uD83C\uDD76\uD83C\uDD82 \uD83C\uDD75\uD83C\uDD74\uD83C\uDD86 \uD83C\uDD7B\uD83C\uDD88\uD83C\uDD7D\uD83C\uDD87.")
            addFontAnnotation(UnicodeFont.SQUARED_INVERSE, 0, 63)
        },
        actual = buildFontTagText("\$sqri\$Mr. Jock, TV quiz PhD, bags few lynx."),
    )

    @Test
    fun `Parentheses font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD83C\uDD23⒣⒠ ⒬⒰⒤⒞⒦ ⒝⒭⒪⒲⒩ ⒡⒪⒳ ⒥⒰⒨⒫⒮ ⒪⒱⒠⒭ ⒯⒣⒠ ⒧⒜⒵⒴ ⒟⒪⒢")
            addFontAnnotation(UnicodeFont.PARENTHESES, 0, 44)
        },
        actual = buildFontTagText("\$par\$The quick brown fox jumps over the lazy dog"),
    )

    @Test
    fun `Fullwidth font`() = assertEquals(
        expected = buildAnnotatedString {
            append("Ｔｈｅ ｑｕｉｃｋ ｂｒｏｗｎ ｆｏｘ ｊｕｍｐｓ ｏｖｅｒ ｔｈｅ ｌａｚｙ ｄｏｇ")
            addFontAnnotation(UnicodeFont.FULLWIDTH, 0, 43)
        },
        actual = buildFontTagText("\$fw\$The quick brown fox jumps over the lazy dog"),
    )

    @Test
    fun `Smallcaps font`() = assertEquals(
        expected = buildAnnotatedString {
            append("Tʜᴇ ǫᴜɪᴄᴋ ʙʀᴏᴡɴ ꜰᴏx ᴊᴜᴍᴘs ᴏᴠᴇʀ ᴛʜᴇ ʟᴀᴢʏ ᴅᴏɢ")
            addFontAnnotation(UnicodeFont.SMALLCAPS, 0, 43)
        },
        actual = buildFontTagText("\$scap\$The quick brown fox jumps over the lazy dog"),
    )

    @Test
    fun `CJK font`() = assertEquals(
        expected = buildAnnotatedString {
            append("丿刄匸ズ问刄山ち 乚口ソ㠪 冊ﾘ 马エム ちㄗ卄エ刀乂 口于 디し刄尺丁乙.")
            addFontAnnotation(UnicodeFont.CJK, 0, 38)
        },
        actual = buildFontTagText("\$cjk\$Jackdaws love my big sphinx of quartz."),
    )

    @Test
    fun `Food emoji font`() = assertEquals(
        expected = buildAnnotatedString {
            append("\uD83C\uDF45\uD83C\uDF2D\uD83E\uDD5A \uD83C\uDF5F\uD83C\uDF68\uD83C\uDF77\uD83E\uDD5A \uD83C\uDF4C\uD83E\uDED2\uD83E\uDE7B\uD83C\uDF68\uD83C\uDF5C\uD83C\uDF47 \uD83C\uDF49\uD83C\uDF68\uD83E\uDD93\uD83C\uDF4E\uD83C\uDF59\uD83C\uDF69\uD83E\uDD6A \uD83E\uDED9\uD83C\uDF79\uD83C\uDF56\uD83C\uDF55 \uD83C\uDF6E\uD83C\uDF79\uD83C\uDF68\uD83C\uDF6A\uD83E\uDD5D\uD83C\uDF4B\uD83C\uDF60.")
            addFontAnnotation(UnicodeFont.FOOD, 0, 68)
        },
        actual = buildFontTagText("\$food\$The five boxing wizards jump quickly."),
    )
}