package nand.apps.chat.text.message

import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.font.FontWeight
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.uid.GroupUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.UserData
import nand.apps.chat.text.*
import nand.apps.chat.ui.chat.message.StylizedMessage
import kotlin.test.Test

private const val GROUP_UID = "098765432109876543210987654321098765432109876543210987654321FFFF"
private const val SELF_UID = "123456789012345678901234567890123456789012345678901234567890ABCD"
private const val PEER_UID = "ABCD123456789012345678901234567890123456789012345678901234567890"

class MentionTagTests {
    
    private val group = ChatGroupData(
        uid = GroupUid(GROUP_UID),
        selfUid = UserUid(SELF_UID),
        peers = mapOf(
            UserUid(SELF_UID) to UserData(
                uid = UserUid(SELF_UID),
                name = "NAND",
            ),
            UserUid(PEER_UID) to UserData(
                uid = UserUid(PEER_UID),
                name = "Johnny",
            )
        )
    )
    
    private val selfStyle = SpanStyle(
        fontWeight = FontWeight.Bold,
        color = TestChatColors.onMention,
        background = TestChatColors.mention,
    )
    
    private val otherStyle = SpanStyle(
        fontWeight = FontWeight.Bold,
        color = TestChatColors.primary,
    )

    @Test
    fun `Message with peer and self mentions`() = assertEquals(
        expected = StylizedMessage(
            text = buildAnnotatedString {
                append("@NAND @Johnny please help")
                addStringAnnotation(MENTION_TAG, SELF_UID, 0, 5)
                addStringAnnotation(MENTION_TAG, PEER_UID, 6, 13)
                addStyle(selfStyle, 0, 5)
                addStyle(otherStyle, 6, 13)
            }
        ),
        actual = buildStylizedMessage(
            text = "@$SELF_UID @$PEER_UID please help",
            isStylesEnabled = true,
            group = group,
        ),
    )

    @Test
    fun `Message with group mention`() {
        assertEquals(
            expected = StylizedMessage(
                text = buildAnnotatedString {
                    append("@here Hi I'm new here")
                    addStringAnnotation(MENTION_TAG, GROUP_UID, 0, 5)
                    addStyle(selfStyle, 0, 5)
                }
            ),
            actual = buildStylizedMessage(
                text = "@$GROUP_UID Hi I'm new here",
                isStylesEnabled = true,
                group = group,
            ),
        )
    }

    @Test
    fun `Input text with peer mention`() {
        assertEquals(
            expected = buildAnnotatedString {
                append("Hey @Johnny how's it going?")
                addStringAnnotation(MENTION_TAG, PEER_UID, 4, 11)
                addStyle(otherStyle, 4, 11)
            },
            actual = buildStylizedInputText(
                text = "Hey @$PEER_UID how's it going?",
                includeTagPreview = true,
                group = group,
            ),
        )
    }
}