package demo

import androidx.compose.foundation.layout.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.Window
import androidx.compose.ui.window.WindowPosition
import androidx.compose.ui.window.application
import androidx.compose.ui.window.rememberWindowState
import nand.apps.chat.model.avatar.ChatAvatar
import nand.apps.chat.model.avatar.ChatGroupAvatar
import nand.apps.chat.model.avatar.ChatUserAvatar
import nand.apps.chat.model.user.UserStatus
import nand.apps.chat.ui.avatar.ChatAvatarBox
import nand.apps.chat.ui.avatar.ChatAvatarSize
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.theme.ChatThemeSurface

fun main() = application {
    val windowState = rememberWindowState(
        width = 400.dp,
        height = 400.dp,
        position = WindowPosition(Alignment.Center),
    )
    Window(
        title = "Avatar Preview",
        onCloseRequest = ::exitApplication,
        state = windowState,
    ) {
        ChatThemeSurface {
            Column(
                verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
                modifier = Modifier.align(Alignment.Center),
            ) {
                AvatarPreviewRow(
                    avatar = ChatUserAvatar(username = "Some Guy"),
                )
                AvatarPreviewRow(
                    avatar = ChatGroupAvatar(groupTitle = "New Group"),
                )
            }
        }
    }
}

@Composable
private fun AvatarPreviewRow(
    avatar: ChatAvatar,
) = Row(
    horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
    verticalAlignment = Alignment.Bottom,
) {
    ChatAvatarSize.entries.forEach { size ->
        ChatAvatarBox(
            avatar = avatar,
            size = size,
            isOnline = size.ordinal >= 1,
            status = UserStatus.entries[(size.ordinal - 1).coerceAtLeast(0)],
            badgeText = size.ordinal.toString(),
        )
    }
}