package demo

import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.Window
import androidx.compose.ui.window.WindowPosition
import androidx.compose.ui.window.application
import androidx.compose.ui.window.rememberWindowState
import nand.apps.chat.di.initKoin
import nand.apps.chat.io.ChatFilePaths
import nand.apps.chat.io.DesktopFilePaths
import nand.apps.chat.ui.file.ChatFilePicker
import nand.apps.chat.ui.file.ChatFilePickerOptions
import nand.apps.chat.ui.theme.ChatThemeSurface
import nand.apps.chat.ui.window.windowIconPainter
import org.koin.dsl.module

fun main() = application {
    initKoin {
        val platformModules = module {
            single<ChatFilePaths> { DesktopFilePaths() }
        }
        modules(platformModules)
    }
    val windowState = rememberWindowState(
        width = 780.dp,
        height = 480.dp,
        position = WindowPosition(Alignment.Center),
    )
    Window(
        title = "File Picker",
        icon = windowIconPainter(false),
        onCloseRequest = ::exitApplication,
        state = windowState,
    ) {
        ChatThemeSurface {
            ChatFilePicker(
                options = ChatFilePickerOptions(),
                modifier = Modifier.fillMaxSize(),
                onConfirm = {},
                onCancel = { exitApplication() },
            )
        }
    }
}