import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.ExperimentalComposeUiApi
import androidx.compose.ui.window.*
import nand.apps.chat.di.initDesktopKoin
import nand.apps.chat.log.logCrash
import nand.apps.chat.ui.DesktopApp
import nand.apps.chat.ui.window.appCrashWindow
import org.koin.compose.KoinContext
import java.awt.event.WindowEvent
import kotlin.system.exitProcess

@OptIn(ExperimentalComposeUiApi::class)
fun main() {
    var lastError: Throwable? = null

    Thread.setDefaultUncaughtExceptionHandler { _, error ->
        logCrash(error)
        lastError = error
    }

    try {
        application(exitProcessOnExit = false) {
            CompositionLocalProvider(
                LocalWindowExceptionHandlerFactory provides WindowExceptionHandlerFactory { window ->
                    WindowExceptionHandler { error ->
                        logCrash(error)
                        lastError = error
                        window.dispatchEvent(WindowEvent(window, WindowEvent.WINDOW_CLOSING))
                    }
                }
            ) {
                // DI initialization
                val coroutineScope = rememberCoroutineScope()
                val koin = remember { initDesktopKoin(coroutineScope) }
                KoinContext(koin) {
                    DesktopApp()
                }
            }
        }
    } catch (error: Throwable) {
        logCrash(error)
        lastError = error
    }

    lastError?.let { error -> appCrashWindow(error) } ?: exitProcess(0)
}