package nand.apps.chat.audio

import nand.apps.soxr.SoxrNative

private const val SAMPLE_DBL = 32768.0

fun AudioFrame.copy(sampleRate: Int, channels: Int) =
    resample(sampleRate).setChannelCount(channels)

private fun AudioFrame.resample(sampleRate: Int): AudioFrame {
    if (this.sampleRate == sampleRate) return this

    val samples = SoxrNative.resample(
        samples = samples,
        channels = channels,
        inputRate = this.sampleRate,
        outputRate = sampleRate,
    ) ?: return this
    
    return AudioFrame(
        samples = samples,
        channels = channels,
        sampleRate = sampleRate,
    )
}

private fun AudioFrame.setChannelCount(channels: Int): AudioFrame {
    if (this.channels == channels) return this
    
    val newSamples = ShortArray((samples.size / this.channels) * channels)
    when {
        channels == 1 -> {
            var outIndex = 0
            for (s in samples.indices step this.channels) {
                var mixed = 0.0
                for (c in 0..<this.channels) {
                    mixed += samples[s + c] / SAMPLE_DBL
                }
                newSamples[outIndex++] = (mixed * SAMPLE_DBL).toInt().toShort()
            }
        }
        this.channels == 1 -> {
            var outIndex = 0
            for (sample in samples) {
                for (c in 0..<channels) {
                    newSamples[outIndex++] = sample
                }
            }
        }
        else -> error("Unsupported conversion")
    }
    return AudioFrame(
        samples = newSamples,
        channels = channels,
        sampleRate = sampleRate,
    )
}