package nand.apps.chat.audio

import kotlin.math.max

class DesktopAudioHandle(val frame: AudioFrame) : AudioHandle {

    override val timestamp get() = frame.getTime(pos)
    override val duration = frame.duration

    var groupId = -1L
    var time = -1L

    var pos = 0
    var srcVol = 1.0
    var dstVol = 1.0
    var volumeCalc: VolumeCalc? = null
    
    var prev: DesktopAudioHandle? = null
    var next: DesktopAudioHandle? = null

    val isFinished get() = pos >= frame.size

    var volume get() = volumeCalc?.volume ?: dstVol
        set(vol) {
            if (time == -1L && pos == 0) this.srcVol = vol
            this.dstVol = vol
        }

    override fun seekTo(timestamp: Double): Boolean {
        pos = frame.getPosition(timestamp)
        return true
    }

    fun increment() {
        pos++
    }

    fun finish() {
        pos = frame.size
        prev = null
    }
    
    fun getNextPlayable(): DesktopAudioHandle? {
        var ret = this
        while (ret.isFinished) {
            ret.prev = null
            ret = ret.next ?: break
        }
        return if (ret.isFinished) null else ret
    }

    fun getTime(): Double {
        return frame.getTime(pos)
    }

    fun setTime(time: Double) {
        pos = max(0, frame.getPosition(time))
        if (pos > 1) srcVol = 0.0
    }

    fun setVolumeCalculator(volumeCalc: VolumeCalc?) {
        this.volumeCalc = volumeCalc
        volume = volume
    }

    interface VolumeCalc {
        val volume: Double
    }
}