package nand.apps.chat.audio

import javazoom.jl.decoder.*
import nand.apps.chat.log.getLogger

private typealias SampleList = ArrayList<Short>
private val logger = getLogger("MpegDecoder")

// Decodes an entire MP3 file (in byte format) into PCM samples
// WARNING: Do not use this for large MP3 files as it is very slow and hogs memory
fun ByteArray.decodeMp3(): AudioFrame? {
    val samples = SampleList(DEFAULT_AUDIO_SAMPLE_RATE * 60)
    var channels = 0
    var sampleRate = 0
    try {
        inputStream().use { fileStream ->
            var output: SampleBuffer? = null
            val decoder = Decoder(null)
            val stream = Bitstream(fileStream)
            var frame = 0
            try {
                while (frame < Int.MAX_VALUE) {
                    try {
                        val header = stream.readFrame() ?: break

                        if (output == null) {
                            channels = if (header.mode() == Header.SINGLE_CHANNEL) 1 else 2
                            sampleRate = header.frequency()
                            output = SampleBuffer(sampleRate, channels)
                            decoder.setOutputBuffer(output)
                        }

                        val decoderOutput = decoder.decodeFrame(header, stream)
                        if (decoderOutput !== output) throw InternalError("Output buffers are different.")

                        samples.addAll(output.buffer.slice(0..<output.bufferLength))

                        stream.closeFrame()
                    } catch (e: Exception) {
                        throw JavaLayerException(e.localizedMessage, e)
                    }
                    frame++
                }
            } finally {
                output?.close()
            }
        }
    } catch (e: Exception) {
        logger.error("Failed to decode MP3 data", e)
    }

    if (samples.isEmpty()) return null

    return AudioFrame(
        channels = channels,
        samples = samples.toShortArray(),
        sampleRate = sampleRate,
    )
}