package nand.apps.chat.audio.opus

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import nand.apps.chat.audio.AudioPlayer
import nand.apps.chat.audio.DesktopAudioHandle
import nand.apps.chat.audio.readToFrame
import nand.apps.chat.log.getLogger
import nand.apps.opus.OpusFileReader

private const val BUFFER_SIZE = 960 * 2
private const val HANDLE_BUFFER_CAPACITY = 5

class OpusAudioStreamer(
    private val audioPlayer: AudioPlayer,
    private val coroutineScope: CoroutineScope,
) {
    
    private val logger = getLogger("OpusAudioStreamer")
    
    fun streamFile(path: String, startTime: Double): OpusAudioHandle {
        val file = OpusFileReader(path)
        if (startTime > 0) file.seekTo(startTime)
        val job = coroutineScope.launch {
            val buffer = ShortArray(BUFFER_SIZE)
            val handleBuffer = mutableListOf<DesktopAudioHandle>()
            var lastHandle: DesktopAudioHandle? = null
            while (file.isOpen) {
                val frame = file.readToFrame(buffer) ?: run {
                    file.close()
                    return@launch
                }
                val handle = audioPlayer.queueFrame(
                    frame = frame,
                    last = lastHandle,
                ) as? DesktopAudioHandle ?: run {
                    logger.error("Error queueing audio frame")
                    return@launch
                }
                lastHandle = handle
                handleBuffer += handle
                handleBuffer.delayUntilVacant(HANDLE_BUFFER_CAPACITY)
            }
        }
        return OpusAudioHandle(
            file = file,
            job = job,
            duration = file.duration,
        )
    }

    private suspend fun MutableList<DesktopAudioHandle>.delayUntilVacant(
        capacity: Int = 1,
        delayMs: Long = 20L,
    ) {
        assert(capacity > 0)
        while (size >= capacity) {
            removeIf { it.isFinished }
            if (size >= capacity) delay(delayMs)
        }
    }
}