package nand.apps.chat.di

import kotlinx.coroutines.CoroutineScope
import nand.apps.chat.audio.AudioPlayer
import nand.apps.chat.audio.AudioRecorder
import nand.apps.chat.audio.DesktopAudioPlayer
import nand.apps.chat.audio.DesktopAudioRecorder
import nand.apps.chat.camera.CameraLauncher
import nand.apps.chat.camera.DesktopCameraLauncher
import nand.apps.chat.dev.DevCommandLogger
import nand.apps.chat.dev.DevConsoleCommandLogger
import nand.apps.chat.dev.DevConsoleRepo
import nand.apps.chat.image.ImagePixelReader
import nand.apps.chat.image.SkiaPixelReader
import nand.apps.chat.io.ChatFileBrowser
import nand.apps.chat.io.ChatFilePaths
import nand.apps.chat.io.DesktopFilePaths
import nand.apps.chat.net.DesktopUriLauncher
import nand.apps.chat.net.UriLauncher
import nand.apps.chat.platform.AppStateProvider
import nand.apps.chat.platform.DesktopAppStateProvider
import nand.apps.chat.profile.DesktopCredentialsStore
import nand.apps.chat.profile.DesktopFileBrowser
import nand.apps.chat.profile.ProfileCredentialsStore
import nand.apps.chat.share.ContentShare
import nand.apps.chat.share.DesktopContentShare
import nand.apps.chat.ui.dev.DevConsoleViewModel
import nand.apps.chat.ui.notification.DesktopNotificationViewModel
import org.koin.core.Koin
import org.koin.core.module.dsl.factoryOf
import org.koin.core.module.dsl.singleOf
import org.koin.dsl.module

fun initDesktopKoin(coroutineScope: CoroutineScope): Koin {
    loadNativeLibraries()
    return initKoin {
        val platformModules = module {
            single<ProfileCredentialsStore> { DesktopCredentialsStore() }
            single<ChatFileBrowser> { DesktopFileBrowser() }
            single<AppStateProvider> { DesktopAppStateProvider() }
            single<UriLauncher> { DesktopUriLauncher() }
            single<ContentShare> { DesktopContentShare() }
            single<ChatFilePaths> { DesktopFilePaths() }
            single<ImagePixelReader> { SkiaPixelReader() }
            single<CameraLauncher> { DesktopCameraLauncher() }
            single<AudioPlayer> { DesktopAudioPlayer(coroutineScope) }
            single<AudioRecorder> { DesktopAudioRecorder() }
            single<DevCommandLogger> { DevConsoleCommandLogger(get()) }
            singleOf(::DevConsoleRepo)
            factoryOf(::DesktopNotificationViewModel)
            factoryOf(::DevConsoleViewModel)
        }
        modules(platformModules)
    }.koin
}