package nand.apps.chat.di

import nand.apps.chat.BuildConfig.MOCK_ENGINE
import nand.apps.chat.platform.ChatPlatform
import nand.apps.chat.platform.getPlatform
import java.io.File

private const val DEV_RESOURCES_DIR = "src/desktopMain/appResources"

private val desktopTarget: String by lazy {
    val hostOs = System.getProperty("os.name")
    val hostArch = System.getProperty("os.arch")
    val osName = when {
        hostOs.startsWith("Mac OS") -> "macos"
        hostOs.startsWith("Linux") -> "linux"
        hostOs.startsWith("Windows") -> "windows"
        else -> error("Host OS $hostOs is not supported")
    }
    val osArch = when (hostArch) {
        "aarch64" -> "arm64"
        "x86_64",
        "amd64" -> "x64"
        else -> error("Host architecture $hostArch is not supported")
    }
    "$osName-$osArch"
}

fun loadNativeLibraries() {
    if (getPlatform() == ChatPlatform.WINDOWS) {
        // MinGW required dependency libraries
        loadLibraryResource("winpthread-1")
        loadLibraryResource("gcc_s_seh-1")
        loadLibraryResource("gomp-1")
    }
    if (!MOCK_ENGINE) {
        loadLibraryResource("toxjni")
    }
    loadLibraryResource("gifjni")
    loadLibraryResource("opusjni")
    loadLibraryResource("soxrjni")
}

@Suppress("UnsafeDynamicallyLoadedCode")
fun loadLibraryResource(libraryName: String) {
    val platformLibraryName = System.mapLibraryName(libraryName)
    val resourceDir = System.getProperty("compose.application.resources.dir") ?: ("$DEV_RESOURCES_DIR/$desktopTarget")
    val libraryDir = File(resourceDir, "lib")
    var libraryFile = File(libraryDir, platformLibraryName)
    if (getPlatform() == ChatPlatform.WINDOWS && !libraryFile.exists()) {
        libraryFile = File(libraryDir, "lib$platformLibraryName")
    }
    System.load(libraryFile.absolutePath)
}