package nand.apps.chat.image

import androidx.compose.ui.graphics.ImageBitmap
import androidx.compose.ui.graphics.toComposeImageBitmap
import org.jetbrains.skia.ColorAlphaType
import org.jetbrains.skia.ColorType
import org.jetbrains.skia.Image
import org.jetbrains.skia.ImageInfo
import java.awt.image.BufferedImage
import java.awt.image.DataBufferInt
import java.io.ByteArrayOutputStream
import javax.imageio.ImageIO
import javax.imageio.ImageWriteParam

actual fun SimpleBitmap.toComposeBitmap(): ImageBitmap {
    return Image.makeRaster(
        imageInfo = ImageInfo(
            width = width,
            height = height,
            colorType = ColorType.RGBA_8888,
            alphaType = ColorAlphaType.PREMUL,
        ),
        bytes = pixels,
        rowBytes = width * 4,
    ).toComposeImageBitmap()
}

actual fun SimpleBitmap.toPngByteArray() = toImageBuffer("png")

actual fun SimpleBitmap.toJpegByteArray(quality: Float) = toImageBuffer("jpg", quality)

private fun SimpleBitmap.toImageBuffer(
    formatName: String,
    quality: Float = 0f,
): ByteArray {
    var p = 0
    val colors = IntArray(pixels.size / 4) {
        val red = pixels[p++].toInt() and 0xFF
        val green = pixels[p++].toInt() and 0xFF
        val blue = pixels[p++].toInt() and 0xFF
        val alpha = pixels[p++].toInt() and 0xFF
        (alpha shl 24) or (red shl 16) or (green shl 8) or blue
    }
    val image = BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB)
    val pixels = (image.raster.dataBuffer as DataBufferInt).data
    colors.copyInto(pixels)
    return ByteArrayOutputStream().use { stream ->
        try {
            val imageWriter = ImageIO.getImageWritersByFormatName(formatName).next()
            if (quality > 0f) {
                with (imageWriter.defaultWriteParam) {
                    compressionMode = ImageWriteParam.MODE_EXPLICIT
                    compressionQuality = quality
                }
            }
            ImageIO.write(image, formatName, stream)
            stream.toByteArray()
        } catch (ignore: Exception) {
            ByteArray(0)
        }
    }
}