package nand.apps.chat.image

import coil3.ImageLoader
import coil3.Uri
import coil3.annotation.InternalCoilApi
import coil3.decode.DataSource
import coil3.decode.ImageSource
import coil3.fetch.FetchResult
import coil3.fetch.Fetcher
import coil3.fetch.SourceFetchResult
import coil3.request.Options
import coil3.util.MimeTypeMap
import nand.apps.chat.io.extension
import nand.apps.chat.io.toFilePath
import okio.Path

@OptIn(InternalCoilApi::class)
class WindowsFileFetcher(
    private val path: Path,
    private val options: Options,
) : Fetcher {

    override suspend fun fetch(): FetchResult {
        return SourceFetchResult(
            source = ImageSource(path, options.fileSystem),
            mimeType = MimeTypeMap.getMimeTypeFromExtension(path.extension),
            dataSource = DataSource.DISK,
        )
    }

    class Factory : Fetcher.Factory<Uri> {
        override fun create(data: Uri, options: Options, imageLoader: ImageLoader): Fetcher? {
            return data.toFilePath()?.let { WindowsFileFetcher(it, options) }
        }
    }
}