package nand.apps.chat.io

import dev.dirs.UserDirectories
import nand.apps.chat.BuildConfig.APP_DIR_NAME
import nand.apps.chat.log.getLogger
import nand.apps.chat.platform.ChatPlatform
import nand.apps.chat.platform.getPlatform
import net.harawata.appdirs.AppDirsFactory
import okio.FileSystem
import okio.Path
import okio.Path.Companion.toOkioPath
import okio.Path.Companion.toPath
import java.io.File

private const val OLD_DIR_NAME = "seers-bank"

class DesktopFilePaths : ChatFilePaths {
    
    private val appDirs = AppDirsFactory.getInstance()
    private val userDirs = UserDirectories.get()
    private val logger = getLogger("DesktopFilePaths")

    override val roots = File.listRoots().mapNotNull { it?.toOkioPath() }.toSet()
    override val userDataPath = appDirs.getUserDataDir(APP_DIR_NAME, null, null).toPath()
    override val cachePath = appDirs.getUserCacheDir(APP_DIR_NAME, null, null).toPath()
    override val configPath = getConfigPath(APP_DIR_NAME)
    override val toxPath = when (getPlatform()) {
        ChatPlatform.MAC -> appDirs.getUserDataDir("Tox", null, null).toPath()
        ChatPlatform.WINDOWS -> appDirs.getUserDataDir("tox", null, null, true).toPath()
        ChatPlatform.LINUX -> appDirs.getUserConfigDir("tox", null, null).toPath()
        else -> userDataPath.resolve("tox")
    }
    override val homePath = userDirs.homeDir.toPath()
    override val documentPath = userDirs.documentDir.toPath()
    override val downloadPath = userDirs.downloadDir.toPath()
    override val picturesPath = userDirs.pictureDir.toPath()
    override val videosPath = userDirs.videoDir.toPath()
    override val musicPath = userDirs.audioDir.toPath()
    
    override fun init() {
        transferOldDirectories(
            userDataPath = appDirs.getUserDataDir(OLD_DIR_NAME, null, null).toPath(),
            cachePath = appDirs.getUserCacheDir(OLD_DIR_NAME, null, null).toPath(),
            configPath = getConfigPath(OLD_DIR_NAME),
        )
    }
    
    private fun transferOldDirectories(
        userDataPath: Path,
        cachePath: Path,
        configPath: Path,
    ) {
        transferDirectory(userDataPath, this.userDataPath)
        transferDirectory(cachePath, this.cachePath)
        transferDirectory(configPath, this.configPath)
    }
    
    private fun transferDirectory(oldDir: Path, newDir: Path) {
        val fileSystem = FileSystem.SYSTEM
        if (fileSystem.exists(oldDir) && !fileSystem.exists(newDir)) {
            var successful = true
            logger.debug("Migrating from $oldDir to $newDir")
            fileSystem.createDirectory(newDir)
            fileSystem.listRecursively(oldDir).forEach { oldPath ->
                val newPath = newDir.resolve(oldPath.relativeTo(oldDir))
                if (fileSystem.metadataOrNull(oldPath)?.isDirectory == true) {
                    fileSystem.createDirectory(newPath)
                } else {
                    fileSystem.copy(oldPath, newPath)
                    if (fileSystem.exists(newPath)) {
                        fileSystem.delete(oldPath)
                    } else {
                        successful = false
                    }
                }
            }
            if (successful) {
                fileSystem.deleteRecursively(oldDir)
            }
        }
    }

    private fun getConfigPath(dirName: String) = when (getPlatform()) {
        ChatPlatform.MAC -> appDirs.getUserDataDir(dirName, null, null).toPath()
        else -> appDirs.getUserConfigDir(dirName, null, null).toPath()
    }
}