package nand.apps.chat.log

import kotlinx.datetime.LocalDateTime
import kotlinx.datetime.format
import kotlinx.datetime.format.Padding
import kotlinx.datetime.format.char
import nand.apps.chat.time.currentTimeMillis
import nand.apps.chat.time.getLocalInstant

private val TIME_FORMAT = LocalDateTime.Format {
    amPmHour(Padding.NONE)
    char(':')
    minute()
    char(':')
    second()
    char('.')
    secondFraction(minLength = 3, maxLength = 3)
    char(' ')
    amPmMarker("AM", "PM")
}

class JvmAppLogger(private val tag: String) : AppLogger {

    override fun verbose(message: String) {
        println(formatMessage(message))
    }
    
    override fun debug(message: String) {
        println(formatMessage(message))
    }

    override fun warn(message: String, exception: Exception?) {
        println(formatMessage("WARN: $message"))
        exception?.printStackTrace(System.out)
    }

    override fun error(message: String, exception: Exception?) {
        System.err.println(formatMessage(message))
        exception?.printStackTrace()
    }
    
    private fun formatMessage(message: String) =
        "[${getLocalInstant(currentTimeMillis()).format(TIME_FORMAT)}] $tag – $message"
}

actual fun getAppLogger(tag: String): AppLogger = JvmAppLogger(tag)