package nand.apps.chat.net

import nand.apps.chat.log.getLogger
import nand.apps.chat.platform.ChatPlatform
import nand.apps.chat.platform.getPlatform
import java.awt.Desktop
import java.net.URI

class DesktopUriLauncher : UriLauncher {
    
    private val logger = getLogger("UriLauncher")

    override fun launchUri(uri: ChatUri) {
        val desktop = when {
            Desktop.isDesktopSupported() -> Desktop.getDesktop().takeIf { it.isSupported(Desktop.Action.BROWSE) }
            else -> null
        }
        try {
            val parsedUri = URI.create(uri.value)
            when {
                desktop != null -> desktop.browse(parsedUri)
                getPlatform() == ChatPlatform.LINUX -> fallbackLaunchUri(parsedUri)
            }
        } catch (e: Exception) {
            logger.error("Failed to launch URI: $uri", e)
        }
    }

    // Fallback to OS specific launch command (only supports Unix-based systems)
    // XXX - I copied this from some very old code used for launching URLs specifically; only tested on Kubuntu
    private fun fallbackLaunchUri(uri: URI) {
        when (uri.scheme) {
            "http",
            "https" -> launchUri(
                uri = uri,
                command = findCommand(WEB_BROWSER_COMMANDS) ?: error("Could not find web browser"),
            )
            "file",
            null -> launchUri(
                uri = uri,
                command = findCommand(FILE_BROWSER_COMMANDS) ?: error("Could not find file browser"),
            )
            else -> error("Unsupported URI: $uri")
        }
    }
}

private val WEB_BROWSER_COMMANDS = arrayOf("firefox", "chromium", "opera", "brave", "vivaldi", "konqueror")
private val FILE_BROWSER_COMMANDS = arrayOf("dolphin", "caja", "nemo", "nautilus", "spacefm")

private fun findCommand(commands: Array<String>) = commands.find {
    Runtime.getRuntime().exec(arrayOf("which", it)).waitFor() == 0
}

private fun launchUri(uri: URI, command: String) =
    Runtime.getRuntime().exec(arrayOf(command, uri.toString()))