package nand.apps.chat.ui.dev

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Terminal
import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.rememberVectorPainter
import androidx.compose.ui.input.key.Key
import androidx.compose.ui.input.key.KeyEventType
import androidx.compose.ui.input.key.key
import androidx.compose.ui.input.key.type
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.unit.Density
import androidx.compose.ui.unit.DpSize
import androidx.compose.ui.window.Window
import androidx.compose.ui.window.WindowPosition
import androidx.compose.ui.window.rememberWindowState
import com.hoc081098.kmp.viewmodel.koin.compose.koinKmpViewModel
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.util.copy
import org.koin.compose.getKoin

@Composable
fun DevConsoleWindow(
    isVisible: Boolean,
    onClose: () -> Unit,
    windowSize: DpSize,
    viewModel: DevConsoleViewModel = koinKmpViewModel(),
) {
    val windowState = rememberWindowState(
        size = windowSize,
        position = WindowPosition(Alignment.Center),
    )
    val windowIcon = remember {
        Icons.Default.Terminal.copy(tintColor = Color.White)
    }
    val settingsRepo: SettingsRepo by getKoin().inject()
    val defaultDensity = LocalDensity.current
    Window(
        title = "$APP_NAME - Developer Console",
        icon = rememberVectorPainter(windowIcon),
        state = windowState,
        undecorated = true,
        transparent = true,
        visible = isVisible,
        onCloseRequest = onClose,
        onPreviewKeyEvent = { event ->
            if (event.type == KeyEventType.KeyDown && event.key == Key.Escape) {
                onClose()
                true
            } else {
                false
            }
        }
    ) {
        CompositionLocalProvider(
            LocalDensity provides Density(defaultDensity.density * settingsRepo.settings.generalSettings.densityScale)
        ) {
            DevConsoleScreen(
                onClose = onClose,
                viewModel = viewModel,
                windowScope = this,
                modifier = Modifier
                    .clip(ChatTheme.shapes.small)
                    .background(Color.Black.copy(alpha = 0.75f))
                    .fillMaxSize()
            )
        }
    }
}