package nand.apps.chat.ui.dispatcher

import androidx.compose.runtime.*

val LocalBackPressDispatcher = staticCompositionLocalOf<BackPressDispatcher> {
    error("No back dispatcher is set")
}

abstract class BackPressCallback(isEnabled: Boolean) : EventCallback(isEnabled) {
    
    abstract fun onBackPressed(): Boolean
}

class BackPressDispatcher : EventDispatcher<BackPressCallback>() {
    
    operator fun invoke() {
        callbacks.forEach { if (it.onBackPressed()) return }
    }
}

@Composable
fun DesktopBackHandler(isEnabled: Boolean = true, onBack: () -> Unit) {
    val currentOnBack by rememberUpdatedState(onBack)
    val backCallback = remember {
        object : BackPressCallback(isEnabled) {
            override fun onBackPressed(): Boolean {
                return if (this.isEnabled) {
                    currentOnBack()
                    true
                } else {
                    false
                }
            }
        }
    }
    DisposableEventHandler(
        eventCallback = backCallback,
        eventDispatcher = LocalBackPressDispatcher.current,
        isEnabled = isEnabled,
    )
}