package nand.apps.chat.ui.dispatcher

import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.ui.input.key.KeyEvent
import nand.apps.chat.model.settings.shortcut.ShortcutAction
import nand.apps.chat.ui.shortcut.KeyEventDispatcher
import nand.apps.chat.ui.shortcut.LocalKeyEventDispatcher
import nand.apps.chat.ui.shortcut.LocalShortcutDispatcher
import nand.apps.chat.ui.shortcut.ShortcutDispatcher

class DesktopKeyEventDispatcher {
    
    private val backPressDispatcher = BackPressDispatcher()
    private val shortcutDispatcher = ShortcutDispatcher()
    private val keyEventDispatcher = KeyEventDispatcher()
    
    fun navigateBack() = backPressDispatcher()
    
    fun onShortcutAction(action: ShortcutAction) = shortcutDispatcher(action)
    
    fun onKeyEvent(event: KeyEvent) = keyEventDispatcher(event)

    @Composable
    fun CompositionLocalProvider(content: @Composable () -> Unit) = CompositionLocalProvider(
        LocalBackPressDispatcher provides backPressDispatcher,
        LocalShortcutDispatcher provides shortcutDispatcher,
        LocalKeyEventDispatcher provides keyEventDispatcher,
        content = content,
    )
}