package nand.apps.chat.ui.event

import androidx.compose.runtime.*
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.engine.ChatEngine
import nand.apps.chat.event.ChatEventHandler
import nand.apps.chat.model.event.*
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.DEFAULT_WINDOW_TITLE
import org.koin.compose.getKoin

@Composable
fun DesktopEventHandler(
    onAppLoaded: () -> Unit,
    onAppFocused: (RequestAppFocusEvent) -> Unit,
    onWindowTitle: (String) -> Unit,
) {
    val coroutineScope = rememberCoroutineScope()
    val chatEngine: ChatEngine by getKoin().inject()
    val eventHandler: ChatEventHandler by getKoin().inject()
    LaunchedEffect(Unit) {
        if (eventHandler.isAppLoaded) {
            // In case we missed the AppLoadedEvent before collecting
            onAppLoaded()
        }
        eventHandler.onEvent.collect { event ->
            when (event) {
                is AppLoadedEvent -> {
                    onAppLoaded()
                }
                is RequestAppFocusEvent -> {
                    onAppFocused(event)
                }
                is LoginCompleteEvent -> {
                    chatEngine.start(coroutineScope)
                }
                is LogoutEvent -> {
                    chatEngine.stop()
                }
                is OpenConversationEvent -> {
                    val contactName = when (val contact = event.contact) {
                        is UserData -> contact.displayName
                        is ChatGroupData -> contact.title
                        else -> null
                    }
                    onWindowTitle(contactName?.let { "$contactName - $APP_NAME" } ?: DEFAULT_WINDOW_TITLE)
                }
                is ExitConversationEvent -> {
                    onWindowTitle(DEFAULT_WINDOW_TITLE)
                }
                else -> Unit
            }
        }
    }
}