@file:OptIn(ExperimentalComposeUiApi::class)

package nand.apps.chat.ui.modifier

import androidx.compose.runtime.Composable
import androidx.compose.ui.ExperimentalComposeUiApi
import androidx.compose.ui.DragData as DesktopDragData
import androidx.compose.ui.ExternalDragValue as DesktopDragValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.onExternalDrag as onDesktopDrag

@Composable
actual fun Modifier.onExternalDrag(
    enabled: Boolean,
    onDragStart: (ExternalDragValue) -> Unit,
    onDrag: (ExternalDragValue) -> Unit,
    onDragExit: () -> Unit,
    onDrop: (ExternalDragValue) -> Unit
) = onDesktopDrag(
    enabled = enabled,
    onDragStart = { drag -> onDragStart(drag.convert()) },
    onDrag = { drag -> onDrag(drag.convert()) },
    onDragExit = onDragExit,
    onDrop = { drag -> onDrop(drag.convert()) },
)


private fun DesktopDragValue.convert() = ExternalDragValue(
    position = dragPosition,
    data = dragData.convert(),
)

private fun DesktopDragData.convert(): DragData = when (val data = this) {
    is DesktopDragData.Text -> DragTextImpl(data)
    is DesktopDragData.Image -> DragImageImpl(data)
    is DesktopDragData.FilesList -> DragFilesListImpl(data)
    else -> error("Unsupported data type")
}

private class DragTextImpl(
    private val data: DesktopDragData.Text,
) : DragData.Text {

    override val bestMimeType get() = data.bestMimeType

    override fun readText() = data.readText()
}

private class DragImageImpl(
    private val data: DesktopDragData.Image,
) : DragData.Image {

    override fun readImage() = data.readImage()
}

private class DragFilesListImpl(
    private val data: DesktopDragData.FilesList,
) : DragData.FilesList {

    override fun readFiles() = data.readFiles()
}

