package nand.apps.chat.ui.notification

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import nand.apps.chat.model.event.*
import nand.apps.chat.model.notification.ChatNotification
import nand.apps.chat.model.notification.MessageNotification
import nand.apps.chat.model.uid.ContactUid
import nand.apps.chat.repo.NotificationRepo
import nand.apps.chat.repo.CallRepo
import nand.apps.chat.repo.ContactRepo
import nand.apps.chat.event.ChatEventHandler
import nand.apps.chat.ui.util.collect
import nand.apps.chat.util.fifoSharedFlow

class DesktopNotificationViewModel(
    private val notificationRepo: NotificationRepo,
    private val eventHandler: ChatEventHandler,
    private val contactRepo: ContactRepo,
    callRepo: CallRepo,
) : ViewModel() {
    
    val state = MutableStateFlow(DesktopNotificationState())
    val actionData = fifoSharedFlow<DesktopNotificationAction>()
    
    init {
        notificationRepo.start(viewModelScope)
        collect(notificationRepo.notificationObserver) { notifications ->
            val showNotifications = notifications.keys.filter { it !in state.value.notifications }
            val dismissNotifications = state.value.notifications.keys.filter { it !in notifications }
            state.emit(state.value.copy(notifications = notifications))
            if (showNotifications.isNotEmpty() || dismissNotifications.isNotEmpty()) {
                actionData.emit(NotificationUpdate(showNotifications, dismissNotifications))
            }
        }
        collect(contactRepo.friendObserver) { friends ->
            state.emit(state.value.copy(friends = friends))
        }
        collect(contactRepo.groupObserver) { groups ->
            state.emit(state.value.copy(groups = groups))
        }
        collect(callRepo.callObserver) { calls ->
            state.emit(state.value.copy(calls = calls))
        }
    }
    
    fun onNotificationClick(notification: ChatNotification) {
        when (notification) {
            is MessageNotification -> {
                openConversation(notification.message.conversationUid)
            }
            else -> return
        }
        notificationRepo.dismiss(notification.group)
        eventHandler += RequestAppFocusEvent()
    }
    
    fun onNotificationDismiss(notification: ChatNotification) {
        notificationRepo.dismiss(notification.group)
    }
    
    private fun openConversation(uid: ContactUid) {
        contactRepo.getContact(uid)?.let { eventHandler += OpenConversationEvent(it) }
    }
}