package nand.apps.chat.ui.notification

import androidx.compose.runtime.*
import nand.apps.chat.repo.ChatMessageRepo
import nand.apps.chat.repo.ContactRepo
import nand.apps.chat.ui.window.WindowFocusListener
import org.koin.compose.getKoin

@Composable
fun UnreadMessageTracker(
    onUnreadMessage: () -> Unit,
    onReadMessages: () -> Unit,
) {
    val messageRepo: ChatMessageRepo by getKoin().inject()
    val contactRepo: ContactRepo by getKoin().inject()
    var isWindowFocused by remember { mutableStateOf(false) }
    LaunchedEffect(Unit) {
        WindowFocusListener { isFocused ->
            isWindowFocused = isFocused
            if (isFocused) onReadMessages()
        }
        messageRepo.onNewMessage.collect {
            if (!isWindowFocused) {
                contactRepo.getContact(it.conversationUid)?.let { contact ->
                    if (!contact.isMuted) {
                        onUnreadMessage()
                    }
                }
            }
        }
    }
}