package nand.apps.chat.ui.window

import androidx.compose.runtime.*
import androidx.compose.ui.awt.ComposeWindow
import androidx.compose.ui.window.WindowState
import nand.apps.chat.event.ChatEventHandler
import nand.apps.chat.model.event.*
import org.koin.compose.getKoin

@Composable
fun WindowFocusEventHandler(
    window: ComposeWindow?,
    windowState: WindowState,
    isWindowVisible: Boolean,
) {
    val eventHandler: ChatEventHandler by getKoin().inject()
    var isWindowActive by remember { mutableStateOf(window?.isActive == true) }
    var isWindowFocused by remember { mutableStateOf<Boolean?>(null) }
    DisposableEffect(Unit) {
        val listener = WindowFocusListener { isWindowActive = it }
        onDispose { listener.dispose() }
    }
    LaunchedEffect(windowState.isMinimized, isWindowVisible, isWindowActive) {
        val isFocused = isWindowVisible && isWindowActive && !windowState.isMinimized
        if (isWindowFocused != null && isFocused != isWindowFocused) {
            eventHandler += AppFocusChangedEvent(isFocused)
        }
        isWindowFocused = isFocused
    }
}