package nand.apps.chat.ui.window

import androidx.compose.runtime.Composable
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.unit.DpRect
import androidx.compose.ui.unit.IntRect
import nand.apps.chat.ui.util.toDp
import java.awt.GraphicsDevice
import java.awt.Toolkit

fun GraphicsDevice.getMaxWindowBounds(): IntRect {
    val config = defaultConfiguration
    val screenBounds = config.bounds
    val insets = Toolkit.getDefaultToolkit().getScreenInsets(config)
    return IntRect(
        left = screenBounds.x + insets.left,
        top = screenBounds.y + insets.top,
        right = screenBounds.x + screenBounds.width - insets.right,
        bottom = screenBounds.y + screenBounds.height - insets.bottom,
    )
}

@Composable
fun GraphicsDevice.getMaxWindowBoundsDp(): DpRect {
    val density = LocalDensity.current
    return getMaxWindowBounds().run {
        DpRect(
            left = left.toDp(density),
            top = top.toDp(density),
            right = right.toDp(density),
            bottom = bottom.toDp(density),
        )
    }
}