import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.window.ComposeUIViewController
import nand.apps.chat.camera.CameraLauncher
import nand.apps.chat.camera.IOSCameraLauncher
import nand.apps.chat.di.initKoin
import nand.apps.chat.engine.ChatEngine
import nand.apps.chat.image.ImagePixelReader
import nand.apps.chat.image.SkiaPixelReader
import nand.apps.chat.io.ChatFilePaths
import nand.apps.chat.io.IOSFilePaths
import nand.apps.chat.net.IOSUriLauncher
import nand.apps.chat.net.UriLauncher
import nand.apps.chat.ui.app.App
import nand.apps.chat.ui.event.AppEventHandler
import org.koin.compose.KoinContext
import org.koin.dsl.module

fun MainViewController() = ComposeUIViewController {
    val koin = remember {
        initKoin {
            val platformModules = module {
                single<UriLauncher> { IOSUriLauncher() }
                single<ChatFilePaths> { IOSFilePaths() }
                single<ImagePixelReader> { SkiaPixelReader() }
                single<CameraLauncher> { IOSCameraLauncher() }
            }
            modules(platformModules)
        }.koin
    }
    
    KoinContext(koin) {
        val chatEngine: ChatEngine by koin.inject()
        var isLoginComplete by rememberSaveable { mutableStateOf(false) }
        val coroutineScope = rememberCoroutineScope()
        LaunchedEffect(isLoginComplete) {
            if (isLoginComplete) {
                chatEngine.start(coroutineScope)
            }
        }
        App()
        AppEventHandler(
            onLoginState = { isLoginComplete = it },
        )
    }
}