# Tox Compatibility

Seers Lodge primarily uses [Tox](https://tox.chat/).
This is a decentralized peer-to-peer messaging protocol that runs on a series of [nodes](https://nodes.tox.chat/).
There are several different [clients](https://tox.chat/clients.html) that use the Tox protocol, with varying degrees of
feature support and inter-compatibility.

Since Seers Lodge uses the same network protocol and nodes that these other chat clients use, they're technically compatible.
However, it's important to note that **Seers Lodge is not a "Tox-first" client**. Meaning that we* make no guarantees
when it comes to compatibility with other Tox clients (besides what's enforced by [toxcore](https://github.com/TokTok/c-toxcore)).
This is why the app isn't called "kTox" or "jTox" or "Toxbox" or "TixTox" or "Gooblox".

### What's not compatible?

At the moment the largest incompatibilities are [chat styles and effects](chat.md). All of these are completely unsupported
in every other Tox client since they're custom-made for Seers Lodge. The only text style that's semi-compatible with other
Tox clients is Markdown support, which isn't actually part of the Tox protocol. Tox only transfers the raw text in
your messages, so all these styles and effects are cosmetically applied using that raw text.

Besides the commands above, Tox clients typically support the `/me` command, which functions similar to chat programs like Steam and Discord.

Other incompatibile features include:
- Message editing and removal
- Reaction emojis
- Networked client versions

These features utilize "custom packets", which allow for open-ended feature support using the Tox protocol. Seers Lodge has a specific JSON-based format for custom packets which other Tox clients are welcome to use if they want. The app does not discriminate which contacts to send custom packets to. This is intentional to allow other chat clients to support these packets regardless of their broadcasted client version (lest we run into a [Mozilla/5.0](https://stackoverflow.com/questions/1114254/why-do-all-browsers-user-agents-start-with-mozilla) sort of scenario).

### Why does Seers Lodge use Tox then?

This project originally started out as a completely new chat protocol and client which heavily utilized a library called
[Ktor](https://ktor.io/). I quickly realized that writing a sophisticated chat architecture practically from scratch was
a complete nightmare and would take 10x as long compared to using an existing protocol and architecture.

Tox also provides some really neat functionality that other messaging protocols do not, such as direct UDP file transfers.
You know, when you want to send a 1.4 GB episode of _Isekai Cheat Magician_ to your friend without needing to invite
them to your super secret private tracker.

Oh yeah, and the privacy stuff is cool too I guess.

### Can I fork this and make my own Tox client?

Well yeah, of course. It's open sores. Jah bless.

I'm also open to creating a "Tox compliant" flavor of Seers Lodge if that doesn't end up being a giant pain. This could
be a separate compilation target that lives in the same repo, or maybe just a setting that disables all the non-Tox stuff.

### Can Seers Lodge handle other messaging protocols?

What a smart, insightful question - I'm glad you asked. 🤓

The "app layer", which is written in Kotlin, is designed in such a way that it uses an abstract "chat engine" interface
which could potentially be swapped out with a non-Tox chat engine. This is intentional to better support future chat
protocols and innovations in the event that Tox is no longer maintained or "up to snuff". It also makes the app better
suited for forking and modifications akin to a "white label" application.

There's an example of this in the [MockChatEngine](../composeApp/src/commonMain/kotlin/nand/apps/chat/engine/mock/MockChatEngine.kt) class.
This engine is used for local development and testing and does not utilize any Tox functionality.

_______________________________________________________________________________________________

```
* "We" refers to the primary and sole developer Cale Stockman (aka NAND) who uses the word "we"
  when writing documentation to make his stupid meme app seem more important than it really is.
```

